/*
 * DefaultScenarioGraph.java
 *
 * Created on 31 marzec 2004, 11:24
 */

package pl.psnc.vlab.graph;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.KeyListener;
import java.awt.geom.Point2D;
import java.util.Hashtable;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import org.jgraph.event.GraphSelectionListener;
import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultPort;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphLayoutCache;
import pl.psnc.vlab.graph.model.CustomCellViewFactory;
import pl.psnc.vlab.graph.model.ScenarioCellObject;
import pl.psnc.vlab.graph.model.ScenarioGraph;
import pl.psnc.vlab.graph.model.ScenarioGraphModel;
import pl.psnc.vlab.graph.model.ScenarioMarqueeHandler;
import pl.psnc.vlab.graph.model.ScenarioUndoManager;
import pl.psnc.vlab.graph.util.GraphKeys;
import pl.psnc.vlab.graph.vertex.CustomPort;
import pl.psnc.vlab.graph.vertex.type.TypeUtil;
import pl.psnc.vlab.ssa.exception.SSAException;
import pl.psnc.vlab.util.image.ImageFactory;

/**
 *
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class ScenarioGraphEditor extends JPanel implements GraphSelectionListener, KeyListener{
    
    
    /** Instance of JGraph*/
    private ScenarioGraph graph = null;
    
    /** Instance of JGraph*/
    private ScenarioGraphModel graphModel = null;
    
    /** Instance of JGraph*/
    private ScenarioUndoManager undoManager = null;
    
    /** Instance of Marquee Handler */
    private ScenarioMarqueeHandler marqueeHandler = null;
    
    /** Instance of graph Scroll Pane  */
    private JScrollPane graphScrolPane = null;
    
    
    
    /** Creates a new instance of DefaultScenarioGraph
     * @throws SSAException
     */
    protected ScenarioGraphEditor() throws SSAException{
        //removed graph 5.10 GraphModelProviderRegistry.addGraphModelProvider(new DefaultGraphModelProvider());
        this.graphModel = new ScenarioGraphModel();                             //construct the model
        
        GraphLayoutCache view = new GraphLayoutCache(graphModel, new CustomCellViewFactory());
        this.graph = new ScenarioGraph(this, this.graphModel, view);                 // Construct the Graph
        // Create a GraphUndoManager which also Updates the ToolBar
        this.undoManager = new ScenarioUndoManager(graph);
        
        // Add Listeners to Graph
        //
        // Register UndoManager with the Model
        graph.getModel().addUndoableEditListener(undoManager);
        // Update ToolBar based on Selection Changes
        graph.getSelectionModel().addGraphSelectionListener(this);
        // Listen for Delete Keystroke when the Graph has Focus
        graph.addKeyListener(this);
        
        //graph.setPreferredSize(new Dimension(400, 200));
        //JScrollPane graphScroll = new JScrollPane(graph);
        this.setLayout(new BorderLayout() );
        //add(graphScroll, BorderLayout.CENTER);
        //add(graph, BorderLayout.CENTER);
    }
    
    
    // Insert a new Vertex at point
    public void insert(Point2D point, DefaultGraphCell vertex) {
        // Construct Vertex with no Label
        // Add one Floating Port
        DefaultPort port = new CustomPort();
        graph.getGraphLayoutCache().setVisible(port, false);
        
        AttributeMap portAttr = new AttributeMap();
        //GraphConstants.setBackground(portAttr, Color.red);
        //GraphConstants.setForeground(portAttr, Color.black);
        GraphConstants.setGradientColor(portAttr, Color.red);
        port.setAttributes(portAttr);
        
        vertex.add(port);
        
        // Snap the Point to the Grid
        
        // 5.10 removed AttributeMap map = graph.getModel().createAttributes();
        AttributeMap map = new AttributeMap();
        point = graph.snap(map.createPoint(point));
        
        // Default Size for the new Vertex
        ScenarioCellObject cellObject = (ScenarioCellObject) vertex.getUserObject();
        ImageIcon vartexIcon = ImageFactory.getImageBean(cellObject.getType().getType().getImage().getVertex().getPath()).getImage();
        Dimension size = new Dimension(vartexIcon.getIconWidth(), vartexIcon.getIconHeight());
        GraphConstants.setIcon(map, vartexIcon);
        // Add a Bounds Attribute to the Map
        GraphConstants.setBounds(map, map.createRect(point.getX(), point.getY(), size.width, size.height + GraphKeys.VERTEX_LABEL_MARGIN));
        // Add a Border Color Attribute to the Map
        //GraphConstants.setBorderColor(map, Color.black);
        GraphConstants.setLineColor(map, Color.red);
        GraphConstants.setLineWidth(map, 4);
        GraphConstants.setConnectable(map, true);
        GraphConstants.setDisconnectable(map, true);
        GraphConstants.setSizeable(map, false);
        GraphConstants.setEditable(map, false);
        
        //check type
        if(cellObject.getType().getTypeId() == TypeUtil.RADIO_TELESCOPE){
            GraphConstants.setMoveable(map, false);
        }
        
        // Make Vertex Opaque
        GraphConstants.setOpaque(map, false);
        // Construct a Map from cells to Maps (for insert)
        Hashtable attributes = new Hashtable();
        // Associate the Vertex with its Attributes
        attributes.put(vertex, map);
        
        // Insert the Vertex and its Attributes (can also use model)
        graph.getGraphLayoutCache().insert( new Object[] { vertex }, attributes,  null,  null,  null);
        
    }
    
    
    
    
    public void keyPressed(java.awt.event.KeyEvent e) {
    }
    
    public void keyReleased(java.awt.event.KeyEvent e) {
    }
    
    public void keyTyped(java.awt.event.KeyEvent e) {
    }
    
    public void valueChanged(org.jgraph.event.GraphSelectionEvent graphSelectionEvent) {
        
        
    }
    
    public ScenarioGraph getGraph(){
        return this.graph;
    }
    
    
    
    public static void main(String[] arg){
        try{
            // Construct Frame
            JFrame frame = new JFrame("GraphEd");
            frame.getContentPane().add(new ScenarioGraphEditor());
            // Set Default Size
            frame.setSize(520, 390);
            //add scenario graph editor to the main frame
            // Show Frame
            frame.show();
        } catch(Exception e){
            e.printStackTrace();
        }
    }
}
