/*
 * @(#)Graphpad.java	1.2 11/11/02
 *
 * Copyright (C) 2001 Gaudenz Alder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package pl.psnc.vlab.graph.model;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultPort;
import pl.psnc.vlab.graph.vertex.ImageCell;
import pl.psnc.vlab.graph.vertex.type.VLabVertexType;

public class GraphCellFactory {
    
    //-----------------------------------------------
    //---   Cell types
    /** Edge type */
    public static String CELL_TYPE_EDGE = "edge";
    /** Port type */
    public static String CELL_TYPE_PORT = "port";
    /** Image type */
    public static String CELL_TYPE_IMAGE = "image";
    /** Rectangle type */
    public static String CELL_TYPE_RECTANGLE = "rect";
    
    /** Instance of logger*/
    private static Log log = LogFactory.getLog("GraphCellFactory");
    
    /** Creates a new cell. The cell type is specyfied by the
     * type object. It can be one of the following image cell,
     * edge, port
     * @param type the type of the cell
     * @return
     */
    public static DefaultGraphCell createCell(Object type, ScenarioCellObject cellObject) {
        //check if vertex is Compute one
        if(type instanceof VLabVertexType){
            VLabVertexType vlabType = (VLabVertexType) type;
            log.debug("VlabVertexType:"+vlabType.getGenericTypeName());
            if(cellObject == null){
                log.debug("CellObject is null");
                cellObject = new ScenarioCellObject("" , vlabType);
            }
            return new ImageCell(cellObject);        
            
        } else if(type instanceof String){
            String stringType = (String) type;
            if(cellObject == null){
                cellObject = new ScenarioCellObject("", null);
            }
            if (stringType.equals(CELL_TYPE_EDGE)){
                return new DefaultEdge(cellObject);
            } else if (type.equals(CELL_TYPE_PORT)){
                return new DefaultPort(cellObject);
            } else if (type.equals(CELL_TYPE_IMAGE)){                
                log.debug("Test:"+cellObject.getType());
                return new ImageCell(cellObject);
            }
        }
        
        return new DefaultGraphCell(cellObject);
    }
    
    
    
    /** Get the type of the given cell
     * @param cell
     * @return
     */
    public static String getType(Object cell) {
        if (cell instanceof DefaultPort)
            return CELL_TYPE_PORT;
        else if (cell instanceof ImageCell)
            return CELL_TYPE_IMAGE;
        else if (cell instanceof DefaultEdge)
            return CELL_TYPE_EDGE;
        return CELL_TYPE_RECTANGLE;
    }
            
    
}
