package pl.psnc.vlab.graph.model;
import java.io.Serializable;
import java.util.Hashtable;
import java.util.Map;
import java.util.ResourceBundle;
import javax.swing.JDialog;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.GraphConstants;
import pl.psnc.vlab.graph.vertex.type.VLabVertexType;
import pl.psnc.vlab.rds.ResourceDocument;
import pl.psnc.vlab.util.FormTools;
import pl.psnc.vlab.xml.*;
import pl.psnc.vlab.xml.util.XmlSerializer;


/**
 *
 * @author Dominik Stoklosa
 * @version 1.0
 *
 */
public class ScenarioCellObject implements Serializable{
    
    
    /** Instance of logger*/
    private static Log log = LogFactory.getLog("GraphCellFactory");
    
    /** THe property indicates, whether the current node is marked as initial */
    private static String MARKED_INITIAL_PROPERTY = "isMarkedAsInitial";
    
    /** The property key for the type of the cell  */
    private static String CELL_TYPE = "cellType";
    
    /** The property key for the type of the cell  */
    private static String CELL_RESOURCE = "resourceCell";
    
    /* defaultKey of the property used to return a value from the toString method
     */
    public static String keyValue = "value";
    
    /* defaultKey of the property used to return a value from the toString method
     */
    public static String keyURI = "url";
    
    /* Map that holds the attributes (key value pairs)
     */
    protected Map properties;
    
    protected transient JDialog propertyDlg;
    
    protected transient JTable table;
    
    protected transient DefaultTableModel dataModel;
    
    /** Holds value of property type. */
    private VLabVertexType type;
    
    /**
     * Holds value of property resource.
     */
    private ResourceDocument.Resource resource;
    
    /**
     * Holds value of property markedInitial.
     */
    private boolean markedInitial;
    
    public ScenarioCellObject(VLabVertexType type) {
        // empty constructor required by XMLDecoder
        this(null, null, type);
        //setType(type);
    }
    
    /**
     *
     */
    public ScenarioCellObject(String label, Map properties, VLabVertexType type) {
        if (properties == null){
            properties = new Hashtable();
        }
        this.properties = new Hashtable(properties);
        setType(type);
        if (label != null){
            valueChanged(label);
        }
        //set resource
        setResource(null);
    }
    
    public ScenarioCellObject(String label, VLabVertexType type) {
        this(label, null, type);
    }
    
    public ScenarioCellObject(Map properties, VLabVertexType type) {
        this(null, properties, type);
    }
    
    /* (non-Javadoc)
     * @see org.jgraph.graph.DefaultGraphCell.ValueChangeHandler#valueChanged(java.lang.Object)
     */
    public Object valueChanged(Object newValue) {
        log.debug("<valueChanged>");
        if (newValue instanceof String){
            log.debug("String:"+newValue );
            return putProperty(keyValue, newValue);
        } else if(newValue instanceof VLabVertexType){
            VLabVertexType type = (VLabVertexType) newValue;
            log.debug("Got new type. Element type:"+type.getTypeName());
            setType(type);
            return this;
        }
        
        log.debug("</valueChanged>");
        return this;
    }
    
    /* (non-Javadoc)
     * @see org.jgraph.graph.DefaultGraphCell.ValueChangeHandler#clone()
     */
    public Object clone() {
        return new ScenarioCellObject(properties, getType());
    }
    
    public Object getProperty(Object key) {
        return properties.get(key);
    }
    
    public Object putProperty(Object key, Object value) {
        if (value != null)
            return properties.put(key, value);
        return properties.remove(key);
    }
    
    /**
     * @return
     */
    public Map getProperties() {
        return properties;
    }
    
    /**
     * @param map
     */
    public void setProperties(Map map) {
        properties = map;
    }
    
    
    protected void apply(ScenarioGraph graph, Object cell, TableModel model) {
        properties.clear();
        for (int i = 0; i < model.getRowCount(); i++) {
            properties.put(model.getValueAt(i, 0), model.getValueAt(i, 1));
        }
        Map nested = new Hashtable();
        AttributeMap attr = new AttributeMap();
        Object value = getProperty(keyValue);
        if (value == null)
            putProperty(keyValue, "");
        GraphConstants.setValue(attr, getProperty(keyValue));
        nested.put(cell, attr);
        graph.getModel().edit(nested, null, null, null);
    }
    
    public String toString() {
        Object label = properties.get(keyValue);
        if (label != null)
            return label.toString();
        return super.toString();
    }
    
    /** Getter for property type.
     * @return Value of property type.
     *
     */
    public VLabVertexType getType() {
        /*if(this.type == null){
            String serType = (String) getProperty(CELL_TYPE);
            log.debug("serType-"+serType);
            this.type = (VLabVertexType) XmlSerializer.decode(serType);
        }*/
        return this.type;
    }
    
    /** Setter for property type.
     * @param type New value of property type.
     *
     */
    public void setType(VLabVertexType type) {
        if(type == null){
            String serType = (String) getProperty(CELL_TYPE);
            log.debug("serType-"+serType);
            this.type = (VLabVertexType) XmlSerializer.decode(serType);
        } else {
            //serialize object
            String typeSerialized = XmlSerializer.encode(type);
            log.debug("typeSerialized-"+typeSerialized);
            putProperty(CELL_TYPE, typeSerialized);
            this.type = type;
        }
    }
    /**
     * Getter for property resource.
     * @return Value of property resource.
     */
    public ResourceDocument.Resource getResource() {
        return this.resource;
    }
    
    /**
     * Setter for property resource.
     * @param resource New value of property resource.
     */
    public void setResource(ResourceDocument.Resource resource) {
        if(resource == null){
            String resAsString = (String) getProperty(CELL_RESOURCE);
            this.resource = ResourceFactory.getResource(resAsString);
        } else {
            resource.setGenericTypeId(type.getGenericTypeId());         //set parent type id    DMS PURPOSE
            putProperty(CELL_RESOURCE, ResourceFactory.resourceAsString(resource));
            this.resource = resource;
        }
    }
    
    /**
     * Getter for property isMarkedInitial.
     * @return Value of property isMarkedInitial.
     */
    public boolean isMarkedInitial() {
        log.debug("<isMarkedInitial>");
        Object o = this.getProperty(MARKED_INITIAL_PROPERTY);
        if(o == null){
            log.debug("Property not set");
        }
        boolean result = o != null ? new Boolean((String)o).booleanValue() : false;
        log.debug("</isMarkedInitial>");
        return result;
    }
    
    /**
     * Setter for property isMarkedInitial.
     * @param isMarkedInitial New value of property isMarkedInitial.
     */
    public void setMarkedInitial(boolean isMarkedInitial) {
        this.putProperty(MARKED_INITIAL_PROPERTY, Boolean.toString(isMarkedInitial));
    }
    
    
    /**
     * Get the node tooltip
     * @return
     */
    public String getToolTip(){
        try{
            String initial = isMarkedInitial() ? ResourceBundle.getBundle("bundle/dynamicScenario").getString("value.yes") :
                ResourceBundle.getBundle("bundle/dynamicScenario").getString("value.no");
            StringBuffer tip = new StringBuffer()
            .append("<html>")
            .append("<table border='0' >")
            .append("<tr>")
            .append("<td><img src='none' border='0'></img></td>")
            .append("<td></td>")
            //put node type in here
            .append("<td><h3>").append( this.type.getGenericTypeName().trim()).append("</h3></td>")
            .append("</tr>")
            .append("</table>")
            
            .append("<table border='0'>")
            .append("<tr>")
            .append("<td width='60'></td>")
            .append("<td><font face='Verdana' size='3' color='red'>Name:</font></td>")
            //put node name in here
            .append("<td><font face='Verdana' size='2' color='blue'>").append(this.resource.getName()).append("</font></td>")
            .append("</tr>")
            .append("<tr>")
            .append("<td width='60'></td>")
            .append("<td><font face='Verdana' size='3' color='red'>Description:</font></td>")
            //put desc in here
            .append("<td><font face='Verdana' size='2' color='blue'>")
            .append(!FormTools.isValidateFormStringNull(resource.getDescription()) ? resource.getDescription() :
                ResourceBundle.getBundle("bundle/dynamicScenario").getString("value.none"))
                .append("</font></td>")
                .append("</tr>")
                
                
                //initital node
                .append("<tr>")
                .append("<td width='60'></td>")
                .append("<td><font face='Verdana' size='3' color='red'>Initial node:</font></td>")
                //put desc in here
                .append("<td><font face='Verdana' size='2' color='blue'>").append(initial).append("</font></td>")
                .append("</tr>")
                
                .append("</table>")
                .append("</html>");
            return tip.toString();
        }catch(Exception e){
            log.error("Bad tooltip");
            return null;
        }
    }
    
    
    
    
}
