/*
 * ScenarioGraphModelListener.java
 *
 * Created on 23 lipiec 2004, 16:56
 */

package pl.psnc.vlab.graph.model;

import java.util.Date;
import javax.swing.Action;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jgraph.event.GraphModelListener;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.util.JGraphUtilities;
import pl.psnc.vlab.exception.VlabException;
import pl.psnc.vlab.graph.util.GraphHelper;
import pl.psnc.vlab.management.session.SessionFactory;
import pl.psnc.vlab.rds.ScenarioDocument;
import pl.psnc.vlab.ssa.actions.ActionFactory;
import pl.psnc.vlab.ssa.exception.SSAException;
import pl.psnc.vlab.ssa.gui.MainFrame;
import pl.psnc.vlab.ssa.util.Keys;
import pl.psnc.vlab.ssa.management.SystemConfigurationFactory;
import pl.psnc.vlab.ssa.management.SystemConfigurationValue;
import pl.psnc.vlab.util.DateTools;
import pl.psnc.vlab.values.management.SessionValue;
import pl.psnc.vlab.xml.LinksFactory;
import pl.psnc.vlab.xml.ScenarioBuilder;

/**
 * The model graph listener is informed when the cells are inserted,
 * removed or when the label, source, target, parent or children of an oject have changed
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class ScenarioGraphModelListener implements GraphModelListener{
    
    /** Instance of logger*/
    private Log log = LogFactory.getLog(getClass().getName());
    
    /** Stores an instance of current graph */
    private ScenarioGraph graph = null;
    
    /**
     * Creates an instance of Scenario Graph Model Listener
     * @param graph
     */
    public ScenarioGraphModelListener(ScenarioGraph graph) {
        this.graph = graph;
    }
    
    
    public void graphChanged(org.jgraph.event.GraphModelEvent graphModelEvent) {
        log.debug("<graphChanged>");
        //get save action
        Action saveAction = ActionFactory.getAction(ActionFactory.ACTION_SAVE_FILE);
        Action saveAsAction = ActionFactory.getAction(ActionFactory.ACTION_SAVE_FILE_AS);
        //get submit scenario action
        Action submitAction = ActionFactory.getAction(ActionFactory.ACTION_SUBMIT_SCENARIO);
        if(graph.isEmpty()){
            saveAction.setEnabled(false);
            saveAsAction.setEnabled(false);
            submitAction.setEnabled(false);
        } else {
            saveAction.setEnabled(true);
            saveAsAction.setEnabled(true);
            submitAction.setEnabled(true);
        }
        //check the connections //remove not valid connetcions
        GraphHelper.validateEdges(graph);
        
        //change the modification date
        updateScenarioDocument();
        log.debug("</graphChanged>");
    }
    
    private void updateScenarioDocument(){
        //set dates
        ScenarioDocument.Scenario scen = ScenarioBuilder.getScenarioDescription();
        if(scen==null){
            return;
        }
        SystemConfigurationValue sysConfVO = SystemConfigurationFactory.getInstance();
        DateTools dateTools = new DateTools(sysConfVO.getCurrentLocale(), Keys.DATE_FORMAT, Keys.TIME_FORMAT);
        Date curDate = MainFrame.getInstance().getStatusBar().getCurrentDate();
        String date = dateTools.getDateAsString(curDate);
        String time = dateTools.getTimeAsString(curDate);
        scen.setModificationDate(date+" "+time);
        //set owner
        /*try{
            SessionValue sessionValue = SessionFactory.getSessionValue();
            scen.setModifiedBy(sessionValue.getUserName());
        }catch(VlabException e){
            log.debug("Error-"+e.getLocalizedMessage());
        }*/
        ScenarioBuilder.setScenarioDescription(scen);
    }
    
    
}
