/*
 * ScenarioMarqueeHandler.java
 *
 * Created on 1 kwiecie� 2004, 11:25
 */

package pl.psnc.vlab.graph.model;

import com.jgoodies.looks.Options;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Hashtable;
import java.util.Map;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;
import javax.swing.tree.TreeNode;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.BasicMarqueeHandler;
import org.jgraph.graph.ConnectionSet;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.Port;
import org.jgraph.graph.PortView;
import pl.psnc.vlab.exception.VlabException;
import pl.psnc.vlab.graph.renderer.CustomPortRenderer;
import pl.psnc.vlab.graph.util.GraphHelper;
import pl.psnc.vlab.graph.util.gui.JPopupHelper;
import pl.psnc.vlab.graph.vertex.ImageCell;
import pl.psnc.vlab.graph.view.CustomPortView;
import pl.psnc.vlab.ssa.gui.MainFrame;
import pl.psnc.vlab.ssa.gui.resource.pane.JAddResourcePopup;
import pl.psnc.vlab.ssa.gui.resource.pane.JTest;
import pl.psnc.vlab.ssa.util.i18n.MessageBundleFactory;
import pl.psnc.vlab.xml.LinksFactory;

/**
 *
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class ScenarioMarqueeHandler extends BasicMarqueeHandler implements ActionListener{
    
    
    /** Instance of logger*/
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Instance of JGraph*/
    private ScenarioGraph graph = null;
    
    /** Instance of JPopup handler */
    private JPopupHelper popupHelper = null;
    
    private AttributeMap map = null;
    
    /** Creates a new instance of ScenarioMarqueeHandler */
    public ScenarioMarqueeHandler(ScenarioGraph graph) {
        super();
        this.graph = graph;        
        // removed Graph 10.1 this.map = graph.getModel().createAttributes();
        this.map = new AttributeMap();
    }
    
    // Holds the Start and the Current Point
    protected Point2D start, current;
    
    public static Point2D tmpPoint = null;
    
    /** Holds an instance of the first cell for nodes connection */
    private ScenarioCellObject firstCell = null;
    
    // Holds the First and the Current Port
    protected PortView port, firstPort;
    private CustomPortView tmpPort;
    
    
    
    /**
     * Override to Gain Control (for PopupMenu and ConnectMode)
     * @param e mouse event
     * @return
     */
    public boolean isForceMarqueeEvent(MouseEvent e) {
        log.debug("<isForceMarqueeEvent>");
        if (SwingUtilities.isRightMouseButton(e)){
            return true;
        }
        
        if (SwingUtilities.isLeftMouseButton((e)) && e.getClickCount()==2){
            return true;
        }
        // Find and Remember Port
        port = getSourcePortAt(e.getPoint());
        // If Port Found and in ConnectMode (=Ports Visible)
        if (port != null && graph.isPortsVisible()){
            return true;
        }
        // Else Call Superclass
        return super.isForceMarqueeEvent(e);
    }
    
    
    /**
     * Handle the mouse click event
     * @param e mouse event
     */
    public void mousePressed(final MouseEvent e) {
        MainFrame.getStatusBar().setMessage("");
        // Scale From Screen to Model
        Point2D loc = graph.fromScreen(map.createPoint(e.getPoint()));
        // Find Cell in Model Coordinates
        Object cell = graph.getFirstCellForLocation(loc.getX(), loc.getY());
        
        if(GraphHelper.isEdge(cell) || GraphHelper.isPort(cell)){     //skip non vertex
            e.consume();
            return;
        }
        
        if (SwingUtilities.isRightMouseButton(e)) {         // If Right Mouse Button
            // Create PopupMenu for the Cell
            graph.setSelectionCells(new Object[] {cell});
            JPopupMenu menu = createPopupMenu(e.getPoint(), cell);
            tmpPoint = e.getPoint();            
            if(menu != null ){
                menu.show(graph, e.getX(), e.getY());
            }
            
            return ;
        }
        
        if(SwingUtilities.isLeftMouseButton(e) && e.getClickCount() == 2){      // 2 left click -> call handler
            if(cell == null){       //no cell -> return
                return;
            }
            popupHelper = new JPopupHelper(this, graph, cell, e.getPoint());
            JPopupMenu nodeTypeMenu = (JPopupMenu) popupHelper.doubleClickHandler();
            if(nodeTypeMenu != null){
                nodeTypeMenu.show(graph, e.getX(), e.getY());
            }
            return;
        }
        
        if(port != null && !e.isConsumed() && cell != null && graph.isPortsVisible()) { // Else if in ConnectMode and Remembered Port is Valid
            if(GraphHelper.isGeneric((DefaultGraphCell) cell)){
                log.debug("generic");
                e.consume();
                return;
            }
            
            // Remember Start Location
            start = graph.toScreen(port.getLocation(null));
            // Remember First Port
            firstPort = port;
            firstCell = GraphHelper.getCellObject(cell);     //remember first cell
            // Consume Event
            e.consume();
            return;
        } else {            
            super.mousePressed(e);
        }
    }
    
    /**
     * Hadle the mouse dragged event
     * @param e mouse event
     */
    public void mouseDragged(MouseEvent e) {
        // If remembered Start Point is Valid
        if (start != null && !e.isConsumed()) {
            // Fetch Graphics from Graph
            Graphics g = graph.getGraphics();
            // Xor-Paint the old Connector (Hide old Connector)
            paintConnector(Color.black, graph.getBackground(), g);
            // Reset Remembered Port
            port = getTargetPortAt(e.getPoint());
            // If Port was found then Point to Port Location
            if (port != null) {
                current = graph.toScreen(port.getLocation(null));
            } else {                // Else If no Port was found then Point to Mouse Location
                current = graph.snap(e.getPoint());
            }
            // Xor-Paint the new Connector
            paintConnector(graph.getBackground(), Color.black, g);
            // Consume Event
            e.consume();
        }
        // Call Superclass
        super.mouseDragged(e);
    }
    
    /**
     *
     * @param point
     * @return
     */
    public PortView getSourcePortAt(Point2D point) {
        // Scale from Screen to Model
        Point2D tmp = graph.fromScreen( map.createPoint(point) );
        // Find a Port View in Model Coordinates and Remember
        return graph.getPortViewAt(tmp.getX(), tmp.getY());
    }
    
    // Find a Cell at point and Return its first Port as a PortView
    public PortView getTargetPortAt(Point2D point) {
        // Find Cell at point (No scaling needed here)
        Object cell = graph.getFirstCellForLocation(point.getX(), point.getY());
        // Loop Children to find PortView
        for (int i = 0; i < graph.getModel().getChildCount(cell); i++) {
            // Get Child from Model
            Object tmp = graph.getModel().getChild(cell, i);
            // Get View for Child using the Graph's View as a Cell Mapper
            tmp = graph.getGraphLayoutCache().getMapping(tmp, false);
            // If Child View is a Port View and not equal to First Port
            if (tmp instanceof PortView && tmp != firstPort)
                // Return as PortView
                return (PortView) tmp;
        }
        // No Port View found
        return getSourcePortAt(point);
    }
    
    // Connect the First Port and the Current Port in the Graph or Repaint
    public void mouseReleased(MouseEvent e) {
        Point p = null;
        if(e != null && (p=e.getPoint()) == null){
            graph.repaint();
            return;
        }
        
        Point2D loc = graph.fromScreen(p);
        if(loc == null){
            graph.repaint();
            return;
        }
        Object cell = graph.getFirstCellForLocation(loc.getX(), loc.getY());
        if(GraphHelper.isGeneric(cell)) {
            MainFrame.getStatusBar().setMessage(MessageBundleFactory.getValue("message.graph.no.links.defined") );
            e.consume();
            graph.repaint();
            firstPort = port = null;
            firstCell = null;
            start = current = null;
            return;
        }
        
        // If Valid Event, Current and First Port
        if (e != null  && !e.isConsumed() && port != null && firstPort != null
                && firstPort != port && firstCell != null && cell !=null) {
            connectNodes(e, GraphHelper.getCellObject(cell));
        } else {
            //remove link
            graph.repaint();
        }
        // Reset Global Vars
        firstPort = port = null;
        firstCell = null;
        start = current = null;
        // Call Superclass
        super.mouseReleased(e);
    }
    
    private Rectangle2D rect;
    
    // Show Special Cursor if Over Port
    public void mouseMoved(MouseEvent e) {
        Point2D loc = graph.fromScreen(map.createPoint(e.getPoint()));
        Object cell = graph.getFirstCellForLocation(loc.getX(), loc.getY());
        if(GraphHelper.isGeneric(cell)) {
            MainFrame.getStatusBar().setMessage(MessageBundleFactory.getValue("message.graph.no.links.defined"));
            e.consume();
            return;
        }
        
        
        //CustomPort port =  (CustomPort) graph.getPortForLocation(loc.getX(), loc.getY());
        //CustomPortView portView = (CustomPortView)  graph.getGraphLayoutCache().getMapping(port, false);
        
        if(cell != null && GraphHelper.isVertex(cell)){
            
            Graphics g = graph.getGraphics();
            AttributeMap map = ( (DefaultGraphCell)  cell).getAttributes();
            rect = GraphConstants.getBounds(map);
            g.setColor(Color.red);
            g.drawRect((int) rect.getX() -1 , (int) rect.getY() -1, (int) rect.getWidth() +1, (int) rect.getHeight()+1);
            
            TreeNode node = ((DefaultGraphCell) cell).getFirstChild();
            if(node != null){
                tmpPort = (CustomPortView)  graph.getGraphLayoutCache().getMapping(node, false);
                tmpPort.setVisible(true);
                graph.repaint( (int) rect.getX(), (int) rect.getY() , (int) rect.getWidth(), (int) rect.getHeight());
            }
            
        } else {
            if(tmpPort != null){
                tmpPort.setVisible(false);
                tmpPort = null;
            }
            
            if(rect != null) {
                graph.repaint( (int) rect.getX() -1, (int) rect.getY() -1 , (int) rect.getWidth() +3, (int) rect.getHeight() +3);
                rect = null;
                graph.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            }
        }
        
        
        // Check Mode and Find Port
        if (e != null && getSourcePortAt(e.getPoint()) != null && !e.isConsumed() && graph.isPortsVisible()) {
            // Set Cusor on Graph (Automatically Reset)
            graph.setCursor(new Cursor(Cursor.HAND_CURSOR));
            // Consume Event
            e.consume();
        } else if( cell != null){
            // Set Cusor on Graph (Automatically Reset)
            graph.setCursor(new Cursor(Cursor.MOVE_CURSOR));
            // Consume Event
            e.consume();
        } else {
            graph.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            e.consume();
        }
        
        // Call Superclass
        super.mouseMoved(e);
    }
    
    // Use Xor-Mode on Graphics to Paint Connector
    protected void paintConnector(Color fg, Color bg, Graphics g) {
        // Set Foreground
        g.setColor(fg);
        // Set Xor-Mode Color
        g.setXORMode(bg);
        // Highlight the Current Port
        //paintPort(graph.getGraphics());
        // If Valid First Port, Start and Current Point
        if (firstPort != null && start != null && current != null){
            // Then Draw A Line From Start to Current Point
            g.drawLine( (int) start.getX(), (int) start.getY(), (int) current.getX(), (int) current.getY());
        }
    }
    
    // Use the Preview Flag to Draw a Highlighted Port
    protected void paintPort(Graphics g) {
        if (port != null) {
            boolean offset =
                    (GraphConstants.getOffset(port.getAllAttributes()) != null);
            Rectangle2D r = (offset) ? port.getBounds() : port.getParentView().getBounds();
            r = (graph.toScreen(new Rectangle(r.getBounds())));
            int s = 4;
            Rectangle tempRect = (Rectangle) r;
            tempRect.translate(-s, -s);
            r.setRect((int)r.getX(),
                    (int)r.getY(),
                    (int)(r.getWidth() + 2 * s),
                    (int)(r.getHeight() + 2 * s));
            graph.getUI().paintCell(g, port, r, true);
        }
    }
    
    /** Creates a JPopup menu
     * @param pt
     * @param cell
     * @return
     */
    public JPopupMenu createPopupMenu(final Point2D pt, final Object cell) {
        JPopupMenu menu = new JPopupMenu();
        popupHelper = new JPopupHelper(this, graph, cell, pt);
        if(cell == null){
            menu.setPreferredSize(new Dimension(200, 25));
            menu.add( popupHelper.getAddNodeMenu());
            menu.putClientProperty(Options.NO_ICONS_KEY, Boolean.TRUE);
            return menu;
        }
        
        
        //add node type menu item
        menu.add( popupHelper.getNodeTypeMenuItem() );
        menu.addSeparator();
        //get the edit menu
        menu = popupHelper.getEditMenu(menu);
        //menu.addSeparator();
        //add resource list submenu
        menu.add( popupHelper.getResourceMenu() );
        //attach the property dialog
        JMenuItem propertyItem = popupHelper.getNodePropertyMenuItem();
        if(propertyItem != null){
            menu.addSeparator();
            menu.add(propertyItem);
        }
        
        if(!GraphHelper.isGeneric(cell)){
            menu.addSeparator();
            menu.add(popupHelper.getMarkInitialNodeMenu());
        }
        return menu;
    }
    
    
    
    
    /**
     * @param e
     */
    public void actionPerformed(ActionEvent e) {
        try{
            JMenuItem source = (JMenuItem)(e.getSource());
            String name = source.getName();
            log.debug("actionPerformed source:"+name);
            popupHelper.handleAction(name);
        }catch(VlabException err){
            log.error("Exception occured: " +err.getMessage());
            //ignored TBD
            //add exception handling in here
        }
        
    }
    
    // Insert a new Edge between source and target
    public void connect(Port source, Port target) {
        // Connections that will be inserted into the Model
        ConnectionSet cs = new ConnectionSet();
        // Construct Edge with no label
        DefaultEdge edge = new DefaultEdge();
        // Create Connection between source and target using edge
        cs.connect(edge, source, target);
        // Create a Map thath holds the attributes for the edge
        //removed Graph 10.1 Map cMap = graph.getModel().createAttributes();
        Map cMap = new AttributeMap();
        // Add a Line End Attribute
        GraphConstants.setLineEnd(cMap, GraphConstants.ARROW_CLASSIC);
        GraphConstants.setEndFill(cMap, true);
        GraphConstants.setLineWidth(cMap, 1);
        
        // Add a label along edge attribute
        GraphConstants.setLabelAlongEdge(cMap, true);
        // Construct a Map from cells to Maps (for insert)
        Hashtable attributes = new Hashtable();
        // Associate the Edge with its Attributes
        attributes.put(edge, cMap);
        // Insert the Edge and its Attributes
        graph.getGraphLayoutCache().insert(new Object[] { edge }, attributes, cs,null,null);
        
    }
    
    
    /**
     * Connects two nodes
     * @param e
     */
    private void connectNodes(MouseEvent e, ScenarioCellObject targetCell){
        //check wheter the connection can be established
        int sourceId = firstCell.getResource().getTypeId();
        int targetId = targetCell.getResource().getTypeId();
        if( LinksFactory.canConnect(sourceId, targetId)){
            // Then Establish Connection
            connect((Port) firstPort.getCell(), (Port) port.getCell());
            MainFrame.getStatusBar().setMessage("");
        } else {
            log.debug("No connection defined");
            MainFrame.getInstance().getStatusBar().setMessage(MessageBundleFactory.getValue("message.graph.no.links.defined") );
        }
        e.consume();    // Consume Event
        firstCell = null;
        graph.repaint();
    }
}

