/*
 * CustomPortRenderer.java
 *
 * Created on 8 listopad 2006, 15:48
 */

package pl.psnc.vlab.graph.renderer;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import javax.swing.ImageIcon;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jgraph.JGraph;
import org.jgraph.graph.CellView;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.PortRenderer;
import org.jgraph.graph.PortView;
import pl.psnc.vlab.graph.view.CustomPortView;
import pl.psnc.vlab.util.image.ImageFactory;

/**
 *
 * @author <a href="mailto:osa@man.poznan.pl">Dominik Stoklosa (~osa~)</a>
 * @email  osa@man.poznan.pl
 */
public class CustomPortRenderer extends PortRenderer {
    
    /** Holds logger instance */
    private Log log = LogFactory.getLog(getClass().getName());
    
    private JGraph graph;
    
    /** Holds instance of port image */
    private static final ImageIcon portImg;
    static {
        portImg = ImageFactory.getImageBean("images/red_dot.gif").getImage();
    }
    
    public CustomPortRenderer(){
        
    }
    
    /**
     * Get Renderer Component for this class
     *
     * @return component renderer
     * @param graph
     * @param cellView
     * @param selected
     * @param focus
     * @param preview
     */
    public Component getRendererComponent(JGraph graph, CellView cellView, boolean selected, boolean focus, boolean preview) {
        //log.debug("Port renderer  -> foces="+focus+" sel="+selected +" prew="+preview);
        if (cellView instanceof PortView && graph != null) {
            graphBackground = Color.red;
            this.view = (PortView) cellView;
            this.hasFocus = focus;
            this.selected = selected;
            this.preview = preview;
            this.graph = graph;
            this.xorEnabled = graph.isXorEnabled();
            return this;
        }
        return null;
    }
    
    
    
    /**
     * Paint the renderer. Overrides superclass paint to add specific painting.
     * Note: The preview flag is interpreted as "highlight" in this context.
     * (This is used to highlight the port if the mouse is over it.)
     */
    public void paint(Graphics g) {
        Dimension d = getSize();
        if(hasFocus){
            g.setColor(Color.red);
            g.fill3DRect(0,0,d.width, d.height, true);
        } else if(selected){
            g.setColor(Color.blue);
            g.fill3DRect(0,0,d.width, d.height, true);
        } else if(preview){
            g.setColor(Color.yellow);
            g.fill3DRect(0,0,d.width, d.height, true);
        } else {
            //super.paint(g);
            //g.drawImage(portImg.getImage(), 0, 0, portImg.getIconWidth(), portImg.getIconHeight(), getParent());
            g.setColor(Color.black);
            //g.drawRect(0,0, 200, 200);
            if(view !=null && ( (CustomPortView) view).isVisible() ){
                g.drawImage(portImg.getImage(), 0, 0, portImg.getIconWidth(), portImg.getIconHeight(), getParent());
            }
        }
    }
    
    
    
    
    //-------------------------------------------------------------------------
    //----  Helper methods
    
    /** Get size of the port image */
    public Dimension getIconSize(){
        return new Dimension(portImg.getIconWidth(), portImg.getIconHeight());
    }
    
    
}
