/*
 * RendererFactory.java
 *
 * Created on 23 kwiecien 2004, 16:32
 */

package pl.psnc.vlab.graph.renderer;

import java.util.Hashtable;
import javax.swing.ImageIcon;
import org.jgraph.JGraph;
import pl.psnc.vlab.ssa.util.ImageKeys;
import pl.psnc.vlab.util.image.ImageFactory;


/**
 * This abstract class is used to get instances of the image
 * vertex renderers for the application. For the current version there
 * are three types of the renderes:
 * <li> compute renderer </li>
 * <li> experiment renderer - NMR </li>
 * <li> experiment renderer - Radiotelescope </li>
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public abstract class RendererFactory {
    
    /** The hashtable which stores all the renderers */
    private static Hashtable rendererList = new Hashtable(4);
    
    /** Stores an instance of the initial */
    protected static final ImageIcon INITIAL_MARK_ICON = ImageFactory.getImageBean(ImageKeys.SMALL_ICON_INITIAL).getImage();
    
    /**
     * Get the ImageVertexRenderer for the given vertex type
     * @param iconName name of the icon
     * @return      
     */
    public static ImageVertexRenderer getRenderer(String iconName){        
        ImageVertexRenderer renderer = (ImageVertexRenderer) rendererList.get(iconName);
        if(renderer == null){            
            ImageIcon imageIcon = ImageFactory.getImageBean(iconName).getImage();
            if(imageIcon == null){
                return null;
            }
            renderer = new ImageVertexRenderer(imageIcon);
            rendererList.put(iconName, renderer);
        }        
        return renderer;
    }
    
}
