/*
 * ScenarioGraphUI.java
 *
 * Created on 1 kwiecie� 2004, 16:47
 */

package pl.psnc.vlab.graph.ui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Paint;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.IOException;
import javax.imageio.ImageIO;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jgraph.plaf.basic.BasicGraphUI;
import pl.psnc.vlab.graph.model.ScenarioGraph;
import pl.psnc.vlab.ssa.util.ImageKeys;
import pl.psnc.vlab.util.swing.gui.ColorKeys;

/**
 * Overriden class BasicGrapUI - it adds method
 * for backgroung image display
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class ScenarioGraphUI extends BasicGraphUI {
    
    /** Logger instance */
    private Log log = LogFactory.getLog(this.getClass().getName() );
    
    
    private static double NINETY_DEGREES = Math.toRadians(90.0);
    
    private double x_div;
    private double y_div;
    
    
    /** Screen divider */
    private static double DIVIDER = 2.5;
    
    /** Screen margin: x*/
    private static int MAP_X_MARGIN = 0;
    
    private static BufferedImage original;
    private static BufferedImage rescaled;
    
    
    public ScenarioGraphUI(){
        try {
            original = ImageIO.read(getClass().getClassLoader().getResource(ImageKeys.WORLD_MAP));
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        rescaled = original;
        
    }
    
    public ScenarioGraph getScenarioGraph() {
        return (ScenarioGraph) graph;
        
    }
    
    
    
    
    protected void paintBackground(java.awt.Graphics g) {
        Dimension pageBounds = graph.getSize();
        Graphics2D g2d = (Graphics2D) g;
        Paint storedPaint = g2d.getPaint();
        g2d.setPaint(new GradientPaint(0, 0, ColorKeys.GRADIENT_LIGHT_GRAY_AND_WHITE[0], pageBounds.width, 0, ColorKeys.GRADIENT_LIGHT_GRAY_AND_WHITE[1]));
        g2d.fillRect(0, 0, pageBounds.width, pageBounds.height);
        g2d.setPaint(storedPaint);
        
        
        if (pageBounds.width != rescaled.getWidth() || pageBounds.height / DIVIDER != rescaled.getHeight() / DIVIDER) {
            int transparency = original.getColorModel().getTransparency();
            rescaled.flush();
            rescaled = rescale(original, pageBounds.width, (int) (pageBounds.height / DIVIDER), transparency);
            g.drawImage(rescaled, MAP_X_MARGIN, 0, null);
            paintSeparator(g);
        }
        
        paintLabels(g, "Vlbi", 0, rescaled.getHeight() / 2 );
        paintLabels(g, "Grid", 0, (int) ((pageBounds.getHeight() - rescaled.getHeight()) /2) +rescaled.getHeight() );
        
        
    }
    
    
    
    
    // -------------------------------------------------------------------------
    // ----- Helper methods
    
    
    /**
     * Painter method: paints graph labels
     * @param g
     */
    private void paintLabels(Graphics g, String text, int x, int y){
        Graphics2D g2d = (Graphics2D) g;
        FontRenderContext frc = g2d.getFontRenderContext();
        Font f = new Font("Helvetica",Font.BOLD, 24);
        TextLayout tl = new TextLayout(text, f, frc);
        Rectangle2D textBounds = tl.getBounds();
        
        y =  (int) (y - (textBounds.getHeight()));
        int fWidth  = (int) (tl.getAscent() + tl.getDescent());
        int fHeight = (int) textBounds.getWidth() +5;
        
        g2d.setPaint(new GradientPaint(x, y,
                GraphDecorator.SEPARATOR_COLOR1,
                x+(int) textBounds.getWidth(), y+tl.getDescent(),
                GraphDecorator.SEPARATOR_COLOR2));
        
        g2d.translate(x+fWidth,y+fHeight);
        
        g2d.rotate(-NINETY_DEGREES);
        tl.draw(g2d, 5, tl.getDescent());
        g2d.rotate(NINETY_DEGREES);
        g2d.translate(-(x+fWidth),-(y+fHeight));
    }
    
    
    /**
     * Painter method: paints graph separator
     * @param g
     */
    private void paintSeparator(Graphics g){
        Graphics2D g2d = (Graphics2D) g;
        Paint storedPaint = g2d.getPaint();
        g2d.setPaint(new GradientPaint(GraphDecorator.SEPARATOR_MARGIN,
                this.rescaled.getHeight() + GraphDecorator.SEPARATOR_MARGIN,
                GraphDecorator.SEPARATOR_COLOR1,
                this.rescaled.getWidth() - GraphDecorator.SEPARATOR_MARGIN,
                this.rescaled.getHeight() + GraphDecorator.SEPARATOR_MARGIN, //+GraphDecorator.SEPARATOR_HEIGTH,
                GraphDecorator.SEPARATOR_COLOR2));
        g2d.fillRect(GraphDecorator.SEPARATOR_MARGIN, this.rescaled.getHeight() + GraphDecorator.SEPARATOR_MARGIN,
                graph.getWidth() - (2 * GraphDecorator.SEPARATOR_MARGIN), GraphDecorator.SEPARATOR_HEIGTH);
        g2d.setPaint(storedPaint);
    }
    
    
    /**
     * Get new position
     * @param x
     * @param y
     * @return
     */
    public  Point2D rescale(int x, int y){
        return new Point( (int) rescaleX(x), (int) rescaleY(y));
    }
    
    
    /**
     * Get rescaled X position
     * @param x
     * @return
     */
    private double rescaleX(double x){
        Dimension pageBounds = graph.getSize();
        x_div = (double) pageBounds.width  / (double) original.getWidth();
        return x*x_div;
    }
    
    /**
     * Get rescaled Y position
     * @param y
     * @return
     */
    private double rescaleY(double y){
        Dimension pageBounds = graph.getSize();
        y_div = (double) pageBounds.height/ (double) original.getHeight() / DIVIDER;
        return y * y_div;
    }
    
    
    /**
     * Scales the image to fit the screen
     * @param original
     * @param width
     * @param height
     * @param transparency
     * @return
     */
    static private BufferedImage rescale(BufferedImage original, int width, int height, int transparency) {
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        GraphicsDevice gd = ge.getDefaultScreenDevice();
        GraphicsConfiguration gc = gd.getDefaultConfiguration();
        BufferedImage rescaled = gc.createCompatibleImage(width, height, transparency);
        Graphics2D g = rescaled.createGraphics();
        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
        g.drawImage(original, 0, 0, width , height, null);
        g.dispose();
        return rescaled;
    }
    
    
}
