package pl.psnc.vlab.graph.util;


import java.awt.Component;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.prefs.Preferences;
import javax.swing.JDialog;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

import javax.swing.filechooser.FileSystemView;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.graph.pad.format.GraphModelFileFormat;
import pl.psnc.vlab.graph.pad.GraphModelProviderRegistry;
import pl.psnc.vlab.ssa.util.AppMessage;
import pl.psnc.vlab.util.FileTools;
import pl.psnc.vlab.util.swing.gui.Message;


/**
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class DynScenFileChooser extends JFileChooser {
    
    /** Instance of logger*/
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    
    public static final String LAST_PATH = "LastPathPosition";
    
    public static final String LAST_FILE_FILTER = "LastFileFilter";
    
    /** Stores instance of JDialog */
    private JDialog dialog;
    private int returnValue = ERROR_OPTION;
    
    /**
     * Constructor for DynScenFileChooser.
     */
    public DynScenFileChooser() {
        super();
        //init();
    }
    
    /**
     * Constructor for DynScenFileChooser.
     * @param currentDirectoryPath
     */
    public DynScenFileChooser(String currentDirectoryPath) {
        super(currentDirectoryPath);
        init();
    }
    
    
    /**
     * Constructor for DynScenFileChooser.
     * @param currentDirectory
     */
    public DynScenFileChooser(File currentDirectory) {
        super(currentDirectory);
        init();
    }
    
    /**
     * Constructor for DynScenFileChooser.
     * @param fsv
     */
    public DynScenFileChooser(FileSystemView fsv) {
        super(fsv);
        init();
    }
    
    /**
     * Constructor for DynScenFileChooser.
     * @param currentDirectory
     * @param fsv
     */
    public DynScenFileChooser(File currentDirectory, FileSystemView fsv) {
        super(currentDirectory, fsv);
        init();
    }
    
    /**
     * Constructor for DynScenFileChooser.
     * @param currentDirectoryPath
     * @param fsv
     */
    public DynScenFileChooser(String currentDirectoryPath, FileSystemView fsv) {
        super(currentDirectoryPath, fsv);
        init();
    }
    
    /** Initializes the gui.
     *
     *  Adds a property change listener to catch
     *  changes at the file filter. If the user
     *  changes the file filter the listener
     *  changes the accessory for the correct file format.
     */
    protected void init() {
        
        // if the dialog was committed we
        // store the path position for the
        // next call
        addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (e.getActionCommand() == JFileChooser.APPROVE_SELECTION) {
                    try {
                        Preferences.userNodeForPackage(this.getClass()).put(LAST_PATH, DynScenFileChooser.this.getSelectedFile().getPath());
                    } catch (Exception ex) {
                        // ignore
                    }
                }
            }
        });
        
        addPropertyChangeListener(new PropertyChangeListener() {
            /**
             * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
             */
            public void propertyChange(PropertyChangeEvent evt) {
                if (evt != null && evt.getPropertyName() != null
                        && (evt.getPropertyName().equals(JFileChooser.FILE_FILTER_CHANGED_PROPERTY)
                        || evt.getPropertyName().equals( JFileChooser.DIALOG_TYPE_CHANGED_PROPERTY))) {
                    FileFilter ff = (FileFilter) getFileFilter();
                    GraphModelFileFormat gmff =
                            GraphModelProviderRegistry.getGraphModelFileFormat(ff);
                    if (gmff == null)
                        setAccessory(null);
                    else if (getDialogType() == SAVE_DIALOG)
                        setAccessory(gmff.getWriteAccessory());
                    else if (getDialogType() == OPEN_DIALOG)
                        setAccessory(gmff.getReadAccessory());
                    Component comp = DynScenFileChooser.this;
                    
                    while (!(comp instanceof Window)) {
                        if (comp == null)
                            return;
                        comp = comp.getParent();
                    }
                    if (comp != null)
                        ((Window) comp).pack();
                }
            }
        });
        
        GraphModelFileFormat[] formats;
        /*if (doc == null) {
            formats = GraphModelProviderRegistry.getGraphModelFileFormats();
        } else {
            formats = doc.getGraphModelProvider().getGraphModelFileFormats();
        }
         */
        // <added by osa>
        formats = GraphModelProviderRegistry.getGraphModelFileFormats();
        // </added by osa>
        
        for (int i = 0; i < formats.length; i++) {
            GraphModelFileFormat format = formats[i];
            addChoosableFileFilter(format.getFileFilter());
        }
        
        try {
            
            String fileName =  Preferences.userNodeForPackage(this.getClass()).get( LAST_PATH, System.getProperty("user.home"));
            try {
                setCurrentDirectory(new File(fileName));
            } catch (Exception ex) {
                // ignore
            }
            
            // set the last selected file filter
            try {
                GraphModelFileFormat format =
                        GraphModelProviderRegistry.getGraphModelFileFormat(fileName);
                this.setFileFilter(format.getFileFilter());
            } catch (Exception ex) {
                // ignore
            }
        } catch (Exception ex) {
            // ignore
        }
        
    }
         
    
    /**
     * Called by the UI when the user hits the Approve button
     * (labeled "Open" or "Save", by default). This can also be
     * called by the programmer.
     * This method causes an action event to fire
     * with the command string equal to
     * <code>APPROVE_SELECTION</code>.
     *
     * @see #APPROVE_SELECTION
     */
    public void approveSelection() {
        log.debug("<approveSelection>");
        returnValue = APPROVE_OPTION;
        File file = getSelectedFile();
        log.debug("File="+file);
        //check whether the file existst
        if( getDialogType() == SAVE_DIALOG && FileTools.fileExists(file) ){
            log.debug("save");
            int res = Message.showConfirmDialog(this, AppMessage.MSG_FILE_EXISTS,
                    AppMessage.MSG_FILE_EXISTS_TITLE, Message.YES_NO_OPTION, Message.MESSAGE_TYPE_INF);
            if(res == Message.NO_OPTION){
                DynScenFileChooserFactory.removeChooser(DynScenFileChooserFactory.TYPE_SAVE_DIALOG);
                fireActionPerformed(APPROVE_SELECTION);
                return;
            }
        } // end of file exists
        
        if(dialog != null) {
            dialog.setVisible(false);
        }
        fireActionPerformed(APPROVE_SELECTION);
        log.debug("</approveSelection>");
    }
    
    /**
     * Called by the UI when the user chooses the Cancel button.
     * This can also be called by the programmer.
     * This method causes an action event to fire
     * with the command string equal to
     * <code>CANCEL_SELECTION</code>.
     *
     * @see #CANCEL_SELECTION
     */
    public void cancelSelection() {
        returnValue = CANCEL_OPTION;
        if(dialog != null) {
            dialog.setVisible(false);
        }
        fireActionPerformed(CANCEL_SELECTION);
    }
    
    /**
     * Pops a custom file chooser dialog with a custom approve button.
     * For example, the following code
     * pops up a file chooser with a "Run Application" button
     * (instead of the normal "Save" or "Open" button):
     * <pre>
     * filechooser.showDialog(parentFrame, "Run Application");
     * </pre>
     *
     * Alternatively, the following code does the same thing:
     * <pre>
     *    JFileChooser chooser = new JFileChooser(null);
     *    chooser.setApproveButtonText("Run Application");
     *    chooser.showDialog(parentFrame, null);
     * </pre>
     *
     * <!--PENDING(jeff) - the following method should be added to the api:
     *      showDialog(Component parent);-->
     * <!--PENDING(kwalrath) - should specify modality and what
     *      "depends" means.-->
     *
     * <p>
     *
     * The <code>parent</code> argument determines two things:
     * the frame on which the open dialog depends and
     * the component whose position the look and feel
     * should consider when placing the dialog.  If the parent
     * is a <code>Frame</code> object (such as a <code>JFrame</code>)
     * then the dialog depends on the frame and
     * the look and feel positions the dialog
     * relative to the frame (for example, centered over the frame).
     * If the parent is a component, then the dialog
     * depends on the frame containing the component,
     * and is positioned relative to the component
     * (for example, centered over the component).
     * If the parent is <code>null</code>, then the dialog depends on
     * no visible window, and it's placed in a
     * look-and-feel-dependent position
     * such as the center of the screen.
     *
     * @param   parent  the parent component of the dialog;
     * 			can be <code>null</code>
     * @param   approveButtonText the text of the <code>ApproveButton</code>
     * @return  the return state of the file chooser on popdown:
     * <ul>
     * <li>JFileChooser.CANCEL_OPTION
     * <li>JFileChooser.APPROVE_OPTION
     * <li>JFileCHooser.ERROR_OPTION if an error occurs or the
     * 			dialog is dismissed
     * </ul>
     * @exception HeadlessException if GraphicsEnvironment.isHeadless()
     * returns true.
     * @see java.awt.GraphicsEnvironment#isHeadless
     */
    public int showDialog(Component parent, String approveButtonText) throws java.awt.HeadlessException {
        
        if(approveButtonText != null) {
            setApproveButtonText(approveButtonText);
            setDialogType(CUSTOM_DIALOG);
        }
        dialog = createDialog(parent);
        dialog.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                returnValue = CANCEL_OPTION;
            }
        });
        returnValue = ERROR_OPTION;
        rescanCurrentDirectory();
        
        dialog.show();
        //dialog.dispose();
        //dialog = null;
        return returnValue;
    }
    
    
    
    
}
