/*
 * FileChooserFactory.java
 *
 * Created on 11 maj 2004, 17:10
 */

package pl.psnc.vlab.graph.util;

import java.util.Hashtable;
import javax.swing.JFileChooser;

/**
 * The abstract factory for the file chooser dialogs
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class DynScenFileChooserFactory {
    
    /** The Open file chooser dialog */
    public static final String TYPE_OPEN_DIALOG = "openDialog";
    
    /** The Save file chooser dialog */
    public static final String TYPE_SAVE_DIALOG = "saveDialog";
    
    /** The choosers list */
    private static Hashtable chooserList = new Hashtable(2);
    
    /** The choosers list */
    private static GXLFileFilter gxlFileFilter = new GXLFileFilter();
    
    private static String activeFileName;
    
    /** Gets the given type of the file chooser
     * @param type the type of the chooser
     * @return
     */
    public static JFileChooser getFileChooser(String type){
        if( chooserList == null){
            return null;
        }
        JFileChooser chooser = (JFileChooser) chooserList.get(type);
        if(chooser == null){
            chooser = new DynScenFileChooser();
            chooser.setFileFilter( gxlFileFilter );
            chooserList.put(type, chooser);              
        }
        return chooser;
    }
    
    
    /**
     * Removes the file chooser from the list 
     * @param type 
     */
    public static void removeChooser(String type){
        if( chooserList == null){
            return;
        }
        chooserList.remove(type);
        activeFileName = null;
    }
    
    
    /**
     * Return the selected file name. If no files has been selected 
     * returns null;
     * @return selected file
     */
    public static String getSelectedFileName() {
        return activeFileName;
    }
    
    /**
     * Return the selected file name. If no files has been selected 
     * returns null;
     * @return selected file
     */
    public static void setSelectedFileName(String fileName) {
        activeFileName = fileName;
    }
}
