/*
 * JPopupHelper.java
 *
 * Created on 31 maj 2004, 09:31
 */

package pl.psnc.vlab.graph.util.gui;
import java.awt.Color;
import java.awt.Image;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.GraphConstants;
import pl.psnc.vlab.exception.VlabException;
import pl.psnc.vlab.graph.model.ScenarioCellObject;
import pl.psnc.vlab.graph.model.ScenarioGraph;
import pl.psnc.vlab.graph.util.GraphHelper;
import pl.psnc.vlab.graph.util.GraphKeys;
import pl.psnc.vlab.graph.vertex.type.ComputeType;
import pl.psnc.vlab.graph.vertex.type.ExperimentType;
import pl.psnc.vlab.graph.vertex.type.VLabVertexType;
import pl.psnc.vlab.rds.ResourceDocument;
import pl.psnc.vlab.ssa.actions.ActionFactory;
import pl.psnc.vlab.ssa.actions.SetNodeAsInitialAction;
import pl.psnc.vlab.ssa.gui.MainFrame;
import pl.psnc.vlab.ssa.gui.resource.JCorrelatorPropertiesTemplate;
import pl.psnc.vlab.ssa.gui.resource.JFileServerPropertiesTemplate;
import pl.psnc.vlab.ssa.gui.resource.JRadioPropertiesTemplate;
import pl.psnc.vlab.ssa.gui.resource.PropertyDialogFactory;
import pl.psnc.vlab.ssa.gui.resource.pane.JResourcePropertyDialog;
import pl.psnc.vlab.util.image.ImageFactory;
import pl.psnc.vlab.ssa.util.ImageKeys;
import pl.psnc.vlab.util.swing.gui.GuiTools;
import pl.psnc.vlab.util.swing.gui.Message;
import pl.psnc.vlab.util.swing.thread.ProgressIndicator;
import pl.psnc.vlab.xml.ResourceFactory;


/**
 * The JPopup Helper class is used to dynamically build a popuop menu for the
 * given jgraph cell
 * @author  Dominik Stokłosa
 * @email  osa@man.poznan.pl
 */
public class JPopupHelper {
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Instance of logger */
    private static final String PROPERTIES_ACTION = "Properties";
    
    /** Contains an instance of Action listener */
    private ActionListener listener = null;
    
    /** The type of the selected cell */
    private VLabVertexType cellType = null;
    
    /** The cell user object */
    private ScenarioCellObject cellObject = null;
    
    /** Holds the value of the JGraph object */
    private ScenarioGraph graph = null;
    
    /** The point */
    private static Point2D point = new AttributeMap().createPoint(10,10);
    
    /** Creates a new instance of JPopupHelper
     * @param listener Action listener
     * @param cell The selected vertex
     */
    public JPopupHelper(ActionListener listener, ScenarioGraph graph,  Object cell, Point2D point ) {
        this.listener = listener;
        this.graph = graph;
        this.point = point;
        //get the type object
        if(cell == null){
            return;
        }
        DefaultGraphCell defaultGraphCell = (DefaultGraphCell) cell;
        cellObject = GraphHelper.getCellObject(cell);
        cellType = cellObject.getType();
    }
    
    
    
    /** Get the node type item
     * @return JMenuItem which represents the node type
     */
    public JComponent getNodeTypeMenuItem(){
        log.debug("<getNodeTypeMenu>");
        StringBuffer node = new StringBuffer(ResourceBundle.getBundle("bundle/jgraph").getString("popup.item.node"))
        .append(": ")
        .append(cellType.getGenericTypeName());
        JLabel nodeType = new JLabel(node.toString());
        nodeType.setFont(GraphKeys.FONT_NODE_TYPE);
        nodeType.setForeground(Color.BLACK);
        nodeType.setIconTextGap(4);
        nodeType.setIcon( GraphHelper.getResourceIcon(cellType.getTypeId()).getImage()) ;
        log.debug("</getNodeTypeMenu>");
        return nodeType;
    }
    
    
    
    /** Get the node type item
     * @return
     */
    public JMenuItem getNodePropertyMenuItem(){
        log.debug("<getNodePropertyMenuItem: "+cellType.getTypeName()+">");
        if(!cellType.isGenericType()){
            JMenuItem prop = new JMenuItem(java.util.ResourceBundle.getBundle("bundle/jgraph").getString("popup.item.properties"));
            prop.setName(PROPERTIES_ACTION);
            prop.setIcon( ImageFactory.getImageBean(ImageKeys.ICON_PROPERTIES).getImage() );
            prop.addActionListener(listener);
            log.debug("</getNodePropertyMenuItem>");
            return prop;
        }
        log.debug("</getNodePropertyMenuItem: NULL>");
        return null;
    }
    
    
    /** Gets the resource list for the given Node type and
     * assigns an action to each resource item.
     * @return
     * @param cell the selected cell object
     */
    public JMenu getResourceMenu(){
        log.debug("<getResourceMenu>");
        JMenu resourceMenu = new JMenu(java.util.ResourceBundle.getBundle("bundle/jgraph").getString("popup.item.resource.list"));
        resourceMenu.setIcon(ImageFactory.getImageBean(ImageKeys.ICON_RESOURCES).getImage());
        
        //get resource list
        ArrayList resourceList = ResourceFactory.getResourceList(cellType);
        if(resourceList == null){
            //no resources found
            JLabel item = new JLabel(java.util.ResourceBundle.getBundle("bundle/jgraph").getString("popup.item.empty.list"));
            item.setName("none");
            resourceMenu.add(item);
            log.debug("</getResourceMenu>");
            return resourceMenu;
        }
        
        Iterator it = resourceList.iterator();
        while(it.hasNext()){
            final ResourceDocument.Resource resVal = (ResourceDocument.Resource) it.next();
            JMenuItem item = getNextResource(resVal);
            resourceMenu.add(item);
            item.addActionListener(listener);
            //add separator
            if(it.hasNext()){
                resourceMenu.addSeparator();
            }
        }
        log.debug("</getResourceMenu>");
        return resourceMenu;
    }
    
    
    /**
     * Create next resource as a JMenuItem
     * @param resVal new resource as a JMenuItem
     * @return
     */
    private JMenuItem getNextResource(final ResourceDocument.Resource resVal){
        JMenuItem item = new JMenuItem(resVal.getName());
        item.setName(String.valueOf(resVal.getId()) );
        item.setIcon(GraphHelper.getResourceIcon(resVal.getTypeId()).getImage());
        
        //check whether the resource is selected
        Object selectedProperty = cellObject.getProperty(GraphKeys.PROPERTY_SELECTED_NODE_ID);
        int selectedId = selectedProperty == null ? -1 : Integer.parseInt( (String) selectedProperty);
        if(selectedId == resVal.getId()){
            item.setIcon(ImageFactory.getImageBean(ImageKeys.ICON_OK).getImage() );
        }
        return item;
    }
    
    /**
     * Get the edit menu items : copy, past, delete ...
     * @param menu
     * @return
     */
    public JPopupMenu getEditMenu(JPopupMenu menu){
        log.debug("<getEditMenu>");
        //Edit menu
        JMenuItem item = new JMenuItem();
        GuiTools.setAction(item, ActionFactory.getAction(ActionFactory.ACTION_COPY), false, true);
        menu.add(item);
        //Paste
        item = new JMenuItem();
        GuiTools.setAction(item, ActionFactory.getAction(ActionFactory.ACTION_PASTE), false, true);
        menu.add(item);
        
        //delete
        item = new JMenuItem();
        GuiTools.setAction(item, ActionFactory.getAction(ActionFactory.ACTION_DELETE), false, true);
        menu.add(item);
        
        //add separator
        menu.addSeparator();
        
        //undo
        item = new JMenuItem();
        GuiTools.setAction(item, ActionFactory.getAction(ActionFactory.ACTION_UNDO), false, true);
        menu.add(item);
        
        //redo
        item = new JMenuItem();
        GuiTools.setAction(item, ActionFactory.getAction(ActionFactory.ACTION_REDO), false, true);
        menu.add(item);
        
        //add separator
        menu.addSeparator();
        log.debug("</getEditMenu>");
        return menu;
    }
    
    
    /**
     * Get the menu for adding new vertex
     * @return
     */
    public JMenu getAddNodeMenu(){
        log.debug("<getAddNodeMenu>");
        JMenu addNodeMenu = new JMenu(ResourceBundle.getBundle("bundle/jgraph").getString("popup.item.add.node"));
        //add a compute vertex
        /*JMenuItem item = new JMenuItem();
        item.setAction(ActionFactory.getAction(ActionFactory.ACTION_ADD_VERTEX_EXPERIMENT));
        addNodeMenu.add(item);
         
        //add separator
        addNodeMenu.addSeparator();
         */
        //add an experiment
        JMenuItem item = new JMenuItem();
        item.setAction(ActionFactory.getAction(ActionFactory.ACTION_ADD_VERTEX_COMPUTE));
        addNodeMenu.add(item);
        log.debug("</getAddNodeMenu>");
        return addNodeMenu;
    }
    
    
    /**
     * Get the menu for setting the root elements
     * @return
     */
    public JMenuItem getMarkInitialNodeMenu(){
        log.debug("<getIsInitialNodeMenu>");
        JMenuItem isInitialMenuItem = new JMenuItem();
        boolean selected = this.cellObject.isMarkedInitial();
        log.debug("Selected:"+selected);
        isInitialMenuItem.setSelected(selected);
        isInitialMenuItem.setAction(new SetNodeAsInitialAction(selected));
        isInitialMenuItem.getAction().putValue(SetNodeAsInitialAction.NODE_CELL_OBJECT, this.cellObject);
        log.debug("</getIsInitialNodeMenu>");
        return isInitialMenuItem;
    }
    
    
    
    /**
     * Inner class performs the long time operation
     */
    private class HandleWorker implements Runnable {
        
        /** Holds instance of parent class */
        private ProgressIndicator progress;
        
        HandleWorker(ProgressIndicator progress){
            this.progress = progress;
        }
        
        public void run(){
            try{
                log.debug("Worker thread started");
                //create dynamic property dialog
                ResourceDocument.Resource resource = cellObject.getResource();
                if(resource == null){
                    progress.fireFinish();
                    return;
                }
                final JFrame tmpFrame = MainFrame.getInstance();
                
                Image mainIcon = tmpFrame.getIconImage();
                tmpFrame.setIconImage(GraphHelper.getResourceIcon(resource.getTypeId()).getImage().getImage() ) ;
                JDialog propertyDialog = null;
                if( resource.getTypeId() == 4) {
                    propertyDialog = new JRadioPropertiesTemplate(tmpFrame);
                }else if(resource.getTypeId() == 5){
                    propertyDialog = new JCorrelatorPropertiesTemplate(tmpFrame);
                } else if( resource.getTypeId() == 6) {
                    propertyDialog = new JFileServerPropertiesTemplate(tmpFrame);
                } else {
                    propertyDialog = (JResourcePropertyDialog) PropertyDialogFactory.getResourcePropertyDialog(tmpFrame, true, resource);
                }
                
                //sleeps
                //simulateLongDelay(5 * 1000);
                
                progress.fireFinish();
                
                propertyDialog.show();
                
                //get the node resource and save it
                //cellObject.setResource(propertyDialog.getResource());
                MainFrame.getInstance().setIconImage(mainIcon);
                log.debug("Worker thread done");
            }catch(Exception err){
                err.printStackTrace();
                //tbd exception handling
            }
        }
        
    }
    
    
    private void simulateLongDelay(int delay){
        try{
            log.debug("Start sleeping for "+delay/1000  +" seconds.");
            Thread.sleep(delay);
            log.debug("Waking up!!!");
        } catch(InterruptedException err){
            err.printStackTrace();
        }
        
        
    }
    
    /**
     * Handle an action event from the JGraph
     * @param actionName action to be fired
     * @return new vertex type or null if the vertex type was not changed
     */
    public Object handleAction(String actionName) throws VlabException{
        log.debug("<handleAction:"+actionName+">");
        
        //an action for the Properties
        if(actionName.equals(PROPERTIES_ACTION) ){
            ProgressIndicator progress = new ProgressIndicator(Locale.getDefault());
            Thread worker = new Thread(new HandleWorker(progress));
            worker.setDaemon(true);
            worker.start();
            return null;
        }
        
        //get the resource by id
        ResourceDocument.Resource resource =  ResourceFactory.getResourceById( new Integer(actionName).intValue() );
        if(resource == null){
            log.debug("</handleAction: resource not found. Id="+actionName+">");
            return null;
        }
        
        
        VLabVertexType newType = null;
        String imageName = GraphHelper.getResourceImage(resource.getTypeId()).getImageName();
        int parentTypeId = cellType.getGenericTypeId();
        log.debug("parentTypeId="+parentTypeId);
        if(parentTypeId == VLabVertexType.GENERIC_TYPE_COMPUTE){
            newType = new ComputeType(resource.getTypeId());
        } else if(parentTypeId == VLabVertexType.GENERIC_TYPE_EXPERIMENT){
            newType = new ExperimentType(resource.getTypeId());
        }else {
            log.debug("</handleAction: no type found");
            return null;
        }
        
        //set the node identyfier
        resource.setNodeId(ResourceFactory.getNextResourceId());
        log.debug("add:New node id-"+resource.getNodeId());
        //set selected resource id
        cellObject.putProperty(GraphKeys.PROPERTY_SELECTED_NODE_ID, actionName);
        //set the resource described by this cell
        cellObject.setResource(resource);
        //set new type
        cellObject.setType(newType);
        log.debug("</handleAction: new subType="+newType.getTypeName()+">");
        //set label
        cellObject.putProperty(ScenarioCellObject.keyValue, resource.getName());
        // removed 10.1 Map map = graph.getModel().createAttributes();
        Map map = new AttributeMap();
        GraphConstants.setIcon(map, GraphHelper.getResourceImage(newType.getTypeId()).getImage());
        graph.setSelectionAttributes((AttributeMap) map);
        
        //check whther there are initial nodes defined
        int verSize = GraphHelper.countConcreteVerticles(graph) -1;
        if( !GraphHelper.hasInitialNodes(graph) && verSize == 0){
            log.debug("Setting default initial mark");
            cellObject.setMarkedInitial(true);
        } else {
            log.debug("No default initial mark: "+verSize);
        }
        
        return newType;
    }
    
    /**
     * Handle an action event from the JGraph
     * @param actionName
     * @return
     */
    public Object doubleClickHandler(){
        log.debug("<doubleClickHandler>");
        if(cellType.isGenericType()){
            log.debug("generic type");
            JPopupMenu resourceList = new JPopupMenu();
            resourceList.add(getResourceMenu());
            return resourceList;
        } else {
            try{
                //get the resource object
                handleAction(PROPERTIES_ACTION);
            }catch(VlabException e){
                log.error("Error occured: "+e.getMessage());
                Message.showInfoMessage(MainFrame.getInstance(), e.getLocalizedMessage(), Message.MESSAGE_TYPE_ERROR);
                e.printStackTrace();
                //tbd ignored
            }
        }
        log.debug("</doubleClickHandler>");
        return null;
    }
    
    
    /**
     * Get the last point of the JPopup menu occureance
     * @return
     */
    public static Point2D getCurrentMenuPoint(){
        return point;
    }
    /**
     * Set the last point of the JPopup menu occureance
     * @return
     */
    public static void setCurrentMenuPoint(Point2D newPoint){
        point = newPoint;
    }
    
    
}

