/*
 * VLabVertexType.java
 *
 * Created on 14 maj 2004, 11:33
 */

package pl.psnc.vlab.graph.vertex.type;

import java.io.Serializable;
import javax.swing.ImageIcon;
import pl.psnc.vlab.tds.TypeDocument;
import pl.psnc.vlab.util.image.ImageFactory;
import pl.psnc.vlab.xml.TypeFactory;


/**
 * The Node generic type. This is the abstract class, which defines the context
 * for the specyfic types available in the VLab System
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public abstract class VLabVertexType implements Serializable{
    
    //--------------------------------------
    //-- The list of defined types
    /** Generic experiment type  */
    public static int GENERIC_TYPE_EXPERIMENT = 1;
    /** Generic compute device type  */
    public static int GENERIC_TYPE_COMPUTE    = 2;
    
    /**
     * Holds value of property typeId.
     */
    private int typeId;    
    
    /** Determines whether the given cell
     * is of Generic Type
     * @return
     */
    public boolean isGenericType(){
        TypeDocument.Type type = TypeFactory.getResourceType(getTypeId());
        return type.getGenericTypeId() == 0;        
    }
                    
    
    /** Returns the name of this type 
     * @return
     */
    public TypeDocument.Type getType(){
        return TypeFactory.getResourceType(getTypeId());        
    }
     
    
    /** Returns the type name
     * @return
     */
    public String getTypeName(){
        TypeDocument.Type type = TypeFactory.getResourceType(getTypeId());                
        return type.getName();        
    }
    
    /** Returns the name of the parent type 
     * @return
     */
    public String getGenericTypeName(){
        TypeDocument.Type type = TypeFactory.getResourceType(getTypeId());
        if(type.getGenericTypeId() == 0){
            return type.getName();
        }        
        return TypeFactory.getResourceType(type.getGenericTypeId()).getName();        
    }
    
    
    /** Returns the id of the parent type 
     * @return
     */
    public int getGenericTypeId(){
        int res = isGenericType() ? getTypeId() : getType().getGenericTypeId();
        return res;
    }
    
    /**
     * Getter for property type.
     * @return Value of property type.
     */
    public int getTypeId() {
        return this.typeId;
    }
    
    /**
     * Setter for property type.
     * @param type New value of property type.
     */
    public void setTypeId(int typeId) {
        this.typeId = typeId;
    }    
}
