/*
 * CustomPortView.java
 *
 * Created on 8 listopad 2006, 16:13
 */

package pl.psnc.vlab.graph.view;

import java.awt.Component;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jgraph.JGraph;
import org.jgraph.graph.CellView;
import org.jgraph.graph.CellViewRenderer;
import org.jgraph.graph.PortView;
import pl.psnc.vlab.graph.renderer.CustomPortRenderer;

/**
 *
 * @author <a href="mailto:osa@man.poznan.pl">Dominik Stoklosa (~osa~)</a>
 * @email  osa@man.poznan.pl
 */
public class CustomPortView extends PortView {
    
    /** Holds logger instance */
    private Log log = LogFactory.getLog(getClass().getName());
    
    //private static transient int SIZE = 16;
    
    /** Holds instance of a renderer for this class */
    private static CellViewRenderer renderer = new CustomPortRenderer();
  
    
    /**
     * Creates a new instance of CustomPortView
     * @param userObject user object
     */
    public CustomPortView(Object userObject) {
        super(userObject);
        SIZE = (int) ( (CustomPortRenderer) getRenderer()).getIconSize().getWidth();
    }
    
    
    
    /**
     * Return a renderer for this class
     * @return renderer instance for this view
     */
    public CellViewRenderer getRenderer() {
        return renderer;
    }
    
    public Rectangle2D getBounds() {
        Point2D loc = getLocation();
        double x = 0;
        double y = 0;
        if (loc != null) {
            x = loc.getX();
            y = loc.getY();            
        }
        Rectangle2D bounds = new Rectangle2D.Double(x, y, 0, 0);        
        bounds.setFrame(bounds.getX() - SIZE/2, bounds.getY() - SIZE/2, SIZE, SIZE);
        return bounds;
    }
    
    /**
     * Get Renderer Component for this class
     *
     * @return component renderer
     * @param graph
     * @param cellView
     * @param selected
     * @param focus
     * @param preview
     */
    public Component getRendererComponent(JGraph graph, boolean selected, boolean focus, boolean preview) {
        //log.debug("Port VIEW -> foces="+focus+" sel="+selected +" prew="+preview);
        return super.getRendererComponent(graph, selected, focus, preview );
    }

    /**
     * Holds value of property visible.
     */
    private boolean visible;

    /**
     * Getter for property visible.
     * @return Value of property visible.
     */
    public boolean isVisible() {
        return this.visible;
    }

    /**
     * Setter for property visible.
     * @param visible New value of property visible.
     */
    public void setVisible(boolean visible) {
        this.visible = visible;
    }
 
    
    
    
    
    
}
