package pl.psnc.vlab.ssa.actions;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jgraph.graph.GraphLayoutCache;
import pl.psnc.vlab.graph.ScenarioGraphEditor;
import pl.psnc.vlab.graph.ScenarioGraphEditorFactory;
import pl.psnc.vlab.graph.model.ScenarioGraph;
import pl.psnc.vlab.ssa.exception.SSAException;
import pl.psnc.vlab.util.FileTools;




/**
 * The class is an parent for every system action.
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public abstract class AbstractActionDefault extends AbstractAction {
    
    /** Instance of logger*/
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** The parent object of this action */
    public final static String PARENT_OBJECT = "parentObject";
    
    
    /**
     * Define an Action Object with a default name (name of the class) and icon.
     */
    public AbstractActionDefault() {
        // build the name for this action
        putValue(Action.NAME, FileTools.getClassNameWithoutPackage(getClass()));
    }
    
    
    /**
     * Define an Action Object with a specyfied action name and icon.
     * @param name name of the action
     * @param icon the icon for the current action
     */
    public AbstractActionDefault(String name, Icon icon) {
        super(name, icon);
    }
    
    
    /**
     * Returns the name of the action
     * @return
     */
    public String getName() {
        return (String) getValue(NAME);
    }
    
    /**
     * Return an instance of the current Scenario
     * Graph for the current Greaph Editor.
     * @return
     */
    public ScenarioGraph getCurrentGraph() {
        try{
            return ScenarioGraphEditorFactory.getGraphEditorInstance().getGraph();
        }catch(SSAException dynScenErr){
            dynScenErr.printStackTrace();
            log.error("Error:"+dynScenErr.getMessage());
            return null;
        }
    }
    
    /**
     * Get the current Graph Layout Cache
     * @return
     */
    public GraphLayoutCache getCurrentGraphLayoutCache() {
        ScenarioGraph graph = getCurrentGraph();
        if(graph != null){
            return graph.getGraphLayoutCache();
        } else {
            return null;
        }
    }
             
    
}
