/*
 * ImageFactory.java
 *
 * Created on 14 maj 2004, 15:29
 */

package pl.psnc.vlab.ssa.actions;

import java.util.Hashtable;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * The Action Factory store the system actions. The action
 * search methods are also available
 *
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public abstract class ActionFactory {
    
    /** Instance of logger*/
    private static Log log = LogFactory.getLog("ImageFactory");
    
    //--------------------------------------------------------------------------
    //----    Actions defined in the system
    
    /** The vertex copy action */
    public static String ACTION_COPY        = "copy";
    /** The vertex paste action */
    public static String ACTION_PASTE       = "paste";
    /** The vertex delete action */
    public static String ACTION_DELETE      = "delete";
    
    /** The undo action*/
    public static String ACTION_UNDO        = "undo";
    /** The redo action*/
    public static String ACTION_REDO        = "redo";
    
    /** The zoom in action*/
    public static String ACTION_ZOOM_IN     = "zoomIn";
    /** The zoom out action*/
    public static String ACTION_ZOOM_OUT    = "zoomOut";
    /** The actual size action*/
    public static String ACTION_ZOOM_ACTUAL = "zoomActualSize";
    
    /** The open file action */
    public static String ACTION_OPEN_FILE   = "openFile";
    
    /** The save file action */
    public static String ACTION_SAVE_FILE   = "saveFile";
    /** The save as action */
    public static String ACTION_SAVE_FILE_AS  = "saveFileAs";
        
    /** The new file action */
    public static String ACTION_NEW_SCENARIO = "newScenario";
    
    /** Close scenario action */
    public static String ACTION_CLOSE_SCENARIO = "closeScenario";
    
    
    /** The app exit action */
    public static String ACTION_APP_EXIT     = "appExit";
    
    
    /** Add new experiment vertex action */
    public static String ACTION_ADD_VERTEX_EXPERIMENT = "addExperimentNode";
    /** Add new compute vertex action */
    public static String ACTION_ADD_VERTEX_COMPUTE    = "addComputeNode";
    /** Mark the current node as initial */
    public static String ACTION_MARK_NODE_INITIAL = "markNodeInitial";
    /** Submit scenario task */
    public static String ACTION_SUBMIT_SCENARIO = "submitDynamicScenario";
    
    /** Scenario Properties Dialog action */
    public static String ACTION_SCENARIO_PROPERTIES = "scenarioPropertiesDialog";
    
    
    //--------------------------------------------------------------------------
    //-- Perspectives
    /** Resource Outline action */
    public static String ACTION_RESOURCE_OUTLINE = "resourceOutline";
    
    /** Resource Outline action */
    public static String ACTION_MESSAGE_LOG_OUTLINE = "msgLogOutline";
    
    /** Available actions Outline action */
    public static String ACTION_AVAILABLE_ACTIONS_OUTLINE = "availableActionsOutline";
    
    /** Welcome Screen Action */
    public static String ACTION_WELCOME_SCREEN = "welcomeScreen";
    
    
    /** The hashtable with all the registered images */
    private static Hashtable actionList = new Hashtable(10);
    
    
    //--------------------------------------------------------------------------
    //----    The action initializer block
    static {
        //initializ copy action
        actionList.put(ACTION_COPY , new CopyAction() );
        //initialize delete action
        actionList.put(ACTION_DELETE, new DeleteAction());
        //initialize paste action
        actionList.put(ACTION_PASTE, new PasteAction());
        //initialize add new experiment action
        actionList.put(ACTION_ADD_VERTEX_EXPERIMENT, new AddExperimentNodeAction());
        //initialize add new experiment action
        actionList.put(ACTION_ADD_VERTEX_COMPUTE, new AddComputeNodeAction());
        //initialize mark as initial
        actionList.put(ACTION_MARK_NODE_INITIAL, new SetNodeAsInitialAction(false));
        //submit scenario action
        actionList.put(ACTION_SUBMIT_SCENARIO, new SubmitScenarioAction());
        
        //undo action
        actionList.put(ACTION_UNDO, new UndoAction());
        //redo action
        actionList.put(ACTION_REDO, new RedoAction());
        
        //zoom in action
        actionList.put(ACTION_ZOOM_IN, new ZoomInAction());
        //zoom out action
        actionList.put(ACTION_ZOOM_OUT, new ZoomOutAction());
        //zoom actual size action
        actionList.put(ACTION_ZOOM_ACTUAL, new ZoomActualAction());
        
        //add properties dialog action
        actionList.put(ACTION_SCENARIO_PROPERTIES, new ScenarioPropertiesAction());
        
        
    } //end of static initializer
    
    
    
    /** Get the Action object by name
     * @param actionName name of the action to retreive
     * @return
     */
    public static AbstractActionDefault getAction(String actionName){
        try{
            AbstractActionDefault action = (AbstractActionDefault)  actionList.get(actionName);
            return action;
        }catch(Exception e){
            e.printStackTrace();
            log.error("Bad");
            return null;
        }
    }
    
    
    /**
     * Put an action on the action list
     * @param actionName
     * @param action
     */
    public  static void putAction(String actionName, Object action){
        actionList.put(actionName, action);
    }
    
}

