package pl.psnc.vlab.ssa.actions;

import java.awt.event.ActionEvent;
import java.awt.geom.Point2D;
import java.util.ResourceBundle;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.KeyStroke;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.GraphConstants;
import pl.psnc.vlab.graph.ScenarioGraphEditorFactory;
import pl.psnc.vlab.graph.model.GraphCellFactory;
import pl.psnc.vlab.graph.model.ScenarioCellObject;
import pl.psnc.vlab.graph.model.ScenarioGraph;
import pl.psnc.vlab.graph.util.GraphHelper;
import pl.psnc.vlab.graph.util.GraphKeys;
import pl.psnc.vlab.graph.util.gui.JPopupHelper;
import pl.psnc.vlab.graph.vertex.type.ExperimentType;
import pl.psnc.vlab.graph.vertex.type.VLabVertexType;
import pl.psnc.vlab.ssa.exception.SSAException;
import pl.psnc.vlab.ssa.util.ImageKeys;
import pl.psnc.vlab.ssa.util.i18n.SSABundleFactory;
import pl.psnc.vlab.util.FormTools;
import pl.psnc.vlab.util.image.ImageFactory;



/**
 * Represents the action of adding new Experiment Node to the Graph
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class AddExperimentNodeAction extends AbstractActionDefault {
    
    /** Instance of logger*/
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /**
     * Creates a new instance of Add Experiment Node ction.
     * The action is created with the default name, icons,
     * mnemonic and accelerator key - these are taken from the
     * resource bundle
     */
    public AddExperimentNodeAction(){
        super(SSABundleFactory.getValue("action.add.vertex.experiment.name") , ImageFactory.getImageBean(ImageKeys.DEAFULT_EXPERIMENT_BIG).getImage() );
        this.putValue(Action.SHORT_DESCRIPTION, SSABundleFactory.getValue("action.add.vertex.experiment.tip"));
        this.putValue(Action.SMALL_ICON, ImageFactory.getImageBean(ImageKeys.DEAFULT_EXPERIMENT_ICON).getImage());
        String mnemonic = SSABundleFactory.getValue("action.add.vertex.experiment.mnemonic");
        KeyStroke keyStroke = KeyStroke.getKeyStroke(FormTools.toUpperLetterAndDigit(mnemonic));
        if(keyStroke != null){
            this.putValue(Action.MNEMONIC_KEY, new Integer(keyStroke.getKeyCode()) );
        }
        this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(SSABundleFactory.getValue("action.add.vertex.experiment.shortcut")));
    }
    
    /**
     * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        log.debug("Adding experiment vertex");
        try{
            VLabVertexType vertexType = new ExperimentType();
            DefaultGraphCell imageGraphCell =  GraphCellFactory.createCell(vertexType, null);
           /* ScenarioGraph graph = getCurrentGraph();
            if(!GraphHelper.hasInitialNodes(graph)){
                //no initial nodes found. Set node as initial
                ((ScenarioCellObject) imageGraphCell.getUserObject()).setMarkedInitial(true);
            }*/
            if(e.getSource() instanceof JButton){
                Point2D point = JPopupHelper.getCurrentMenuPoint();
                AttributeMap map = new AttributeMap();
                JPopupHelper.setCurrentMenuPoint(map.createPoint(point.getX() + GraphKeys.NEW_VERTEX_OFFSET , point.getY()+ GraphKeys.NEW_VERTEX_OFFSET ));
            }                       
            ScenarioGraphEditorFactory.getGraphEditorInstance().insert( JPopupHelper.getCurrentMenuPoint() , imageGraphCell);
        }catch(SSAException dynScenErr){
            dynScenErr.printStackTrace();
            log.error("Error"+dynScenErr.getMessage());
        }
        
    }
}
