package pl.psnc.vlab.ssa.actions;

import java.awt.event.ActionEvent;
import javax.swing.Action;
import javax.swing.KeyStroke;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.exception.VlabException;
import pl.psnc.vlab.graph.model.ScenarioGraph;
import pl.psnc.vlab.management.session.SessionFactory;
import pl.psnc.vlab.ssa.gui.MainFrame;
import pl.psnc.vlab.ssa.util.AppMessage;
import pl.psnc.vlab.ssa.util.i18n.MessageBundleFactory;
import pl.psnc.vlab.ssa.util.i18n.SSABundleFactory;
import pl.psnc.vlab.util.FormTools;
import pl.psnc.vlab.util.swing.gui.Message;




/**
 *
 * Close application action
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class CloseAppAction extends AbstractActionDefault {
    
    /** Instance of logger*/
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Instance of MainFrame*/
    private MainFrame mainFrame;
    
    /**
     * Creates a new instance of Exit Action.  The action is created with the default
     * name, icons, mnemonic and accelerator key - these are taken from the
     * resource bundle
     */
    public CloseAppAction(MainFrame mainFrame){
        super(SSABundleFactory.getValue("action.exit.name") , null );
        this.mainFrame = mainFrame;
        this.putValue(Action.SHORT_DESCRIPTION, SSABundleFactory.getValue("action.exit.tip"));
        String mnemonic = SSABundleFactory.getValue("action.exit.mnemonic");
        KeyStroke keyStroke = KeyStroke.getKeyStroke(FormTools.toUpperLetterAndDigit(mnemonic));
        this.putValue(Action.MNEMONIC_KEY, new Integer(keyStroke.getKeyCode()) );
        this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(SSABundleFactory.getValue("action.exit.shortcut")));
    }
    
    /**
     * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        try{
            //check wheter the current graph is empty
            ScenarioGraph graph = getCurrentGraph();
            int result = 0;
            if(graph != null && !graph.isEmpty()
            && ActionFactory.getAction(ActionFactory.ACTION_SAVE_FILE).isEnabled() ){
                log.debug("An old secenario exists");
                result = Message.showConfirmDialog(mainFrame, MessageBundleFactory.getValue("message.another.scenario.in.use"),
                        MessageBundleFactory.getValue("message.another.scenario.in.use.title"),
                        Message.YES_NO_OPTION, Message.MESSAGE_TYPE_INF);
                log.debug("result="+result);
                if(result != Message.YES_OPTION){
                    log.debug("Do not close scenario");
                    return;
                }
            }
            //SessionFactory.removeSession();
            System.exit(0);
        }catch(Exception err){
            err.printStackTrace();
            Message.showInfoMessage(mainFrame, AppMessage.MSG_ERR_INVALID_CLOSING, Message.MESSAGE_TYPE_ERROR);
            System.exit(0);
        }
        
    }
    
    
}




