package pl.psnc.vlab.ssa.actions;

import java.awt.event.ActionEvent;
import javax.swing.Action;
import javax.swing.KeyStroke;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.graph.model.ScenarioGraph;
import pl.psnc.vlab.graph.util.DynScenFileChooserFactory;
import pl.psnc.vlab.ssa.event.EventLogFactory;
import pl.psnc.vlab.ssa.exception.SSAException;
import pl.psnc.vlab.ssa.gui.MainFrame;
import pl.psnc.vlab.ssa.util.ImageKeys;
import pl.psnc.vlab.ssa.util.i18n.MessageBundleFactory;
import pl.psnc.vlab.ssa.util.i18n.SSABundleFactory;
import pl.psnc.vlab.util.FormTools;
import pl.psnc.vlab.util.image.ImageFactory;
import pl.psnc.vlab.util.swing.gui.GuiTools;
import pl.psnc.vlab.util.swing.gui.Message;
import pl.psnc.vlab.xml.ScenarioBuilder;



/**
 * Close Scenario Action is used for closing reating existing dynamic scenario
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class CloseScenarioAction extends AbstractActionDefault {
    
    /** Instance of logger*/
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Instance of main frame*/
    private MainFrame mainFrame;
    
    /**
     * Creates a new instance of new Action. The new action is used
     * to create a new empty dynamic scenario. The action is created with the default
     * name, icons, mnemonic and accelerator key - these are taken from the
     * resource bundle
     * @param source the source swing component, which uses this action
     */
    public CloseScenarioAction(MainFrame mainFrame){
        super(SSABundleFactory.getValue("action.close.scen.name") , ImageFactory.getImageBean(ImageKeys.ICON_DELETE).getImage() );
        this.mainFrame = mainFrame;
        this.putValue(Action.SHORT_DESCRIPTION, SSABundleFactory.getValue("action.close.scen.tip"));
        this.putValue(Action.SMALL_ICON, ImageFactory.getImageBean(ImageKeys.ICON_DELETE).getImage());
        String mnemonic = SSABundleFactory.getValue("action.close.scen.mnemonic");
        KeyStroke keyStroke = KeyStroke.getKeyStroke(FormTools.toUpperLetterAndDigit(mnemonic));
        // this.putValue(Action.MNEMONIC_KEY, new Integer(keyStroke.getKeyCode()) );
        //this.putValue(MNEMONIC_KEY, new Integer(keyStroke.getKeyCode()) );
        this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(SSABundleFactory.getValue("action.close.scen.shortcut"))); //keyStroke);
        this.putValue(GuiTools.PROPERTY_DISABLED_ICON, ImageFactory.getImageBean(ImageKeys.ICON_DELETE_DISABLED).getImage());
    }
    
    /**
     * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        //check wheter the current graph is empty
        ScenarioGraph graph = getCurrentGraph();
        int result = 0;
        if(graph != null && !graph.isEmpty()
            && ActionFactory.getAction(ActionFactory.ACTION_SAVE_FILE).isEnabled() ){
            log.debug("An old secenario exists");
            result = Message.showConfirmDialog(mainFrame, MessageBundleFactory.getValue("message.another.scenario.in.use"),
                    MessageBundleFactory.getValue("message.another.scenario.in.use.title"),
                    Message.YES_NO_OPTION, Message.MESSAGE_TYPE_INF);
            log.debug("result="+result);
            if(result != Message.YES_OPTION){
                log.debug("Do not close scenario");
                return;
            }
        }
        try{
            mainFrame.installGraph();
            EventLogFactory.addInfoMessage("Scenario closed");
            mainFrame.getBasePane().getJScenDetailsPane().update(null, ScenarioBuilder.getScenarioDescription());
        }catch(SSAException err){
            log.error("Error"+err.getMessage());
            Message.showInfoMessage(MainFrame.getInstance(), err.getLocalizedMessage(), Message.MESSAGE_TYPE_ERROR);
        }
        
        //remove save chooser
        DynScenFileChooserFactory.removeChooser(DynScenFileChooserFactory.TYPE_SAVE_DIALOG);
        
        //disable actions
        ActionFactory.getAction(ActionFactory.ACTION_COPY).setEnabled(false);
        ActionFactory.getAction(ActionFactory.ACTION_PASTE).setEnabled(false);
        ActionFactory.getAction(ActionFactory.ACTION_DELETE).setEnabled(false);
        
        ActionFactory.getAction(ActionFactory.ACTION_SUBMIT_SCENARIO).setEnabled(false);
        ActionFactory.getAction(ActionFactory.ACTION_SAVE_FILE).setEnabled(false);
        ActionFactory.getAction(ActionFactory.ACTION_SAVE_FILE_AS).setEnabled(false);
        
        //enable actions
        ActionFactory.getAction(ActionFactory.ACTION_ZOOM_ACTUAL).setEnabled(true);
        ActionFactory.getAction(ActionFactory.ACTION_ZOOM_IN).setEnabled(true);
        ActionFactory.getAction(ActionFactory.ACTION_ZOOM_OUT).setEnabled(true);
        
        ActionFactory.getAction(ActionFactory.ACTION_ADD_VERTEX_COMPUTE).setEnabled(true);
        ActionFactory.getAction(ActionFactory.ACTION_ADD_VERTEX_EXPERIMENT).setEnabled(true);
        
        
    }
    
    
}
