package pl.psnc.vlab.ssa.actions;

import java.awt.event.ActionEvent;
import javax.swing.Action;
import javax.swing.KeyStroke;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jgraph.JGraph;
import org.jgraph.util.JGraphUtilities;
import pl.psnc.vlab.graph.model.ScenarioGraph;
import pl.psnc.vlab.ssa.util.ImageKeys;
import pl.psnc.vlab.ssa.util.i18n.SSABundleFactory;
import pl.psnc.vlab.util.FormTools;
import pl.psnc.vlab.util.image.ImageFactory;
import pl.psnc.vlab.util.swing.gui.GuiTools;



/**
 * 
 * Added some changes
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class DeleteAction extends AbstractActionDefault {
    
    /** Instance of logger*/
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /**
     * Creates a new instance of Copy Action. The copy action is used
     * to clone the existing nodes. The action is created with the default
     * name, icons, mnemonic and accelerator key - these are taken from the
     * resource bundle
     */
    public DeleteAction(){
        super(SSABundleFactory.getValue("action.delete.name") , ImageFactory.getImageBean(ImageKeys.ICON_DELETE).getImage() );
        this.putValue(Action.SHORT_DESCRIPTION, SSABundleFactory.getValue("action.delete.tip"));
        this.putValue(Action.SMALL_ICON, ImageFactory.getImageBean(ImageKeys.ICON_DELETE).getImage());
        String mnemonic = SSABundleFactory.getValue("action.delete.mnemonic");
        KeyStroke keyStroke = KeyStroke.getKeyStroke(FormTools.toUpperLetterAndDigit(mnemonic));
        this.putValue(Action.MNEMONIC_KEY, new Integer(keyStroke.getKeyCode()) );
        this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(SSABundleFactory.getValue("action.delete.shortcut")));
        this.putValue(GuiTools.PROPERTY_DISABLED_ICON, ImageFactory.getImageBean(ImageKeys.ICON_DELETE_DISABLED).getImage());
    }
    
    /**
     * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        // Get Selected Cells
        ScenarioGraph graph = getCurrentGraph();
        Object[] cells = graph.getSelectionCells();
        if (cells != null) {            
            // Remove Cells (incl.  Descendants) from the Model
            graph.getModel().remove(graph.getDescendants(cells));
            
            // remove arrows 
            Object edges[] = JGraphUtilities.getEdges(graph);
            removeEdges(graph, edges);
        }
    }
    
    
    /**
     * Removes the unused edges
     * @param edges 
     */
    private void removeEdges(JGraph graph, Object edges[]){
        int size = edges != null ? edges.length : 0;
        for(int i=0; i<size; i++){
            if(JGraphUtilities.getSourceVertex(graph, edges[i]) == null || 
               JGraphUtilities.getTargetVertex(graph, edges[i]) == null){
                JGraphUtilities.removeCells(graph, new Object[] {edges[i]} );
            }
        }
    }
    
}




