package pl.psnc.vlab.ssa.actions;

import java.awt.event.ActionEvent;
import java.awt.geom.Point2D;
import java.util.Map;
import javax.swing.Action;
import javax.swing.KeyStroke;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.GraphConstants;
import pl.psnc.vlab.dev.DeviceDocument.Device;
import pl.psnc.vlab.graph.ScenarioGraphEditorFactory;
import pl.psnc.vlab.graph.model.GraphCellFactory;
import pl.psnc.vlab.graph.model.ScenarioCellObject;
import pl.psnc.vlab.graph.model.ScenarioGraph;
import pl.psnc.vlab.graph.ui.ScenarioGraphUI;
import pl.psnc.vlab.graph.util.DynScenFileChooserFactory;
import pl.psnc.vlab.graph.util.GraphHelper;
import pl.psnc.vlab.graph.vertex.type.ExperimentType;
import pl.psnc.vlab.graph.vertex.type.VLabVertexType;
import pl.psnc.vlab.rds.ResourceDocument;
import pl.psnc.vlab.ssa.event.EventLogFactory;
import pl.psnc.vlab.ssa.exception.SSAException;
import pl.psnc.vlab.ssa.gui.MainFrame;
import pl.psnc.vlab.ssa.util.ImageKeys;
import pl.psnc.vlab.ssa.util.i18n.MessageBundleFactory;
import pl.psnc.vlab.ssa.util.i18n.SSABundleFactory;
import pl.psnc.vlab.util.FormTools;
import pl.psnc.vlab.util.image.ImageFactory;
import pl.psnc.vlab.util.swing.gui.Message;
import pl.psnc.vlab.xml.DeviceFactory;
import pl.psnc.vlab.xml.ResourceFactory;
import pl.psnc.vlab.xml.ScenarioBuilder;



/**
 * The New Scenario Action is used for creating a new dynamic scenario
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class NewScenarioAction extends AbstractActionDefault {
    
    /** Instance of logger*/
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Instance of main frame*/
    private MainFrame mainFrame;
    
    /**
     * Creates a new instance of new Action. The new action is used
     * to create a new empty dynamic scenario. The action is created with the default
     * name, icons, mnemonic and accelerator key - these are taken from the
     * resource bundle
     * @param source the source swing component, which uses this action
     */
    public NewScenarioAction(MainFrame mainFrame){
        super(SSABundleFactory.getValue("action.new.name") , ImageFactory.getImageBean(ImageKeys.ICON_NEW).getImage() );
        this.mainFrame = mainFrame;
        this.putValue(Action.SHORT_DESCRIPTION, SSABundleFactory.getValue("action.new.tip"));
        this.putValue(Action.SMALL_ICON, ImageFactory.getImageBean(ImageKeys.ICON_NEW).getImage());
        String mnemonic = SSABundleFactory.getValue("action.new.mnemonic");
        KeyStroke keyStroke = KeyStroke.getKeyStroke(FormTools.toUpperLetterAndDigit(mnemonic));
        this.putValue(Action.MNEMONIC_KEY, new Integer(keyStroke.getKeyCode()) );
        this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(SSABundleFactory.getValue("action.new.shortcut"))); //keyStroke);
    }
    
    /**
     * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        //check wheter the current graph is empty
        ScenarioGraph graph = getCurrentGraph();
        int result = 0;
        if(graph != null && !graph.isEmpty()
        && ActionFactory.getAction(ActionFactory.ACTION_SAVE_FILE).isEnabled() ){
            log.debug("An old secenario exists");
            result = Message.showConfirmDialog(mainFrame, MessageBundleFactory.getValue("message.another.scenario.in.use"),
                    MessageBundleFactory.getValue("message.another.scenario.in.use.title"),
                    Message.YES_NO_OPTION, Message.MESSAGE_TYPE_INF);
            log.debug("result="+result);
            if(result != Message.YES_OPTION){
                log.debug("Do not close scenario");
                return;
            }
        }
        try{
            mainFrame.installGraph();
            mainFrame.getBasePane().getJScenDetailsPane().update(null, ScenarioBuilder.getScenarioDescription());
            EventLogFactory.addInfoMessage("New scenario created");
        }catch(SSAException err){
            log.error("Error"+err.getMessage());
            Message.showInfoMessage(MainFrame.getInstance(), err.getLocalizedMessage(), Message.MESSAGE_TYPE_ERROR);
        }
        
        //remove save chooser
        DynScenFileChooserFactory.removeChooser(DynScenFileChooserFactory.TYPE_SAVE_DIALOG);
        
        
        
        try{
            Device device[] = DeviceFactory.getInstance().getDeviceList();
            int size = device != null ? device.length : 0;
            for (int i = 0; i < size; i++) {
                
                ResourceDocument.Resource resource =  ResourceFactory.getResourceById(12);
                
                VLabVertexType vertexType = new ExperimentType(resource.getTypeId());
                ScenarioCellObject cellObject = new ScenarioCellObject(vertexType);
                resource.setNodeId(ResourceFactory.getNextResourceId());
                //set the resource described by this cell
                resource.setId(i+1);
                cellObject.setResource(resource);
                
                //set label
                cellObject.putProperty(ScenarioCellObject.keyValue, resource.getName());
                // removed 10.1 Map map = graph.getModel().createAttributes();
                Map map = new AttributeMap();
                GraphConstants.setIcon(map, GraphHelper.getResourceImage(vertexType.getTypeId()).getImage());                
                GraphConstants.setMoveable(map, false);
                graph.setSelectionAttributes((AttributeMap) map);
                DefaultGraphCell imageGraphCell =  GraphCellFactory.createCell(vertexType, cellObject);
                
                //imageGraphCell.setUserObject(cellObject);
                
                Point2D p = ( (ScenarioGraphUI) graph.getUI()).rescale(device[i].getLocation().getX(), device[i].getLocation().getY() );
                log.debug("Point = "+p);
                // AttributeMap cellAttrMap = imageGraphCell.getAttributes();
                // GraphConstants.setBounds(cellAttrMap,
                ScenarioGraphEditorFactory.getGraphEditorInstance().insert( p , imageGraphCell);
                
            }
        }catch(Exception err1){
            err1.printStackTrace();
            Message.showInfoMessage(this.mainFrame,  err1.getMessage(), Message.MESSAGE_TYPE_ERROR);
        }
        
        
        //disable actions
        ActionFactory.getAction(ActionFactory.ACTION_COPY).setEnabled(false);
        ActionFactory.getAction(ActionFactory.ACTION_PASTE).setEnabled(false);
        ActionFactory.getAction(ActionFactory.ACTION_DELETE).setEnabled(false);
        
        ActionFactory.getAction(ActionFactory.ACTION_SUBMIT_SCENARIO).setEnabled(false);
        ActionFactory.getAction(ActionFactory.ACTION_SAVE_FILE).setEnabled(false);
        ActionFactory.getAction(ActionFactory.ACTION_SAVE_FILE_AS).setEnabled(false);
        
        //enable actions
        ActionFactory.getAction(ActionFactory.ACTION_CLOSE_SCENARIO).setEnabled(true);
        ActionFactory.getAction(ActionFactory.ACTION_ZOOM_ACTUAL).setEnabled(true);
        ActionFactory.getAction(ActionFactory.ACTION_ZOOM_IN).setEnabled(true);
        ActionFactory.getAction(ActionFactory.ACTION_ZOOM_OUT).setEnabled(true);
        ActionFactory.getAction(ActionFactory.ACTION_WELCOME_SCREEN).setEnabled(true);
        ActionFactory.getAction(ActionFactory.ACTION_SCENARIO_PROPERTIES).setEnabled(true);
        Action resOutline = ActionFactory.getAction(ActionFactory.ACTION_RESOURCE_OUTLINE);
        resOutline.setEnabled(true);
        resOutline.actionPerformed(e);
        
        ActionFactory.getAction(ActionFactory.ACTION_ADD_VERTEX_COMPUTE).setEnabled(true);
        ActionFactory.getAction(ActionFactory.ACTION_ADD_VERTEX_EXPERIMENT).setEnabled(true);
        
        
    }
    
    
}
