package pl.psnc.vlab.ssa.actions;

import java.awt.event.ActionEvent;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Hashtable;
import javax.swing.Action;
import javax.swing.JFileChooser;
import javax.swing.KeyStroke;
import javax.swing.filechooser.FileFilter;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.GraphModel;
import org.jgraph.util.JGraphUtilities;
import pl.psnc.vlab.exception.VlabException;
import pl.psnc.vlab.graph.ScenarioGraphEditor;
import pl.psnc.vlab.graph.model.ScenarioCellObject;
import pl.psnc.vlab.graph.util.DynScenFileChooser;
import pl.psnc.vlab.graph.model.ScenarioGraph;
import pl.psnc.vlab.graph.pad.GraphModelProvider;
import pl.psnc.vlab.graph.pad.GraphModelProviderRegistry;
import pl.psnc.vlab.graph.pad.format.GraphModelFileFormat;
import pl.psnc.vlab.graph.util.DynScenFileChooserFactory;
import pl.psnc.vlab.psc.ScenarioDocument;
import pl.psnc.vlab.rds.ResourceDocument;
import pl.psnc.vlab.ssa.exception.SSAException;
import pl.psnc.vlab.ssa.gui.MainFrame;

import pl.psnc.vlab.ssa.util.AppMessage;
import pl.psnc.vlab.ssa.util.ImageKeys;
import pl.psnc.vlab.ssa.util.i18n.MessageBundleFactory;
import pl.psnc.vlab.ssa.util.i18n.SSABundleFactory;
import pl.psnc.vlab.util.FileTools;
import pl.psnc.vlab.util.FormTools;
import pl.psnc.vlab.util.image.ImageFactory;
import pl.psnc.vlab.util.swing.gui.Message;
import pl.psnc.vlab.xml.ResourceFactory;
import pl.psnc.vlab.xml.ScenarioBuilder;



/**
 *
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class OpenFileAction extends AbstractActionDefault {
    
    /** Instance of logger*/
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Instance of main frame*/
    private MainFrame mainFrame;
    
    
    /**
     * Creates a new instance of Open Action. The open action is used
     * to clone the existing nodes. The action is created with the default
     * name, icons, mnemonic and accelerator key - these are taken from the
     * resource bundle
     * @param source the source swing component, which uses this action
     */
    public OpenFileAction(MainFrame mainFrame){
        super(SSABundleFactory.getValue("action.open.name") , ImageFactory.getImageBean(ImageKeys.ICON_OPEN).getImage() );
        this.mainFrame = mainFrame;
        this.putValue(Action.SHORT_DESCRIPTION, SSABundleFactory.getValue("action.open.tip"));
        this.putValue(Action.SMALL_ICON, ImageFactory.getImageBean(ImageKeys.ICON_OPEN).getImage());
        String mnemonic = SSABundleFactory.getValue("action.open.mnemonic");
        KeyStroke keyStroke = KeyStroke.getKeyStroke(FormTools.toUpperLetterAndDigit(mnemonic));
        this.putValue(Action.MNEMONIC_KEY, new Integer(keyStroke.getKeyCode()) );
        this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(SSABundleFactory.getValue("action.open.shortcut"))); //keyStroke);
    }
    
    /**
     * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        log.debug("<actionPerformed>");
        //check wheter the current graph is empty
        ScenarioGraph graph = getCurrentGraph();
        int result = 0;
        if(graph != null && !graph.isEmpty()){
            log.debug("An old secenario exists");
            result = Message.showConfirmDialog(null, MessageBundleFactory.getValue("message.another.scenario.in.use"),
                    MessageBundleFactory.getValue("message.another.scenario.in.use.title"),
                    Message.YES_NO_OPTION, Message.MESSAGE_TYPE_INF);
            switch(result){
                case Message.NO_OPTION:
                    log.debug("Do not close scenario");
                    log.debug("</actionPerformed>");
                    return;
            } //end of switch
        }
        
        try{
            URL file = null;
            
            file = getFileToOpen();
            
            if(file == null){   //CANCEL OPTION
                log.debug("</actionPerformed:cancel>");
                return;
            }
            log.debug("File="+file);
            loadGraph(file);
            updateActions(new File(file.getFile()), e);
            log.debug("</actionPerformed>");
        } catch (Exception ex) {
            ex.printStackTrace();
            log.error("Error"+ex.getMessage());
            Message.showInfoMessage(MainFrame.getInstance(), ex.getLocalizedMessage(), Message.MESSAGE_TYPE_ERROR);
            return ;
        }
        
    }
    
    
    
    /**
     * Update the Main Fram Base top component title
     * @param modified
     */
    public void updateFrameTitle(String name){
        ((MainFrame)mainFrame).getBasePane().setTitle(name, false);
    }
    
    
    //--------------------------------------------------------------------------
    //----- Private, internal helper methods
    
    
    /**
     * Returns the File object representing the file to be opened.
     * @return File object or null if Cancel option has been chosen.
     */
    private URL getFileToOpen() throws MalformedURLException{
        DynScenFileChooser chooser = (DynScenFileChooser) DynScenFileChooserFactory.getFileChooser( DynScenFileChooserFactory.TYPE_OPEN_DIALOG);
        int result = chooser.showOpenDialog(mainFrame);
        if (result == JFileChooser.CANCEL_OPTION){
            return null;
        }
        return chooser.getSelectedFile().toURL();
    }
    
    
    /**
     * Load graph from the given URL
     * @param url URL representing graph
     * @throws java.lang.Exception
     */
    private void loadGraph(URL url) throws Exception{
        log.debug("<loadGraph>");
        String file = url.getFile();
        // exract the provider and the file format
        final GraphModelProvider graphModelProvider = GraphModelProviderRegistry.getGraphModelProvider(file);
        final GraphModelFileFormat graphModelFileFormat = GraphModelProviderRegistry.getGraphModelFileFormat(file);
        
        //create new grap editor object instance
        
        ScenarioGraphEditor graphEditor = MainFrame.getInstance().installGraph();
        ScenarioGraph graph = graphEditor.getGraph();
        
        GraphModel newModel = graphModelFileFormat.read(url, null, graph);
        
        if(newModel == null){
            log.debug("</loadGraph:no model>");
            throw new SSAException(MessageBundleFactory.getValue("message.file.incorrect"));
        }
        //load scenario details
        ScenarioBuilder.loadScenarioDetails(url);
        graph.repaint();
        graph.invalidate();
        Object list[] = JGraphUtilities.getVertices(graph, JGraphUtilities.getAll(graph));
        int size = list != null ? list.length : 0;
        for(int i=0; i<size; i++){
            DefaultGraphCell cell = (DefaultGraphCell) list[i];
            ScenarioCellObject cellObject = (ScenarioCellObject) cell.getUserObject();
            ResourceDocument.Resource resource = cellObject != null ? cellObject.getResource() : null;
            if(resource != null ){
                resource.setNodeId(ResourceFactory.getNextResourceId());
                log.debug("New node id-"+resource.getNodeId());
            }
        } //end of for
        log.debug("</loadGraph>");
    }
    
    
    /**
     * Updates the coresponding and related actions
     * @param file loaded file
     * @param e ActionEvent
     */
    private void updateActions(File file, ActionEvent e){
        ActionFactory.getAction(ActionFactory.ACTION_SAVE_FILE).setEnabled(false);
        ActionFactory.getAction(ActionFactory.ACTION_ADD_VERTEX_COMPUTE).setEnabled(true);
        ActionFactory.getAction(ActionFactory.ACTION_ADD_VERTEX_EXPERIMENT).setEnabled(true);
        ActionFactory.getAction(ActionFactory.ACTION_CLOSE_SCENARIO).setEnabled(true);
        ActionFactory.getAction(ActionFactory.ACTION_WELCOME_SCREEN).setEnabled(true);
        ActionFactory.getAction(ActionFactory.ACTION_SCENARIO_PROPERTIES).setEnabled(true);
        Action resOutline = ActionFactory.getAction(ActionFactory.ACTION_RESOURCE_OUTLINE);
        resOutline.setEnabled(true);
        resOutline.actionPerformed(e);
        
        
        DynScenFileChooserFactory.getFileChooser(DynScenFileChooserFactory.TYPE_SAVE_DIALOG).setSelectedFile(file);
        DynScenFileChooserFactory.setSelectedFileName(file.getName());
        updateFrameTitle(file.getName());
        mainFrame.getBasePane().getJScenDetailsPane().update(null, ScenarioBuilder.getScenarioDescription());
    }
    
}
