package pl.psnc.vlab.ssa.actions;

import java.awt.event.ActionEvent;
import java.io.File;
import java.util.Hashtable;
import javax.swing.Action;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.KeyStroke;
import javax.swing.filechooser.FileFilter;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.graph.util.DynScenFileChooser;
import pl.psnc.vlab.graph.model.ScenarioGraph;
import pl.psnc.vlab.graph.pad.GraphModelProviderRegistry;
import pl.psnc.vlab.graph.pad.format.GraphModelFileFormat;
import pl.psnc.vlab.graph.util.DynScenFileChooserFactory;
import pl.psnc.vlab.ssa.gui.MainFrame;
import pl.psnc.vlab.ssa.gui.resource.model.ResourceElementObserver;
import pl.psnc.vlab.ssa.util.AppMessage;
import pl.psnc.vlab.ssa.util.ImageKeys;
import pl.psnc.vlab.ssa.util.i18n.MessageBundleFactory;
import pl.psnc.vlab.ssa.util.i18n.SSABundleFactory;
import pl.psnc.vlab.util.FileTools;
import pl.psnc.vlab.util.FormTools;
import pl.psnc.vlab.util.image.ImageFactory;
import pl.psnc.vlab.util.swing.gui.GuiTools;
import pl.psnc.vlab.util.swing.gui.Message;
import pl.psnc.vlab.xml.ScenarioBuilder;



/**
 * The action for saving the scenario.
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class SaveFileAction extends AbstractActionDefault implements ResourceElementObserver{
    
    /** Instance of logger*/
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    private String scenarioInfoFile = null;
    
    /** Stores an instance od MainFrame */
    private JFrame mainFrame;
    
    /**
     * Creates a new instance of save Action. The save action is used
     * to clone the existing nodes. The action is created with the default
     * name, icons, mnemonic and accelerator key - these are taken from the
     * resource bundle
     * @param source the source swing component, which uses this action
     */
    public SaveFileAction(JFrame mainFrame){
        super(SSABundleFactory.getValue("action.save.name") , ImageFactory.getImageBean(ImageKeys.ICON_SAVE).getImage() );
        this.mainFrame = mainFrame;
        this.putValue(Action.SHORT_DESCRIPTION, SSABundleFactory.getValue("action.save.tip"));
        this.putValue(Action.SMALL_ICON, ImageFactory.getImageBean(ImageKeys.ICON_SAVE).getImage());
        String mnemonic = SSABundleFactory.getValue("action.save.mnemonic");
        KeyStroke keyStroke = KeyStroke.getKeyStroke(FormTools.toUpperLetterAndDigit(mnemonic));
        this.putValue(Action.MNEMONIC_KEY, new Integer(keyStroke.getKeyCode()) );
        this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(SSABundleFactory.getValue("action.save.shortcut"))); //keyStroke);
        this.putValue(GuiTools.PROPERTY_DISABLED_ICON, ImageFactory.getImageBean(ImageKeys.ICON_SAVE_DISABLED).getImage());
    }
    
    /**
     * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        
        //check wheter the current graph is empty
        ScenarioGraph graph = getCurrentGraph();
        int result = 0;
        try{
            DynScenFileChooser chooser = (DynScenFileChooser) DynScenFileChooserFactory.getFileChooser( DynScenFileChooserFactory.TYPE_SAVE_DIALOG);
            File file = chooser.getSelectedFile();
            log.debug("File="+file);
            if(file == null){
                result = chooser.showSaveDialog(mainFrame);
                if (result == JFileChooser.CANCEL_OPTION){
                    chooser.setSelectedFile(null);
                    return;
                }
                file = chooser.getSelectedFile();
            }
            
            // exract the provider and the file format
            GraphModelFileFormat graphModelFileFormat = GraphModelProviderRegistry.getGraphModelFileFormat(file);
            // get the file format
            FileFilter fileFilter = chooser.getFileFilter();
            
            scenarioInfoFile = FileTools.getBase(file.getAbsolutePath());
            //check if the file has an extension
            if( FileTools.getExtension(file) == null ){
                String fileName = new StringBuffer()
                .append(scenarioInfoFile)
                .append(".")
                .append( graphModelFileFormat.getFileExtension()).toString();
                file = new File( fileName);
            }
            scenarioInfoFile += ".info";
            
            // extract the writer properties
            Hashtable props = graph.getWriteProperties();
            
            // try to write the file
            
            graphModelFileFormat.write(file.toURL(), props,graph, graph.getModel());
            DynScenFileChooserFactory.getFileChooser(DynScenFileChooserFactory.TYPE_SAVE_DIALOG).setSelectedFile(file);
            DynScenFileChooserFactory.setSelectedFileName(file.getName());
            ActionFactory.getAction(ActionFactory.ACTION_SAVE_FILE_AS).setEnabled(false);
            setEnabled(false);
            //save scenario details
            ScenarioBuilder.saveScenarioDetails(scenarioInfoFile);
            
            ((MainFrame)mainFrame).getStatusBar().setMessage(MessageBundleFactory.getValue("message.save.ok"));
        } catch (Exception ex) {
            Message.showInfoMessage(MainFrame.getInstance(), ex.getLocalizedMessage(), Message.MESSAGE_TYPE_ERROR);
            ex.printStackTrace();
            return ;
        } finally {
            graph.invalidate();
            //graphpad.invalidate();
        }
    }
    
    /**
     * Notifies the observers that they should update their data
     */
    public void update() {
        setEnabled(true);
    }
    
    /**
     * Enables or disables the action.
     *
     * @param newValue  true to enable the action, false to
     *                  disable it
     * @see Action#setEnabled
     */
    public void setEnabled(boolean newValue) {
        super.setEnabled(newValue);
        //update the file name
        updateFrameTitle(newValue);
    }
    
    
    /**
     * Update the Main Fram Base top component title
     * @param modified
     */
    public void updateFrameTitle(boolean modified){
        String fileName = DynScenFileChooserFactory.getSelectedFileName() != null ?
            DynScenFileChooserFactory.getSelectedFileName() : SSABundleFactory.getValue("msg.untitled");
        if(modified){
            ((MainFrame)mainFrame).getBasePane().setTitle(fileName, true);
        } else {
            ((MainFrame)mainFrame).getBasePane().setTitle(fileName, false);
        }
    }
    
    
    
    
    
    
    
}
