package pl.psnc.vlab.ssa.actions;

import java.awt.event.ActionEvent;
import javax.swing.Action;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.graph.model.ScenarioCellObject;
import pl.psnc.vlab.graph.model.ScenarioGraph;
import pl.psnc.vlab.ssa.util.ImageKeys;
import pl.psnc.vlab.ssa.util.i18n.SSABundleFactory;
import pl.psnc.vlab.util.FormTools;
import pl.psnc.vlab.util.image.ImageFactory;




/**
 * This action is used to mark the node as initial one. The initial node
 * is the first node to be taken into consideration while decoding the 
 * Dynamic Scenario
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class SetNodeAsInitialAction extends AbstractActionDefault {
    
    /** Instance of the node cell object  */
    public static String NODE_CELL_OBJECT = "nodeCellObject";
    
    /** Instance of logger*/
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /**
     * Creates a new instance of mark.initial Action. The mark.initial action is used
     * to clone the existing nodes. The action is created with the default
     * name, icons, mnemonic and accelerator key - these are taken from the 
     * resource bundle
     */
    public SetNodeAsInitialAction(boolean isSelected){
        super(SSABundleFactory.getValue("action.mark.initial.name") , ImageFactory.getImageBean(ImageKeys.ICON_COPY).getImage() );
        this.putValue(Action.SHORT_DESCRIPTION, SSABundleFactory.getValue("action.mark.initial.tip"));
        updateIcon(isSelected);
        String mnemonic = SSABundleFactory.getValue("action.mark.initial.mnemonic");
        KeyStroke keyStroke = KeyStroke.getKeyStroke(FormTools.toUpperLetterAndDigit(mnemonic));
        this.putValue(Action.MNEMONIC_KEY, new Integer(keyStroke.getKeyCode()) );
        this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(SSABundleFactory.getValue("action.mark.initial.shortcut"))); //keyStroke);
        
    }
    
    /**
     * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        log.debug("<actionPerformed>"); 
        ScenarioGraph graph = getCurrentGraph();
        //get the checkbox item
        JMenuItem checkBoxItem = (JMenuItem) e.getSource();
        //get the scenario cell object
        ScenarioCellObject cellObject = (ScenarioCellObject) this.getValue(NODE_CELL_OBJECT);
        if(checkBoxItem == null){
            log.debug("CheckBox is null");
        }
        //update selection
        cellObject.setMarkedInitial(!checkBoxItem.isSelected());
        updateIcon(!checkBoxItem.isSelected());        
        graph.repaint();                
        graph.getScenarioGraphSelectionListener().valueChanged(null);
        log.debug("</actionPerformed>");
    }
    
    
    /**
     * Updates the source component icon depending upon the
     * selection value
     * @param isSelected
     */    
    private void updateIcon(boolean isSelected){
        if(isSelected){
            log.debug("ok");
            this.putValue(Action.SMALL_ICON, ImageFactory.getImageBean(ImageKeys.SMALL_ICON_OK).getImage());
        } else {
            log.debug("cancel");
            this.putValue(Action.SMALL_ICON, ImageFactory.getImageBean(ImageKeys.SMALL_ICON_CANCEL).getImage());
        }        
    }
}
