package pl.psnc.vlab.ssa.actions;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Hashtable;
import javax.swing.Action;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.util.JGraphUtilities;
import pl.psnc.vlab.graph.model.ScenarioCellObject;
import pl.psnc.vlab.graph.model.ScenarioGraph;
import pl.psnc.vlab.graph.pad.GraphModelProviderRegistry;
import pl.psnc.vlab.graph.pad.format.GraphModelFileFormat;
import pl.psnc.vlab.graph.util.GraphHelper;
import pl.psnc.vlab.management.session.SessionFactory;
import pl.psnc.vlab.rds.ScenarioDocument;
import pl.psnc.vlab.ssa.gui.MainFrame;
import pl.psnc.vlab.ssa.gui.resource.pane.JResourcePropertyDialog;
import pl.psnc.vlab.ssa.gui.scenario.ScenarioPropertiesDialog;

import pl.psnc.vlab.ssa.util.AppMessage;
import pl.psnc.vlab.ssa.util.ImageKeys;
import pl.psnc.vlab.ssa.util.i18n.MessageBundleFactory;
import pl.psnc.vlab.ssa.util.i18n.SSABundleFactory;
import pl.psnc.vlab.statusbar.StatusBar;
import pl.psnc.vlab.util.FormTools;
import pl.psnc.vlab.util.image.ImageFactory;
import pl.psnc.vlab.util.swing.gui.GuiTools;
import pl.psnc.vlab.util.swing.gui.Message;
import pl.psnc.vlab.xml.ScenarioBuilder;



/**
 * Represents the Dynamic Scenario Submission action
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class SubmitScenarioAction extends AbstractActionDefault {
    
    /** Instance of logger*/
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** The VLab Remote interface key */
    private String MAIN_FRAMEVLAB_REMOTE_INTERFACE = "vlabRemoteInterface";
    
    /**
     * Creates a new instance of Scenario Submission Action.
     *
     * The action is created with the default name, icons,
     * mnemonic and accelerator key - these are taken from the
     * resource bundle
     */
    public SubmitScenarioAction(){
        super(SSABundleFactory.getValue("action.submit.scenario.name") , ImageFactory.getImageBean(ImageKeys.ICON_SUBMIT).getImage() );
        this.putValue(Action.SHORT_DESCRIPTION, SSABundleFactory.getValue("action.submit.scenario.tip"));
        this.putValue(Action.SMALL_ICON, ImageFactory.getImageBean(ImageKeys.ICON_SUBMIT).getImage());
        String mnemonic = SSABundleFactory.getValue("action.submit.scenario.mnemonic");
        KeyStroke keyStroke = KeyStroke.getKeyStroke(FormTools.toUpperLetterAndDigit(mnemonic));
        this.putValue(Action.MNEMONIC_KEY, new Integer(keyStroke.getKeyCode()) );
        this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(SSABundleFactory.getValue("action.submit.scenario.shortcut")));
        this.putValue(GuiTools.PROPERTY_DISABLED_ICON, ImageFactory.getImageBean(ImageKeys.ICON_SUBMIT_DISABLED).getImage());
    }
    
    /**
     * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        log.debug("<SubmitScenario>");
        try{
            setEnabled(false);
            StatusBar statusBar = MainFrame.getStatusBar();
            statusBar.attachIndeterminedProgressBar(MessageBundleFactory.getValue("message.wait"));
            statusBar.setMessage("Sending scenario. Please wait!");
            
            
            //check wheter the scenario name has been defined
            ScenarioDocument.Scenario scenario = ScenarioBuilder.getScenarioDescription();
            if(FormTools.isValidateFormStringNull(scenario.getName()) ){
                statusBar.detachProgressBar();
                statusBar.setMessage("");
                Message.showInfoMessage(MainFrame.getInstance(), AppMessage.SCEN_NO_NAME_DEFINED, Message.MESSAGE_TYPE_INF);
                ScenarioPropertiesDialog dialog = new ScenarioPropertiesDialog(MainFrame.getInstance(), true, scenario);
                dialog.show();
                setEnabled(true);
                if(dialog.getUserOption() != JOptionPane.OK_OPTION){
                    return;
                }
                actionPerformed(e);
                return;
            }
            
            
            //check session first
            /*if( !SessionFactory.isSessionValid() ){
                statusBar.detachProgressBar();
                log.debug("Session is invalid");
                MainFrame.getInstance().criticalError(AppMessage.MSG_ERR_INVALID_SESSION, "");
            }
            */
            //check the existance of the initial nodes
            ScenarioGraph graph = getCurrentGraph();
                  
            
            
            //todo Show save dialog. Get the jgx
            GraphModelFileFormat graphModelFileFormat = GraphModelProviderRegistry.getGraphModelFileFormat(".jgx");
            
            // extract the writer properties
            Hashtable props = graph.getWriteProperties();
            // try to write the file
            String jgxFile = graphModelFileFormat.write(props, graph, graph.getModel());
            
            ArrayList resourceList = graph.getAllResourcesDesc();
            ScenarioBuilder.insertResources(resourceList);
            String scenarioXml = ScenarioBuilder.toStirng();
            statusBar.detachProgressBar();
            
            Message.showInfoMessage(MainFrame.getInstance(), "Scenario has been submitted.", Message.MESSAGE_TYPE_INF);                        
            log.debug("</SubmitScenario>");
        } catch(Exception err){
            err.printStackTrace();
            Message.showInfoMessage(MainFrame.getInstance(), err.getLocalizedMessage(), Message.MESSAGE_TYPE_ERROR);
            setEnabled(true);
        }
        
    }
}
