package pl.psnc.vlab.ssa.actions;

import java.awt.event.ActionEvent;
import javax.swing.Action;
import javax.swing.KeyStroke;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.graph.model.ScenarioGraph;
import pl.psnc.vlab.ssa.util.ImageKeys;
import pl.psnc.vlab.ssa.util.i18n.SSABundleFactory;
import pl.psnc.vlab.util.FormTools;
import pl.psnc.vlab.util.image.ImageFactory;
import pl.psnc.vlab.util.swing.gui.GuiTools;



/**
 * The zoom actual Action
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class ZoomActualAction extends AbstractActionDefault {
    
    /** Instance of logger*/
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /**
     * Creates a new instance of zoom actual Action.  The action is created with the default
     * name, icons, mnemonic and accelerator key - these are taken from the
     * resource bundle
     */
    public ZoomActualAction(){
        super(SSABundleFactory.getValue("action.zoom.actual.name") , ImageFactory.getImageBean(ImageKeys.ICON_ACTUAL_SIZE).getImage() );
        this.putValue(Action.SHORT_DESCRIPTION, SSABundleFactory.getValue("action.zoom.actual.tip"));
        this.putValue(Action.SMALL_ICON, ImageFactory.getImageBean(ImageKeys.ICON_ACTUAL_SIZE).getImage());
        String mnemonic = SSABundleFactory.getValue("action.zoom.actual.mnemonic");
        KeyStroke keyStroke = KeyStroke.getKeyStroke(FormTools.toUpperLetterAndDigit(mnemonic));
        this.putValue(Action.MNEMONIC_KEY, new Integer(keyStroke.getKeyCode()) );
        this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(SSABundleFactory.getValue("action.zoom.actual.shortcut"))); //keyStroke);
        this.putValue(GuiTools.PROPERTY_DISABLED_ICON, ImageFactory.getImageBean(ImageKeys.ICON_ACTUAL_SIZE_DISABLED).getImage());
    }
    
    /**
     * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        ScenarioGraph graph = getCurrentGraph();
        //graph.setResizeAction(null);
        double scale = graph.getScale();
        if(scale == 1.0){
            return;
        }       
        graph.setScale(1.0);
        scrollCellToVisible(graph);
        ActionFactory.getAction(ActionFactory.ACTION_ZOOM_OUT).setEnabled(true);
        ActionFactory.getAction(ActionFactory.ACTION_ZOOM_IN).setEnabled(true);
    }
    
    /**
     * Scrolls the cell to the given scale
     * @param graph
     */
    private void scrollCellToVisible(ScenarioGraph graph){
        Object selectionCells[] = graph.getSelectionCells();
        if (selectionCells != null){
            graph.scrollCellToVisible(selectionCells);
        }
    }
    
}
