package pl.psnc.vlab.ssa.actions;

import java.awt.event.ActionEvent;
import javax.swing.Action;
import javax.swing.KeyStroke;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.graph.model.ScenarioGraph;
import pl.psnc.vlab.ssa.util.ImageKeys;
import pl.psnc.vlab.ssa.util.i18n.SSABundleFactory;
import pl.psnc.vlab.util.FormTools;
import pl.psnc.vlab.util.image.ImageFactory;
import pl.psnc.vlab.util.swing.gui.GuiTools;



/**
 * The zoom in Action
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class ZoomInAction extends AbstractActionDefault {
    
    /** Instance of logger*/
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    private static double MAX  = 5.0;
    private static double STEP = 1.5;
    
    /**
     * Creates a new instance of zoom in Action.  The action is created with the default
     * name, icons, mnemonic and accelerator key - these are taken from the
     * resource bundle
     */
    public ZoomInAction(){
        super(SSABundleFactory.getValue("action.zoom.in.name") , ImageFactory.getImageBean(ImageKeys.ICON_ZOOM_IN).getImage() );
        this.putValue(Action.SHORT_DESCRIPTION, SSABundleFactory.getValue("action.zoom.in.tip"));
        this.putValue(Action.SMALL_ICON, ImageFactory.getImageBean(ImageKeys.ICON_ZOOM_IN).getImage());
        String mnemonic = SSABundleFactory.getValue("action.zoom.in.mnemonic");
        KeyStroke keyStroke = KeyStroke.getKeyStroke(FormTools.toUpperLetterAndDigit(mnemonic));
        this.putValue(Action.MNEMONIC_KEY, new Integer(keyStroke.getKeyCode()) );
        this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(SSABundleFactory.getValue("action.zoom.in.shortcut"))); //keyStroke);
        this.putValue(GuiTools.PROPERTY_DISABLED_ICON, ImageFactory.getImageBean(ImageKeys.ICON_ZOOM_IN_DISABLED).getImage());
    }
    
    /**
     * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        ScenarioGraph graph = getCurrentGraph();
        //graph.setResizeAction(null);
        double scale = graph.getScale();
        scale *= STEP;
        log.debug("Current scale-"+scale);        
        if (scale >= MAX) {
            this.setEnabled(false);            
        } else {
            graph.setScale(scale);
            scrollCellToVisible(graph);            
            ActionFactory.getAction(ActionFactory.ACTION_ZOOM_OUT).setEnabled(true);
        }
    }
    
    /**
     * Scrolls the cell to the given scale
     * @param graph
     */    
    private void scrollCellToVisible(ScenarioGraph graph){
        Object selectionCells[] = graph.getSelectionCells();
        if (selectionCells != null){
            graph.scrollCellToVisible(selectionCells);
        }
    }
}
