package pl.psnc.vlab.ssa.actions.perspective;

import java.awt.event.ActionEvent;
import javax.swing.Action;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.graph.model.ScenarioGraph;
import pl.psnc.vlab.ssa.actions.AbstractActionDefault;
import pl.psnc.vlab.ssa.actions.ActionFactory;
import pl.psnc.vlab.ssa.gui.MainFrame;
import pl.psnc.vlab.ssa.util.i18n.MessageBundleFactory;
import pl.psnc.vlab.ssa.util.i18n.SSABundleFactory;
import pl.psnc.vlab.util.swing.gui.Message;




/**
 * Welcome Pane Action - shows the welcome pane, which is
 * displayed when the application is launched
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class WelcomePaneAction extends AbstractActionDefault {
    
    /** Instance of logger*/
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Instance of the MainFrame*/
    private MainFrame mainFrame;
    
    /**
     * Creates a new instance of zoom in Action.  The action is created with the default
     * name, icons, mnemonic and accelerator key - these are taken from the
     * resource bundle
     */
    public WelcomePaneAction(MainFrame mainFrame){
        super(SSABundleFactory.getValue("action.welcome.pane.name") , null);
        this.mainFrame = mainFrame;
        this.putValue(Action.SHORT_DESCRIPTION, SSABundleFactory.getValue("action.welcome.pane.tip"));
    }
    
    /**
     * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        log.debug("WelcomePaneAction");
        //check wheter the current graph is empty
        ScenarioGraph graph = getCurrentGraph();
        int result = 0;
        if(graph != null && !graph.isEmpty()
        && ActionFactory.getAction(ActionFactory.ACTION_SAVE_FILE).isEnabled() ){
            log.debug("An old secenario exists");
            result = Message.showConfirmDialog(mainFrame, MessageBundleFactory.getValue("message.another.scenario.in.use"),
                    MessageBundleFactory.getValue("message.another.scenario.in.use.title"),
                    Message.YES_NO_OPTION, Message.MESSAGE_TYPE_INF);
            log.debug("result="+result);
            if(result != Message.YES_OPTION){
                log.debug("Do not close scenario");
                return;
            }
        }
        
        try{
            mainFrame.addWelcomePane();
            ResInfPerspAction resOutline = (ResInfPerspAction) ActionFactory.getAction(ActionFactory.ACTION_RESOURCE_OUTLINE);
            if(resOutline.isVisible() ){
                resOutline.actionPerformed(e);
            }
            resOutline.setEnabled(false);
            ActionFactory.getAction(ActionFactory.ACTION_SCENARIO_PROPERTIES).setEnabled(false);
        }catch(Exception err){
            Message.showInfoMessage(mainFrame, err.getMessage(), Message.MESSAGE_TYPE_ERROR);
        }
        
    }
    
    
}
