        /*
         * MainFrame.java
         *
         * Created on 12 stycze� 2002, 19:54
         */

package pl.psnc.vlab.ssa.gui;
import com.jgoodies.looks.BorderStyle;
import com.jgoodies.looks.HeaderStyle;
import com.jgoodies.looks.Options;
import com.jgoodies.looks.plastic.PlasticLookAndFeel;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.TimeZone;
import javax.swing.Action;
import javax.swing.ButtonGroup;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.Timer;
import javax.swing.UIManager;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.GraphConstants;
import pl.psnc.vlab.conf.ConfigFactory;
import pl.psnc.vlab.conf.SslFactory;
import pl.psnc.vlab.dev.DeviceDocument.Device;

//logging
import pl.psnc.vlab.graph.ScenarioGraphEditor;
import pl.psnc.vlab.graph.ScenarioGraphEditorFactory;
import pl.psnc.vlab.graph.model.CustomCellViewFactory;
import pl.psnc.vlab.graph.model.ScenarioCellObject;
import pl.psnc.vlab.graph.model.ScenarioGraph;
import pl.psnc.vlab.graph.ui.ScenarioGraphUI;
import pl.psnc.vlab.graph.vertex.type.ComputeType;
import pl.psnc.vlab.graph.vertex.type.ExperimentType;
import pl.psnc.vlab.graph.vertex.type.VLabVertexType;
import pl.psnc.vlab.management.PropertiesFactory;
import pl.psnc.vlab.management.session.SessionFactory;
import pl.psnc.vlab.management.session.SessionManager;
import pl.psnc.vlab.ssa.actions.ActionFactory;
import pl.psnc.vlab.ssa.actions.CloseAppAction;
import pl.psnc.vlab.ssa.actions.CloseScenarioAction;
import pl.psnc.vlab.ssa.actions.NewScenarioAction;
import pl.psnc.vlab.ssa.actions.OpenFileAction;
import pl.psnc.vlab.ssa.actions.SaveFileAction;
import pl.psnc.vlab.ssa.actions.SaveFileAsAction;
import pl.psnc.vlab.ssa.actions.perspective.AvailableActionsPerpAction;
import pl.psnc.vlab.ssa.actions.perspective.MessageLogPerspAction;
import pl.psnc.vlab.ssa.actions.perspective.ResInfPerspAction;
import pl.psnc.vlab.ssa.actions.perspective.WelcomePaneAction;
import pl.psnc.vlab.ssa.exception.SSAException;

import pl.psnc.vlab.ssa.gui.about.AboutBox;
import pl.psnc.vlab.ssa.gui.about.SplashDialog;
import pl.psnc.vlab.ssa.gui.main.WelcomePane;
import pl.psnc.vlab.ssa.management.AppInitializer;
import pl.psnc.vlab.ssa.management.SSAConfigurationImpl;
import pl.psnc.vlab.ssa.management.SplashDialogWorker;

import pl.psnc.vlab.ssa.util.AppMessage;
import pl.psnc.vlab.ssa.util.ImageKeys;
import pl.psnc.vlab.ssa.util.Keys;
import pl.psnc.vlab.ssa.management.SystemConfigurationFactory;
import pl.psnc.vlab.ssa.management.SystemConfigurationValue;

import pl.psnc.vlab.exception.VlabException;
import pl.psnc.vlab.graph.model.GraphCellFactory;
import pl.psnc.vlab.graph.util.GraphHelper;
import pl.psnc.vlab.management.session.SessionError;
import pl.psnc.vlab.rds.ResourceDocument;

import pl.psnc.vlab.ssa.remote.scenario.SendScenarioTask;
import pl.psnc.vlab.ssa.util.i18n.MessageBundleFactory;
import pl.psnc.vlab.statusbar.StatusBar;
import pl.psnc.vlab.util.FormTools;
import pl.psnc.vlab.util.TimeZoneTools;
import pl.psnc.vlab.util.image.ImageFactory;
import pl.psnc.vlab.util.swing.gui.GuiTools;
import pl.psnc.vlab.util.swing.gui.Message;
import pl.psnc.vlab.util.swing.thread.LongTask;
import pl.psnc.vlab.values.management.SessionValue;
import pl.psnc.vlab.xml.DeviceFactory;
import pl.psnc.vlab.xml.ResourceFactory;

/**
 *
 * @author  Osa
 */
public class MainFrame extends JFrame implements SessionError {
    
    /** Creates Log4j instance */
    private Log log = null;
    
    /** Instance of status bar */
    private static StatusBar statusBar = null;
    
    /** Instance of main frame */
    private static MainFrame instance=null;
    
    /** Instance of Welcome Pane */
    private JPanel welcomePane;
            
    /** Instance of Scenario Graph Editor*/
    private ScenarioGraphEditor scenGraphEditor = null;
    
    
    /** Instance of the MainFrameBase*/
    private JPanel mainFrameBase;
    
    /** Stores instance of config file */
    private final static String CFG_FILE = "wfmConf";
    
    /** Creates new form MainFrame */
    public MainFrame() {
        try{
            initComponents();
            log = LogFactory.getLog(this.getClass().getName() );
            initGui();
            //this.scenarioTimer.stop();
            this.mainFrameBase = new MainFrameBase();
            jBase.add(mainFrameBase);
            jBase.validate();
            jBase.setVisible(true);
            initStatusBar();    //sets the status bar
            initActions();      //init actions
            addWelcomePane();   //add initial screen
            
        }catch(Exception e){
            log.debug("IN HERE");
            e.printStackTrace();
            Message.showInfoMessage(this, e.getMessage(), Message.MESSAGE_TYPE_ERROR);
            this.setVisible(false);
            this.dispose();
            System.exit(Keys.EXIT_CODE_ERROR);
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jMenuToolar = new javax.swing.JToolBar();
        jSeparatorPane11 = new javax.swing.JPanel();
        jNewTb = new javax.swing.JButton();
        jOpenTb = new javax.swing.JButton();
        jSaveTb = new javax.swing.JButton();
        jSaveAsTb = new javax.swing.JButton();
        jSeparatorPane10 = new javax.swing.JPanel();
        jSeparator1 = new javax.swing.JSeparator();
        jSeparatorPane8 = new javax.swing.JPanel();
        jCopyBtn = new javax.swing.JButton();
        jPasteBtn = new javax.swing.JButton();
        jDeleteBtn = new javax.swing.JButton();
        jSeparatorPane7 = new javax.swing.JPanel();
        jSeparator10 = new javax.swing.JSeparator();
        jSeparatorPane9 = new javax.swing.JPanel();
        jUndoBtn = new javax.swing.JButton();
        jRedoBtn = new javax.swing.JButton();
        jSeparatorPane5 = new javax.swing.JPanel();
        jSeparator8 = new javax.swing.JSeparator();
        jSeparatorPane = new javax.swing.JPanel();
        jSubmitBtn = new javax.swing.JButton();
        jSeparatorPane1 = new javax.swing.JPanel();
        jSeparator5 = new javax.swing.JSeparator();
        jSeparatorPane4 = new javax.swing.JPanel();
        jSeparatorPane6 = new javax.swing.JPanel();
        jSeparatorPane2 = new javax.swing.JPanel();
        jZoomActualSizeBtn = new javax.swing.JButton();
        jZoomInBtn = new javax.swing.JButton();
        jZoomOutBtn = new javax.swing.JButton();
        jSeparatorPane3 = new javax.swing.JPanel();
        jMakeDemoGraph = new javax.swing.JButton();
        jBottomPane = new javax.swing.JPanel();
        jControlPane = new javax.swing.JPanel();
        jResourceInfoTBtn = new javax.swing.JToggleButton();
        jActionInfoTBtn = new javax.swing.JToggleButton();
        jSeparator11 = new javax.swing.JSeparator();
        jLogMessageTBtn = new javax.swing.JToggleButton();
        jSeparatorPanel = new javax.swing.JPanel();
        jSeparator7 = new javax.swing.JSeparator();
        jRightPane = new javax.swing.JPanel();
        jContentPane = new javax.swing.JPanel();
        jBase = new javax.swing.JPanel();
        jStatus = new javax.swing.JPanel();
        menuBar = new javax.swing.JMenuBar();
        ButtonGroup workType = new ButtonGroup();
        jFileMenu = new javax.swing.JMenu();
        jNewMenuItem = new javax.swing.JMenuItem();
        jOpenMenuItem = new javax.swing.JMenuItem();
        jCloseMenuItem = new javax.swing.JMenuItem();
        jSeparator4 = new javax.swing.JSeparator();
        jSaveMenuItem = new javax.swing.JMenuItem();
        jSaveAsMenuItem = new javax.swing.JMenuItem();
        jSeparator3 = new javax.swing.JSeparator();
        jExit = new javax.swing.JMenuItem();
        jEditMenu = new javax.swing.JMenu();
        jUndoMenuItem = new javax.swing.JMenuItem();
        jRedoMenuItem = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        jCopyMenuItem = new javax.swing.JMenuItem();
        jPaseMenuItem = new javax.swing.JMenuItem();
        jDeleteMenuItem = new javax.swing.JMenuItem();
        jScenarioMenu = new javax.swing.JMenu();
        jSubmitScenarioItem = new javax.swing.JMenuItem();
        jSeparator6 = new javax.swing.JSeparator();
        jScenarioPropertiesItem = new javax.swing.JMenuItem();
        jViewMenu = new javax.swing.JMenu();
        jActualSizeMenuItem = new javax.swing.JMenuItem();
        jZoomInMenuItem = new javax.swing.JMenuItem();
        jZoomOutMenuItem = new javax.swing.JMenuItem();
        jSeparator12 = new javax.swing.JSeparator();
        jResourceInfoMenuItem = new javax.swing.JCheckBoxMenuItem();
        jActionInfoMenuItem = new javax.swing.JCheckBoxMenuItem();
        jLogMessageMenuItem = new javax.swing.JCheckBoxMenuItem();
        jSeparator13 = new javax.swing.JSeparator();
        jWelcomePaneMenuItem = new javax.swing.JMenuItem();
        jHelpMenu = new javax.swing.JMenu();
        aboutMenuItem = new javax.swing.JMenuItem();

        getContentPane().setLayout(new javax.swing.BoxLayout(getContentPane(), javax.swing.BoxLayout.Y_AXIS));

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        getAccessibleContext().setAccessibleParent(this);
        jMenuToolar.setEnabled(false);
        jMenuToolar.setMaximumSize(new java.awt.Dimension(2000, 33));
        jMenuToolar.setMinimumSize(new java.awt.Dimension(100, 33));
        jMenuToolar.setPreferredSize(new java.awt.Dimension(600, 33));
        jSeparatorPane11.setMaximumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane11.setMinimumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane11.setPreferredSize(new java.awt.Dimension(5, 5));
        jMenuToolar.add(jSeparatorPane11);

        jNewTb.setFocusPainted(false);
        jNewTb.setFocusable(false);
        jNewTb.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jNewTb.setMargin(new java.awt.Insets(2, 5, 2, 5));
        jNewTb.setMaximumSize(new java.awt.Dimension(25, 25));
        jNewTb.setMinimumSize(new java.awt.Dimension(25, 25));
        jNewTb.setPreferredSize(new java.awt.Dimension(25, 25));
        jNewTb.setRequestFocusEnabled(false);
        jMenuToolar.add(jNewTb);

        jOpenTb.setFocusPainted(false);
        jOpenTb.setFocusable(false);
        jOpenTb.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jOpenTb.setMargin(new java.awt.Insets(2, 5, 2, 5));
        jOpenTb.setMaximumSize(new java.awt.Dimension(25, 25));
        jOpenTb.setMinimumSize(new java.awt.Dimension(25, 25));
        jOpenTb.setPreferredSize(new java.awt.Dimension(25, 25));
        jOpenTb.setRequestFocusEnabled(false);
        jMenuToolar.add(jOpenTb);

        jSaveTb.setFocusPainted(false);
        jSaveTb.setFocusable(false);
        jSaveTb.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jSaveTb.setMargin(new java.awt.Insets(2, 5, 2, 5));
        jSaveTb.setMaximumSize(new java.awt.Dimension(25, 25));
        jSaveTb.setMinimumSize(new java.awt.Dimension(25, 25));
        jSaveTb.setPreferredSize(new java.awt.Dimension(25, 25));
        jSaveTb.setRequestFocusEnabled(false);
        jMenuToolar.add(jSaveTb);

        jSaveAsTb.setFocusPainted(false);
        jSaveAsTb.setFocusable(false);
        jSaveAsTb.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jSaveAsTb.setMargin(new java.awt.Insets(2, 5, 2, 5));
        jSaveAsTb.setMaximumSize(new java.awt.Dimension(25, 25));
        jSaveAsTb.setMinimumSize(new java.awt.Dimension(25, 25));
        jSaveAsTb.setPreferredSize(new java.awt.Dimension(25, 25));
        jSaveAsTb.setRequestFocusEnabled(false);
        jMenuToolar.add(jSaveAsTb);

        jSeparatorPane10.setMaximumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane10.setMinimumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane10.setPreferredSize(new java.awt.Dimension(5, 5));
        jMenuToolar.add(jSeparatorPane10);

        jSeparator1.setOrientation(javax.swing.SwingConstants.VERTICAL);
        jSeparator1.setEnabled(false);
        jSeparator1.setMaximumSize(new java.awt.Dimension(2, 20));
        jSeparator1.setPreferredSize(new java.awt.Dimension(2, 20));
        jSeparator1.setRequestFocusEnabled(false);
        jMenuToolar.add(jSeparator1);

        jSeparatorPane8.setMaximumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane8.setMinimumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane8.setPreferredSize(new java.awt.Dimension(5, 5));
        jMenuToolar.add(jSeparatorPane8);

        jCopyBtn.setEnabled(false);
        jCopyBtn.setFocusPainted(false);
        jCopyBtn.setFocusable(false);
        jCopyBtn.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jCopyBtn.setMargin(new java.awt.Insets(2, 5, 2, 5));
        jCopyBtn.setMaximumSize(new java.awt.Dimension(25, 25));
        jCopyBtn.setMinimumSize(new java.awt.Dimension(25, 25));
        jCopyBtn.setPreferredSize(new java.awt.Dimension(25, 25));
        jCopyBtn.setRequestFocusEnabled(false);
        jMenuToolar.add(jCopyBtn);

        jPasteBtn.setEnabled(false);
        jPasteBtn.setFocusPainted(false);
        jPasteBtn.setFocusable(false);
        jPasteBtn.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jPasteBtn.setMargin(new java.awt.Insets(2, 5, 2, 5));
        jPasteBtn.setMaximumSize(new java.awt.Dimension(25, 25));
        jPasteBtn.setMinimumSize(new java.awt.Dimension(25, 25));
        jPasteBtn.setPreferredSize(new java.awt.Dimension(25, 25));
        jPasteBtn.setRequestFocusEnabled(false);
        jMenuToolar.add(jPasteBtn);

        jDeleteBtn.setEnabled(false);
        jDeleteBtn.setFocusPainted(false);
        jDeleteBtn.setFocusable(false);
        jDeleteBtn.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jDeleteBtn.setMargin(new java.awt.Insets(2, 5, 2, 5));
        jDeleteBtn.setMaximumSize(new java.awt.Dimension(25, 25));
        jDeleteBtn.setMinimumSize(new java.awt.Dimension(25, 25));
        jDeleteBtn.setPreferredSize(new java.awt.Dimension(25, 25));
        jDeleteBtn.setRequestFocusEnabled(false);
        jMenuToolar.add(jDeleteBtn);

        jSeparatorPane7.setMaximumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane7.setMinimumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane7.setPreferredSize(new java.awt.Dimension(5, 5));
        jMenuToolar.add(jSeparatorPane7);

        jSeparator10.setOrientation(javax.swing.SwingConstants.VERTICAL);
        jSeparator10.setEnabled(false);
        jSeparator10.setMaximumSize(new java.awt.Dimension(2, 20));
        jSeparator10.setPreferredSize(new java.awt.Dimension(2, 20));
        jSeparator10.setRequestFocusEnabled(false);
        jMenuToolar.add(jSeparator10);

        jSeparatorPane9.setMaximumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane9.setMinimumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane9.setPreferredSize(new java.awt.Dimension(5, 5));
        jMenuToolar.add(jSeparatorPane9);

        jUndoBtn.setEnabled(false);
        jUndoBtn.setFocusPainted(false);
        jUndoBtn.setFocusable(false);
        jUndoBtn.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jUndoBtn.setMargin(new java.awt.Insets(2, 5, 2, 5));
        jUndoBtn.setMaximumSize(new java.awt.Dimension(25, 25));
        jUndoBtn.setMinimumSize(new java.awt.Dimension(25, 25));
        jUndoBtn.setPreferredSize(new java.awt.Dimension(25, 25));
        jUndoBtn.setRequestFocusEnabled(false);
        jMenuToolar.add(jUndoBtn);

        jRedoBtn.setEnabled(false);
        jRedoBtn.setFocusPainted(false);
        jRedoBtn.setFocusable(false);
        jRedoBtn.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jRedoBtn.setMargin(new java.awt.Insets(2, 5, 2, 5));
        jRedoBtn.setMaximumSize(new java.awt.Dimension(25, 25));
        jRedoBtn.setMinimumSize(new java.awt.Dimension(25, 25));
        jRedoBtn.setPreferredSize(new java.awt.Dimension(25, 25));
        jRedoBtn.setRequestFocusEnabled(false);
        jMenuToolar.add(jRedoBtn);

        jSeparatorPane5.setMaximumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane5.setMinimumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane5.setPreferredSize(new java.awt.Dimension(5, 5));
        jMenuToolar.add(jSeparatorPane5);

        jSeparator8.setOrientation(javax.swing.SwingConstants.VERTICAL);
        jSeparator8.setEnabled(false);
        jSeparator8.setMaximumSize(new java.awt.Dimension(2, 20));
        jSeparator8.setPreferredSize(new java.awt.Dimension(2, 20));
        jSeparator8.setRequestFocusEnabled(false);
        jMenuToolar.add(jSeparator8);

        jSeparatorPane.setMaximumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane.setMinimumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane.setPreferredSize(new java.awt.Dimension(5, 5));
        jMenuToolar.add(jSeparatorPane);

        jSubmitBtn.setFocusPainted(false);
        jSubmitBtn.setFocusable(false);
        jSubmitBtn.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jSubmitBtn.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jSubmitBtn.setMaximumSize(new java.awt.Dimension(25, 25));
        jSubmitBtn.setMinimumSize(new java.awt.Dimension(25, 25));
        jSubmitBtn.setPreferredSize(new java.awt.Dimension(25, 25));
        jSubmitBtn.setRequestFocusEnabled(false);
        jMenuToolar.add(jSubmitBtn);

        jSeparatorPane1.setMaximumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane1.setMinimumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane1.setPreferredSize(new java.awt.Dimension(5, 5));
        jMenuToolar.add(jSeparatorPane1);

        jSeparator5.setOrientation(javax.swing.SwingConstants.VERTICAL);
        jSeparator5.setEnabled(false);
        jSeparator5.setMaximumSize(new java.awt.Dimension(2, 20));
        jSeparator5.setPreferredSize(new java.awt.Dimension(2, 20));
        jSeparator5.setRequestFocusEnabled(false);
        jMenuToolar.add(jSeparator5);

        jSeparatorPane4.setMaximumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane4.setMinimumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane4.setPreferredSize(new java.awt.Dimension(5, 5));
        jMenuToolar.add(jSeparatorPane4);

        jSeparatorPane6.setMaximumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane6.setMinimumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane6.setPreferredSize(new java.awt.Dimension(5, 5));
        jMenuToolar.add(jSeparatorPane6);

        jSeparatorPane2.setMaximumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane2.setMinimumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane2.setPreferredSize(new java.awt.Dimension(5, 5));
        jMenuToolar.add(jSeparatorPane2);

        jZoomActualSizeBtn.setEnabled(false);
        jZoomActualSizeBtn.setFocusPainted(false);
        jZoomActualSizeBtn.setFocusable(false);
        jZoomActualSizeBtn.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jZoomActualSizeBtn.setMargin(new java.awt.Insets(2, 5, 2, 5));
        jZoomActualSizeBtn.setMaximumSize(new java.awt.Dimension(25, 25));
        jZoomActualSizeBtn.setMinimumSize(new java.awt.Dimension(25, 25));
        jZoomActualSizeBtn.setPreferredSize(new java.awt.Dimension(25, 25));
        jZoomActualSizeBtn.setRequestFocusEnabled(false);
        jMenuToolar.add(jZoomActualSizeBtn);

        jZoomInBtn.setEnabled(false);
        jZoomInBtn.setFocusPainted(false);
        jZoomInBtn.setFocusable(false);
        jZoomInBtn.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jZoomInBtn.setMargin(new java.awt.Insets(2, 5, 2, 5));
        jZoomInBtn.setMaximumSize(new java.awt.Dimension(25, 25));
        jZoomInBtn.setMinimumSize(new java.awt.Dimension(25, 25));
        jZoomInBtn.setPreferredSize(new java.awt.Dimension(25, 25));
        jZoomInBtn.setRequestFocusEnabled(false);
        jMenuToolar.add(jZoomInBtn);

        jZoomOutBtn.setEnabled(false);
        jZoomOutBtn.setFocusPainted(false);
        jZoomOutBtn.setFocusable(false);
        jZoomOutBtn.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jZoomOutBtn.setMargin(new java.awt.Insets(2, 5, 2, 5));
        jZoomOutBtn.setMaximumSize(new java.awt.Dimension(25, 25));
        jZoomOutBtn.setMinimumSize(new java.awt.Dimension(25, 25));
        jZoomOutBtn.setPreferredSize(new java.awt.Dimension(25, 25));
        jZoomOutBtn.setRequestFocusEnabled(false);
        jMenuToolar.add(jZoomOutBtn);

        jSeparatorPane3.setMaximumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane3.setMinimumSize(new java.awt.Dimension(5, 5));
        jSeparatorPane3.setPreferredSize(new java.awt.Dimension(5, 5));
        jMenuToolar.add(jSeparatorPane3);

        jMakeDemoGraph.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/eclipse/menu/ch_callers.gif")));
        jMakeDemoGraph.setEnabled(false);
        jMakeDemoGraph.setFocusPainted(false);
        jMakeDemoGraph.setFocusable(false);
        jMakeDemoGraph.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jMakeDemoGraph.setMargin(new java.awt.Insets(2, 5, 2, 5));
        jMakeDemoGraph.setMaximumSize(new java.awt.Dimension(25, 25));
        jMakeDemoGraph.setMinimumSize(new java.awt.Dimension(25, 25));
        jMakeDemoGraph.setPreferredSize(new java.awt.Dimension(25, 25));
        jMakeDemoGraph.setRequestFocusEnabled(false);
        jMakeDemoGraph.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMakeDemoGraphActionPerformed(evt);
            }
        });

        jMenuToolar.add(jMakeDemoGraph);

        getContentPane().add(jMenuToolar);

        jBottomPane.setLayout(new javax.swing.BoxLayout(jBottomPane, javax.swing.BoxLayout.X_AXIS));

        jControlPane.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER, 5, 10));

        jControlPane.setMaximumSize(new java.awt.Dimension(27, 32767));
        jControlPane.setMinimumSize(new java.awt.Dimension(25, 30));
        jControlPane.setOpaque(false);
        jControlPane.setPreferredSize(new java.awt.Dimension(27, 30));
        jResourceInfoTBtn.setBorderPainted(false);
        jResourceInfoTBtn.setFocusable(false);
        jResourceInfoTBtn.setMaximumSize(new java.awt.Dimension(20, 20));
        jResourceInfoTBtn.setMinimumSize(new java.awt.Dimension(20, 20));
        jResourceInfoTBtn.setPreferredSize(new java.awt.Dimension(20, 20));
        jControlPane.add(jResourceInfoTBtn);

        jActionInfoTBtn.setBorderPainted(false);
        jActionInfoTBtn.setFocusable(false);
        jActionInfoTBtn.setMaximumSize(new java.awt.Dimension(20, 20));
        jActionInfoTBtn.setMinimumSize(new java.awt.Dimension(20, 20));
        jActionInfoTBtn.setPreferredSize(new java.awt.Dimension(20, 20));
        jControlPane.add(jActionInfoTBtn);

        jSeparator11.setMaximumSize(new java.awt.Dimension(25, 2));
        jSeparator11.setPreferredSize(new java.awt.Dimension(20, 2));
        jControlPane.add(jSeparator11);

        jLogMessageTBtn.setBorderPainted(false);
        jLogMessageTBtn.setFocusable(false);
        jLogMessageTBtn.setMaximumSize(new java.awt.Dimension(20, 20));
        jLogMessageTBtn.setMinimumSize(new java.awt.Dimension(20, 20));
        jLogMessageTBtn.setPreferredSize(new java.awt.Dimension(20, 20));
        jControlPane.add(jLogMessageTBtn);

        jBottomPane.add(jControlPane);

        jSeparatorPanel.setLayout(new javax.swing.BoxLayout(jSeparatorPanel, javax.swing.BoxLayout.X_AXIS));

        jSeparatorPanel.setMaximumSize(new java.awt.Dimension(2, 2147483647));
        jSeparatorPanel.setMinimumSize(new java.awt.Dimension(2, 0));
        jSeparatorPanel.setOpaque(false);
        jSeparatorPanel.setPreferredSize(new java.awt.Dimension(2, 500));
        jSeparator7.setOrientation(javax.swing.SwingConstants.VERTICAL);
        jSeparator7.setMaximumSize(new java.awt.Dimension(1, 1000));
        jSeparator7.setMinimumSize(new java.awt.Dimension(1, 200));
        jSeparator7.setPreferredSize(new java.awt.Dimension(1, 500));
        jSeparatorPanel.add(jSeparator7);

        jBottomPane.add(jSeparatorPanel);

        jRightPane.setLayout(new javax.swing.BoxLayout(jRightPane, javax.swing.BoxLayout.Y_AXIS));

        jContentPane.setLayout(new javax.swing.BoxLayout(jContentPane, javax.swing.BoxLayout.X_AXIS));

        jContentPane.setPreferredSize(new java.awt.Dimension(630, 405));
        jBase.setLayout(new java.awt.GridLayout(1, 1));

        jBase.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                jBaseComponentResized(evt);
            }
        });

        jContentPane.add(jBase);

        jRightPane.add(jContentPane);

        jStatus.setLayout(new java.awt.GridLayout(1, 0));

        jStatus.setMaximumSize(new java.awt.Dimension(32767, 20));
        jStatus.setMinimumSize(new java.awt.Dimension(100, 20));
        jStatus.setPreferredSize(new java.awt.Dimension(600, 20));
        jRightPane.add(jStatus);

        jBottomPane.add(jRightPane);

        getContentPane().add(jBottomPane);

        menuBar.setMaximumSize(new java.awt.Dimension(2000, 30));
        menuBar.setMinimumSize(new java.awt.Dimension(100, 30));
        menuBar.setPreferredSize(new java.awt.Dimension(600, 25));
        jFileMenu.setMnemonic('f');
        jFileMenu.setText(java.util.ResourceBundle.getBundle("bundle/dynamicScenario").getString("frame.main.menu.file"));
        jFileMenu.setToolTipText(java.util.ResourceBundle.getBundle("bundle/dynamicScenario").getString("frame.main.menu.file.tip"));
        jNewMenuItem.setIconTextGap(0);
        jNewMenuItem.setMargin(new java.awt.Insets(4, 2, 4, 2));
        jFileMenu.add(jNewMenuItem);

        jOpenMenuItem.setIconTextGap(0);
        jOpenMenuItem.setMargin(new java.awt.Insets(4, 2, 4, 2));
        jFileMenu.add(jOpenMenuItem);

        jCloseMenuItem.setIconTextGap(0);
        jCloseMenuItem.setMargin(new java.awt.Insets(4, 2, 4, 2));
        jFileMenu.add(jCloseMenuItem);

        jFileMenu.add(jSeparator4);

        jSaveMenuItem.setIconTextGap(0);
        jSaveMenuItem.setMargin(new java.awt.Insets(4, 2, 4, 2));
        jFileMenu.add(jSaveMenuItem);

        jSaveAsMenuItem.setIconTextGap(0);
        jSaveAsMenuItem.setMargin(new java.awt.Insets(4, 2, 4, 2));
        jFileMenu.add(jSaveAsMenuItem);

        jFileMenu.add(jSeparator3);

        jExit.setAlignmentY(1.5F);
        jExit.setIconTextGap(0);
        jExit.setMargin(new java.awt.Insets(4, 2, 4, 2));
        jFileMenu.add(jExit);

        menuBar.add(jFileMenu);

        jEditMenu.setMnemonic('e');
        jEditMenu.setText(java.util.ResourceBundle.getBundle("bundle/dynamicScenario").getString("menu.edit.name"));
        jEditMenu.setToolTipText(java.util.ResourceBundle.getBundle("bundle/dynamicScenario").getString("menu.edit.tip"));
        jEditMenu.setAlignmentX(2.0F);
        jEditMenu.setMargin(new java.awt.Insets(2, 4, 2, 2));
        jUndoMenuItem.setAlignmentX(2.0F);
        jUndoMenuItem.setMargin(new java.awt.Insets(4, 2, 2, 4));
        jEditMenu.add(jUndoMenuItem);

        jRedoMenuItem.setAlignmentX(2.0F);
        jRedoMenuItem.setMargin(new java.awt.Insets(4, 2, 2, 4));
        jEditMenu.add(jRedoMenuItem);

        jSeparator2.setAlignmentX(2.0F);
        jEditMenu.add(jSeparator2);

        jCopyMenuItem.setAlignmentX(2.0F);
        jCopyMenuItem.setMargin(new java.awt.Insets(4, 2, 2, 4));
        jEditMenu.add(jCopyMenuItem);

        jPaseMenuItem.setAlignmentX(2.0F);
        jPaseMenuItem.setMargin(new java.awt.Insets(4, 2, 2, 4));
        jEditMenu.add(jPaseMenuItem);

        jDeleteMenuItem.setAlignmentX(2.0F);
        jDeleteMenuItem.setMargin(new java.awt.Insets(4, 2, 2, 4));
        jEditMenu.add(jDeleteMenuItem);

        menuBar.add(jEditMenu);

        jScenarioMenu.setMnemonic('s');
        jScenarioMenu.setText(java.util.ResourceBundle.getBundle("bundle/dynamicScenario").getString("menu.tools.scenario.menu.name"));
        jSubmitScenarioItem.setIconTextGap(0);
        jSubmitScenarioItem.setMargin(new java.awt.Insets(4, 2, 4, 2));
        jScenarioMenu.add(jSubmitScenarioItem);

        jScenarioMenu.add(jSeparator6);

        jScenarioPropertiesItem.setMargin(new java.awt.Insets(4, 2, 4, 2));
        jScenarioMenu.add(jScenarioPropertiesItem);

        menuBar.add(jScenarioMenu);

        jViewMenu.setMnemonic('v');
        jViewMenu.setText(java.util.ResourceBundle.getBundle("bundle/dynamicScenario").getString("menu.view.name"));
        jViewMenu.setToolTipText(java.util.ResourceBundle.getBundle("bundle/dynamicScenario").getString("menu.view.tip"));
        jActualSizeMenuItem.setMargin(new java.awt.Insets(4, 2, 4, 2));
        jViewMenu.add(jActualSizeMenuItem);

        jZoomInMenuItem.setMargin(new java.awt.Insets(4, 2, 4, 2));
        jViewMenu.add(jZoomInMenuItem);

        jZoomOutMenuItem.setMargin(new java.awt.Insets(4, 2, 4, 2));
        jViewMenu.add(jZoomOutMenuItem);

        jViewMenu.add(jSeparator12);

        jResourceInfoMenuItem.setText("CheckBox");
        jViewMenu.add(jResourceInfoMenuItem);

        jActionInfoMenuItem.setText("CheckBox");
        jViewMenu.add(jActionInfoMenuItem);

        jLogMessageMenuItem.setText("CheckBox");
        jViewMenu.add(jLogMessageMenuItem);

        jViewMenu.add(jSeparator13);

        jWelcomePaneMenuItem.setMargin(new java.awt.Insets(4, 2, 4, 2));
        jViewMenu.add(jWelcomePaneMenuItem);

        menuBar.add(jViewMenu);

        jHelpMenu.setMnemonic('H');
        jHelpMenu.setText(java.util.ResourceBundle.getBundle("bundle/dynamicScenario").getString("frame.main.menu.help"));
        jHelpMenu.setToolTipText(java.util.ResourceBundle.getBundle("bundle/dynamicScenario").getString("frame.main.menu.help.tip"));
        aboutMenuItem.setText("About ...");
        aboutMenuItem.setMargin(new java.awt.Insets(4, 2, 4, 2));
        aboutMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aboutMenuItemActionPerformed(evt);
            }
        });

        jHelpMenu.add(aboutMenuItem);

        menuBar.add(jHelpMenu);

        setJMenuBar(menuBar);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-800)/2, (screenSize.height-600)/2, 800, 600);
    }// </editor-fold>//GEN-END:initComponents
    
    private void jBaseComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_jBaseComponentResized
        
        
        
    }//GEN-LAST:event_jBaseComponentResized
    
    private void jMakeDemoGraphActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMakeDemoGraphActionPerformed
        try{
            Device device[] = DeviceFactory.getInstance().getDeviceList();
            int size = device != null ? device.length : 0;
            for (int i = 0; i < size; i++) {
                
                ResourceDocument.Resource resource =  ResourceFactory.getResourceById(12);
                
                VLabVertexType vertexType = new ExperimentType(resource.getTypeId());
                ScenarioCellObject cellObject = new ScenarioCellObject(vertexType);
                resource.setNodeId(ResourceFactory.getNextResourceId());
                //set the resource described by this cell
                resource.setId(i+1);
                cellObject.setResource(resource);
                
                //set label
                cellObject.putProperty(ScenarioCellObject.keyValue, resource.getName());
                // removed 10.1 Map map = graph.getModel().createAttributes();
                Map map = new AttributeMap();
                GraphConstants.setIcon(map, GraphHelper.getResourceImage(vertexType.getTypeId()).getImage());
                ScenarioGraph graph = (ScenarioGraph) scenGraphEditor.getGraph();
                graph.setSelectionAttributes((AttributeMap) map);
                DefaultGraphCell imageGraphCell =  GraphCellFactory.createCell(vertexType, cellObject);
                
                //imageGraphCell.setUserObject(cellObject);
                
                Point2D p = ( (ScenarioGraphUI) graph.getUI()).rescale(device[i].getLocation().getX(), device[i].getLocation().getY() );
                log.debug("Point = "+p);
                // AttributeMap cellAttrMap = imageGraphCell.getAttributes();
                // GraphConstants.setBounds(cellAttrMap,
                ScenarioGraphEditorFactory.getGraphEditorInstance().insert( p , imageGraphCell);
                
            }
        }catch(Exception e){
            e.printStackTrace();
            Message.showInfoMessage(this, e.getMessage(), Message.MESSAGE_TYPE_ERROR);
        }
    }//GEN-LAST:event_jMakeDemoGraphActionPerformed
        
    private void aboutMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aboutMenuItemActionPerformed
        // Add your handling code here:
        new AboutBox(this,true).show();
    }//GEN-LAST:event_aboutMenuItemActionPerformed
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        ActionFactory.getAction(ActionFactory.ACTION_APP_EXIT).actionPerformed(null);
    }//GEN-LAST:event_exitForm
    
    
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem aboutMenuItem;
    private javax.swing.JCheckBoxMenuItem jActionInfoMenuItem;
    private javax.swing.JToggleButton jActionInfoTBtn;
    private javax.swing.JMenuItem jActualSizeMenuItem;
    private javax.swing.JPanel jBase;
    private javax.swing.JPanel jBottomPane;
    private javax.swing.JMenuItem jCloseMenuItem;
    private javax.swing.JPanel jContentPane;
    private javax.swing.JPanel jControlPane;
    private javax.swing.JButton jCopyBtn;
    private javax.swing.JMenuItem jCopyMenuItem;
    private javax.swing.JButton jDeleteBtn;
    private javax.swing.JMenuItem jDeleteMenuItem;
    private javax.swing.JMenu jEditMenu;
    private javax.swing.JMenuItem jExit;
    private javax.swing.JMenu jFileMenu;
    private javax.swing.JMenu jHelpMenu;
    private javax.swing.JCheckBoxMenuItem jLogMessageMenuItem;
    private javax.swing.JToggleButton jLogMessageTBtn;
    private javax.swing.JButton jMakeDemoGraph;
    private javax.swing.JToolBar jMenuToolar;
    private javax.swing.JMenuItem jNewMenuItem;
    private javax.swing.JButton jNewTb;
    private javax.swing.JMenuItem jOpenMenuItem;
    private javax.swing.JButton jOpenTb;
    private javax.swing.JMenuItem jPaseMenuItem;
    private javax.swing.JButton jPasteBtn;
    private javax.swing.JButton jRedoBtn;
    private javax.swing.JMenuItem jRedoMenuItem;
    private javax.swing.JCheckBoxMenuItem jResourceInfoMenuItem;
    private javax.swing.JToggleButton jResourceInfoTBtn;
    private javax.swing.JPanel jRightPane;
    private javax.swing.JMenuItem jSaveAsMenuItem;
    private javax.swing.JButton jSaveAsTb;
    private javax.swing.JMenuItem jSaveMenuItem;
    private javax.swing.JButton jSaveTb;
    private javax.swing.JMenu jScenarioMenu;
    private javax.swing.JMenuItem jScenarioPropertiesItem;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator10;
    private javax.swing.JSeparator jSeparator11;
    private javax.swing.JSeparator jSeparator12;
    private javax.swing.JSeparator jSeparator13;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JSeparator jSeparator3;
    private javax.swing.JSeparator jSeparator4;
    private javax.swing.JSeparator jSeparator5;
    private javax.swing.JSeparator jSeparator6;
    private javax.swing.JSeparator jSeparator7;
    private javax.swing.JSeparator jSeparator8;
    private javax.swing.JPanel jSeparatorPane;
    private javax.swing.JPanel jSeparatorPane1;
    private javax.swing.JPanel jSeparatorPane10;
    private javax.swing.JPanel jSeparatorPane11;
    private javax.swing.JPanel jSeparatorPane2;
    private javax.swing.JPanel jSeparatorPane3;
    private javax.swing.JPanel jSeparatorPane4;
    private javax.swing.JPanel jSeparatorPane5;
    private javax.swing.JPanel jSeparatorPane6;
    private javax.swing.JPanel jSeparatorPane7;
    private javax.swing.JPanel jSeparatorPane8;
    private javax.swing.JPanel jSeparatorPane9;
    private javax.swing.JPanel jSeparatorPanel;
    private javax.swing.JPanel jStatus;
    private javax.swing.JButton jSubmitBtn;
    private javax.swing.JMenuItem jSubmitScenarioItem;
    private javax.swing.JButton jUndoBtn;
    private javax.swing.JMenuItem jUndoMenuItem;
    private javax.swing.JMenu jViewMenu;
    private javax.swing.JMenuItem jWelcomePaneMenuItem;
    private javax.swing.JButton jZoomActualSizeBtn;
    private javax.swing.JButton jZoomInBtn;
    private javax.swing.JMenuItem jZoomInMenuItem;
    private javax.swing.JButton jZoomOutBtn;
    private javax.swing.JMenuItem jZoomOutMenuItem;
    private javax.swing.JMenuBar menuBar;
    // End of variables declaration//GEN-END:variables
    
    
    
    
    //--------------------------------------------------------------------------
    //----- Public helper methods
    
    
    /** Handles the critical error - prints the message and closes
     * the application.
     * @param message message which will be printed
     */
    public void criticalError(String message, String details){
        try{
            log.debug("<criticalError>");
            if(!FormTools.isValidateFormStringNull(details) ){
                message = new StringBuffer(message).append(details).toString();
            }
            Message.showInfoMessage(this, message, Message.MESSAGE_TYPE_ERROR);
            
            //removes the session entry
            SessionFactory.removeSession();
            
            this.setVisible(false);
            this.dispose();
            log.debug("</criticalError>");
            System.exit(Keys.EXIT_CODE_ERROR);
        }catch (Exception e){
            e.printStackTrace();
            Message.showInfoMessage(this, e.getLocalizedMessage(), Message.MESSAGE_TYPE_ERROR);
            this.setVisible(false);
            this.dispose();
            System.exit(Keys.EXIT_CODE_ERROR);
        }
    }
    
    
    /**
     * Handles invalid seesion critical error
     * @param exception
     */
    public void sessionError(Exception exception) {
        criticalError(AppMessage.MSG_ERR_INVALID_SESSION, exception.getMessage());
    }
    
    /**
     * Installes an instance of Scenario Editor Pane in the
     * scroll pane and place it on the main frame
     * @param graphEditor
     * @throws SSAException
     */
    public ScenarioGraphEditor installGraph() throws SSAException{
        if(this.scenGraphEditor != null){
            ((MainFrameBase) mainFrameBase).remove(this.scenGraphEditor);
        }
        ((MainFrameBase) mainFrameBase).remove(this.welcomePane);
        
        this.scenGraphEditor = ScenarioGraphEditorFactory.createNewInstance();
        ((MainFrameBase) mainFrameBase).addScenarioGraph(scenGraphEditor);
        //((MainFrameBase) mainFrameBase).setTitle(SSABundleFactory.getValue("frame.main.graph.design.pane"));
        ((MainFrameBase) mainFrameBase).initResourceInfo(this.scenGraphEditor);
        // jBaseComponentResized(null);
        updateTitle(null);
        return this.scenGraphEditor;
    }
    
    
    
    /** Adds a welcome pane into the main content pane */
    public void addWelcomePane() throws SSAException{
        ScenarioGraphEditorFactory.createNewInstance();
        this.welcomePane  = new WelcomePane();
        ((MainFrameBase) mainFrameBase).add(welcomePane);
        ((MainFrameBase) mainFrameBase).setTitle(java.util.ResourceBundle.getBundle("pl/psnc/vlab/ssa/gui/main/WelcomePane").getString("welcome.pane.title"), false);
        updateTitle(null);
    }
    
    
    /**
     * Update the main window title
     * @param name
     */
    public void updateTitle(String name){
        log.debug("<updateTitle:"+name+">");
        if(FormTools.isValidateFormStringNull(name)){
            name = ResourceBundle.getBundle("bundle/dynamicScenario").getString("scenario.new.file.name");
        }
        
        
        this.setTitle(new StringBuffer()
        .append(ResourceBundle.getBundle("bundle/dynamicScenario").getString("main.frame.title"))
        .append(" - [")
        .append(name)
        .append("]").toString());
    }
    
    
    /**
     * Updates main thread after finishing the Long Task
     * @param taskContext
     */
    public void update(LongTask taskContext){
        log.debug("<update>");
        // tbd OSA
        // przemyslec konczenie watkow i informaowanie o rezultacie usera
        this.statusBar.detachProgressBar();
        this.jSubmitBtn.setEnabled(true);
        
        log.debug("was thrown:"+taskContext.exceptionThrown());
        if(taskContext != null && taskContext.exceptionThrown() ){
            log.debug("in EX");
            SSAException dynExcp = (SSAException ) taskContext.getException();
            this.statusBar.setMessage("");
            Message.showInfoMessage(this, taskContext.getResult(), Message.MESSAGE_TYPE_ERROR);
            log.debug("</update:exceptionThrown>");
            return;
        }
        String message = MessageBundleFactory.getValue((String) taskContext.getResult());
        this.statusBar.setMessage(message);
        Message.showInfoMessage(this, message, Message.MESSAGE_TYPE_INF);
        log.debug("</update>");
    }
    
    
    /**
     * Start the scenario timer, which periodically checks the
     * scenario submissin thread for the result.
     * @return
     */
    public void startScenarioTimer(){
        //this.scenarioTimer.start();
    }
    
    
    
    //--------------------------------------------------------------------------
    //----- getters /setters
    
    
    /**
     * Returns the Main Frame Base Pane, where the Graph Design Pane
     * is installed
     * @return MainFrameBase pane (central pane of the SSA)
     */
    public MainFrameBase getBasePane(){
        return (MainFrameBase) mainFrameBase;
    }
    
    
    /** Returns an instacne of the status bar attached to the main frame
     * @return StatusBar instance
     */
    public static StatusBar getStatusBar(){
        return statusBar;
    }
    
    /** Get instance of MainFrame */
    public static MainFrame getInstance(){
        if(instance == null){
            instance = new MainFrame();
        }
        return instance;
    }
    
    
    //--------------------------------------------------------------------------
    //----- Private, internal helper methods
    
    
    /** Method is called only once - when the application starts. Among
     * other things it checks wheter the session is valid.
     */
    private  void initAppData(final JDialog splash) throws SSAException{
        //start session manager thread
        int delay = Keys.INIT_APP_DATA_TIMER_DELAY; //milliseconds
        final AppInitializer appInitializer = new AppInitializer((SplashDialog) splash);
        
        ActionListener performer = new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                if(appInitializer!= null && appInitializer.isDone()){
                    Object o = evt.getSource();
                    if (o instanceof Timer){
                        log.debug("Timer instance");
                        ((Timer)o).stop();
                        splash.setVisible(false);
                        splash.dispose();
                        checkResourceActions();
                        show();
                    } else {
                        log.debug("Not Timer instance");
                    }
                    splash.setVisible(false);
                    splash.dispose();
                    return;
                }
            }
        };
        
        Timer initTimer = new Timer(delay, performer);
        initTimer.start();
        splash.show();
        
    }
    
    
    
    
    /** Helper method for Gui initialization
     */
    private void initGui(){
        //set vlab logo
        this.setIconImage(ImageFactory.getImageBean(ImageKeys.LOGO_32_VLAB_LOGO_ICON).getImage().getImage());
        
        jExit.setIcon( ImageFactory.getImageBean(ImageKeys.SMALL_ICON_BLANK).getImage() );
        
        menuBar.putClientProperty(Options.HEADER_STYLE_KEY, HeaderStyle.BOTH);
        jHelpMenu.putClientProperty(Options.NO_ICONS_KEY, Boolean.TRUE);
        
        //set the border style
        jEditMenu.putClientProperty(PlasticLookAndFeel.BORDER_STYLE_KEY, BorderStyle.ETCHED);
               
        //toolbar
        jMenuToolar.putClientProperty(Options.HEADER_STYLE_KEY, HeaderStyle.BOTH);
        jMenuToolar.setRollover(true);
        //jMenuToolar.putClientProperty(PlasticLookAndFeel.BORDER_STYLE_KEY, BorderStyle.ETCHED);
        
    }
    
    
    /** Initialize the components actions
     */
    private void initActions(){
        //--- Toolbars
        //new scenario action
        Action newScenAction = new NewScenarioAction(this);
        ActionFactory.putAction(ActionFactory.ACTION_NEW_SCENARIO, newScenAction);
        GuiTools.setAction(jNewTb, newScenAction, true);             //new graph
        GuiTools.setAction(jNewMenuItem, newScenAction, true);       //new graph
        
        //clsoe scenario action
        Action closeScenAction = new CloseScenarioAction(this);
        ActionFactory.putAction(ActionFactory.ACTION_CLOSE_SCENARIO, closeScenAction);
        GuiTools.setAction(jCloseMenuItem, closeScenAction, false, true);  //close scenario
        
        
        //save scenario action
        Action saveScenAction = new SaveFileAction(this);
        ActionFactory.putAction(ActionFactory.ACTION_SAVE_FILE, saveScenAction);
        GuiTools.setAction(jSaveTb, saveScenAction, false, true);             //save
        GuiTools.setAction(jSaveMenuItem, saveScenAction, false, true);       //save
        
        //save as scenario action
        Action saveAsScenAction = new SaveFileAsAction(this);
        ActionFactory.putAction(ActionFactory.ACTION_SAVE_FILE_AS, saveAsScenAction);
        GuiTools.setAction(jSaveAsTb, saveAsScenAction, false, true);             //save
        GuiTools.setAction(jSaveAsMenuItem, saveAsScenAction, false, true);       //save
        
        
        //open scenario action
        Action openScenAction = new OpenFileAction(this);
        ActionFactory.putAction(ActionFactory.ACTION_OPEN_FILE, openScenAction);
        GuiTools.setAction(jOpenTb, openScenAction, true, true);             //open
        GuiTools.setAction(jOpenMenuItem, openScenAction, true, true);       //open
        
        
        GuiTools.setAction(jSubmitBtn, ActionFactory.getAction(ActionFactory.ACTION_SUBMIT_SCENARIO), false, true);    //submit scenario
        GuiTools.setAction(jSubmitScenarioItem, ActionFactory.getAction(ActionFactory.ACTION_SUBMIT_SCENARIO), false, true);    //submit scenario
        
        // removed GuiTools.setAction(jAddVertexComputeBtn,  ActionFactory.getAction(ActionFactory.ACTION_ADD_VERTEX_COMPUTE), false);  //add vertex
        // removed GuiTools.setAction(jAddVertexExpBtn, ActionFactory.getAction(ActionFactory.ACTION_ADD_VERTEX_EXPERIMENT), false);
        
        GuiTools.setAction(jUndoBtn, ActionFactory.getAction(ActionFactory.ACTION_UNDO), false, true);         //undo
        GuiTools.setAction(jUndoMenuItem, ActionFactory.getAction(ActionFactory.ACTION_UNDO), false, true);    //unymi  do
        
        GuiTools.setAction(jRedoBtn, ActionFactory.getAction(ActionFactory.ACTION_REDO), false, true );         //redo
        GuiTools.setAction(jRedoMenuItem, ActionFactory.getAction(ActionFactory.ACTION_REDO), false, true );    //redo
        
        GuiTools.setAction(jZoomActualSizeBtn, ActionFactory.getAction(ActionFactory.ACTION_ZOOM_ACTUAL), false, true);    //actual size
        GuiTools.setAction(jActualSizeMenuItem, ActionFactory.getAction(ActionFactory.ACTION_ZOOM_ACTUAL), false, true);   //actual size
        
        GuiTools.setAction(jZoomInBtn, ActionFactory.getAction(ActionFactory.ACTION_ZOOM_IN), false, true );             //zoon in
        GuiTools.setAction(jZoomInMenuItem, ActionFactory.getAction(ActionFactory.ACTION_ZOOM_IN), false, true);        //zoon in
        
        GuiTools.setAction(jZoomOutBtn, ActionFactory.getAction(ActionFactory.ACTION_ZOOM_OUT), false, true);            //zoom out
        GuiTools.setAction(jZoomOutMenuItem, ActionFactory.getAction(ActionFactory.ACTION_ZOOM_OUT), false, true);       //zoom out
        
        GuiTools.setAction(jCopyBtn, ActionFactory.getAction(ActionFactory.ACTION_COPY), false, true );            //copy
        GuiTools.setAction(jCopyMenuItem, ActionFactory.getAction(ActionFactory.ACTION_COPY), false, true );       //copy
        
        GuiTools.setAction(jPasteBtn, ActionFactory.getAction(ActionFactory.ACTION_PASTE), false, true );          //paste
        GuiTools.setAction(jPaseMenuItem, ActionFactory.getAction(ActionFactory.ACTION_PASTE), false, true );      //paste
        
        GuiTools.setAction(jDeleteBtn, ActionFactory.getAction(ActionFactory.ACTION_DELETE), false, true );        //delete
        GuiTools.setAction(jDeleteMenuItem, ActionFactory.getAction(ActionFactory.ACTION_DELETE), false, true );   //delete
        
        //add app close action
        Action closeAppAction = new CloseAppAction(this);
        ActionFactory.putAction(ActionFactory.ACTION_APP_EXIT, closeAppAction);
        GuiTools.setAction(jExit, closeAppAction, true);   //close app
        
        GuiTools.setAction(jScenarioPropertiesItem, ActionFactory.getAction(ActionFactory.ACTION_SCENARIO_PROPERTIES), false, true );   //scenario properties
        
        //perspective
        ResInfPerspAction resOutlineAction = new ResInfPerspAction(new Object[]{jResourceInfoTBtn, jResourceInfoMenuItem}, (MainFrameBase) mainFrameBase);
        ActionFactory.putAction(ActionFactory.ACTION_RESOURCE_OUTLINE, resOutlineAction);
        GuiTools.setAction(jResourceInfoTBtn, resOutlineAction, false , true);  //show resource inf persp.
        GuiTools.setAction(jResourceInfoMenuItem, resOutlineAction, false, true);  //show resource inf persp.
        
        //message log perspective
        MessageLogPerspAction msgLogAction = new MessageLogPerspAction( (MainFrameBase) mainFrameBase);
        ActionFactory.putAction(ActionFactory.ACTION_MESSAGE_LOG_OUTLINE, msgLogAction);
        GuiTools.setAction(jLogMessageTBtn, msgLogAction, true, true );  //show resource inf persp.
        GuiTools.setAction(jLogMessageMenuItem, msgLogAction, true, true );  //show resource inf persp.
        
        //available actions
        Action availActions = new AvailableActionsPerpAction( (MainFrameBase) mainFrameBase);
        ActionFactory.putAction(ActionFactory.ACTION_AVAILABLE_ACTIONS_OUTLINE, availActions);
        GuiTools.setAction(jActionInfoTBtn, availActions, false, true);  //show resource inf persp.
        GuiTools.setAction(jActionInfoMenuItem, availActions, false, true);  //show resource inf persp.
        
        //welcome screen
        Action welcomeAction = new WelcomePaneAction( this);
        ActionFactory.putAction(ActionFactory.ACTION_WELCOME_SCREEN, welcomeAction);
        GuiTools.setAction(jWelcomePaneMenuItem, welcomeAction, false, true);  //show resource inf persp.
        
    }
    
    
    /**
     * Initializes application status bar
     */
    private void initStatusBar() throws SQLException{
        SystemConfigurationValue systemConfVO = SystemConfigurationFactory.getInstance();
        TimeZoneTools timeZoneTools = new TimeZoneTools();
        TimeZone baseTimeZone = timeZoneTools.getTimeZone(PropertiesFactory.getTimeZone());
        boolean isSummerTime = PropertiesFactory.isSummerTime();
        statusBar = new StatusBar(systemConfVO.getCurrentLocale(), TimeZone.getDefault(), baseTimeZone, isSummerTime);
        jStatus.add(statusBar);
        //set user name
        log.debug("Setting user name to-"+systemConfVO.getUserName());
        statusBar.setUserName(systemConfVO.getUserName());
        statusBar.setTimeZone(PropertiesFactory.getTimeZone());
    }
    
    
    /** Check the list of resources for the generic types. If no resources
     * have been defined the action for a given type will be disabled
     */
    private void checkResourceActions(){
        //iterate through the generic types
        VLabVertexType type = new ExperimentType();
        ArrayList list = ResourceFactory.getResourceList(type);
        int size = list != null ? list.size() : 0;
        if(size <= 0){
            Action action = ActionFactory.getAction(ActionFactory.ACTION_ADD_VERTEX_EXPERIMENT);
            action.setEnabled(false);
        }
        
        type = new ComputeType();
        list = ResourceFactory.getResourceList(type);
        size = list != null ? list.size() : 0;
        if(size <= 0){
            Action action = ActionFactory.getAction(ActionFactory.ACTION_ADD_VERTEX_COMPUTE);
            action.setEnabled(false);
        }
    }
    
    
    
    
    
    //--------------------------------------------------------------------------
    //----- MainFrame main method
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        
        float f[] = {1, 3,4};
        
        
        
        //create settings dir if it not exist
        //SplashDialog splashDialog = null;
        SplashDialog splashDialog = null;
        try {
            
            //initialize config file
            ConfigFactory.initConfig(CFG_FILE, new SSAConfigurationImpl(CFG_FILE));
            
            //initialize ssl
            SslFactory.initSsl(MainFrame.class.getClassLoader(), ConfigFactory.getConfigFactory());
            
            //PlasticLookAndFeel.setTabStyle(PlasticLookAndFeel.TAB_STYLE_METAL_VALUE);
            UIManager.setLookAndFeel("com.jgoodies.looks.plastic.Plastic3DLookAndFeel");
            UIManager.put(Options.DEFAULT_ICON_SIZE_KEY, new Dimension(18, 18));            
            
            SystemConfigurationValue systemConfVO = SystemConfigurationFactory.getInstance();
            //set locale
            systemConfVO.setCurrentLocale(Locale.ENGLISH);
            SystemConfigurationFactory.setInstance(systemConfVO);
            SystemConfigurationFactory.serializeToFile();
            
            //start splash dialog
            splashDialog = new SplashDialog(null, true);
            SplashDialogWorker splashWorker = new SplashDialogWorker(splashDialog);
            MainFrame mf = getInstance();
            mf.initAppData(splashDialog);
        } catch (Exception exc) {
            exc.printStackTrace();
            Message.showInfoMessage(null, exc.getLocalizedMessage(), Message.MESSAGE_TYPE_ERROR);
            System.exit(-1);
        }
    }
    
    
}
