/*
 * DefaultTabContentPane.java
 *
 * Created on January 20, 2006, 2:24 PM
 */

package pl.psnc.vlab.ssa.gui.layout;
import javax.swing.JDialog;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.exception.VlabException;
import pl.psnc.vlab.rds.ElementDocument;
import pl.psnc.vlab.rds.ElementGroupDocument;
import pl.psnc.vlab.rds.TabDocument;
import pl.psnc.vlab.ssa.gui.resource.ComponentBuilder;
import pl.psnc.vlab.ssa.gui.resource.components.ElementComponent;
import pl.psnc.vlab.ssa.gui.resource.pane.ResourceElementPanel;
import pl.psnc.vlab.ssa.gui.resource.pane.ResourceGroupElementPanel;

/**
 *
 * @author  @author <a href="mailto:osa@man.poznan.pl">Dominik Stoklosa (~osa~)</a>
 */
public class DefaultTabContentPane extends javax.swing.JPanel implements PaneDecorator{
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Stores an instance of parent dialog */
    JDialog parent;
    
    /** Stores an instance of tab */
    TabDocument.Tab tab;
    
    /** Stores the current tab index */
    int tabIndex;
    
    /**
     * Creates new form DefaultTabContentPane
     * @param tabIndex 
     * @param parent the parent dialog (Resource Properties Dialog)
     * @param tab current tab to be rendered
     */
    public DefaultTabContentPane(JDialog parent, TabDocument.Tab tab, int tabIndex) {
        initComponents();
        //jContentScrollPane.setBorder(Borders.createEmptyBorder("0dlu, 0dlu, 0dlu, 0dlu"));
        this.parent = parent;
        this.tab = tab;
        this.tabIndex = tabIndex;
        setName(tab.getName());
    }
    
    
    /**
     * Creates new form DefaultTabContentPane
     * @param parent the parent dialog (Resource Properties Dialog)
     * @param tab current tab to be rendered
     */
    public DefaultTabContentPane(JDialog parent, TabDocument.Tab tab) {
        this(parent, tab, 0);
    }
    
    
    /**
     * Creates new form DefaultTabContentPane with no properties dialog and tab index 
     * defined
     * @param tab current tab to be rendered
     */
    public DefaultTabContentPane(TabDocument.Tab tab) {
        this(null, tab, 0);
    }
    
    
    /**
     * Add the resource element to the given container
     * @param component
     * @param constraints
     */
    public void addElement(ElementDocument.Element element[]) throws VlabException{
        log.debug("<addElement>");
        int elemSize = element != null ? element.length : 0;
        log.debug("Elements count:"+elemSize);
        for(int i=0; i<elemSize; i++){
            //ElementComponent component = ComponentBuilder.createComponent(element[i]);
            if(element[i].getComponent().isSetVisible() && !element[i].getComponent().getVisible()){
                continue;
            }
            ResourceElementPanel elementPanel = new ResourceElementPanel(this.parent, element[i], tabIndex);           
            add(elementPanel);
        }
        log.debug("</addElement>");
    }
    
    /**
     * Add the resource group elements to the given container
     * @param elementGroupList list of group element
     */
    public void addGroupElement(ElementGroupDocument.ElementGroup elementGroupList[]) throws VlabException{
        log.debug("<addGroupElement>");
        int groupSize = elementGroupList != null ? elementGroupList.length : 0;
        log.debug("Groups count:"+groupSize);
        for(int i=0; i<groupSize; i++){
            if(elementGroupList[i].isSetVisible() && !elementGroupList[i].getVisible()){
                continue;
            }
            ResourceGroupElementPanel groupPanel = new ResourceGroupElementPanel(this.parent, elementGroupList[i], tabIndex);
            add(groupPanel);
        }
        log.debug("</addGroupElement>");
        
    }
    
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {

        setLayout(new javax.swing.BoxLayout(this, javax.swing.BoxLayout.Y_AXIS));

    }
    // </editor-fold>//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    
    
}
