/*
 * JFormTabContentPane.java
 *
 * Created on January 20, 2006, 3:17 PM
 *
 */

package pl.psnc.vlab.ssa.gui.layout;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.exception.VlabException;
import pl.psnc.vlab.rds.ConstraintsDocument;
import pl.psnc.vlab.rds.ElementDocument;
import pl.psnc.vlab.rds.ElementGroupDocument;
import pl.psnc.vlab.rds.LayoutDocument;
import pl.psnc.vlab.rds.TabDocument;
import pl.psnc.vlab.ssa.gui.resource.ComponentBuilder;
import pl.psnc.vlab.ssa.gui.resource.components.ElementComponent;
import pl.psnc.vlab.ssa.gui.resource.pane.JResourcePropertyDialog;
import pl.psnc.vlab.ssa.gui.resource.pane.ResourceElementSubject;

/**
 *
 * @author @author <a href="mailto:osa@man.poznan.pl">Dominik Stoklosa (~osa~)</a>
 * @email osa@man.poznan.pl
 */
public class JFormTabContentPane extends DefaultTabContentPane{
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Location to row templates */
    static final  String ROW_TEMPLATES = "rowTemplates";
    
    /** Location to column templates */
    static final  String COL_TEMPLATES = "columnTemplates";
    
    /** Stores instance of Layput Cell Constraints */
    CellConstraints cc;
    
    //fake
    JComboBox timeSpecType;
    
    /** Creates a new instance of JFormTabContentPane */
    public JFormTabContentPane(JDialog parent, TabDocument.Tab tab, LayoutDocument.Layout layout,int tabIndex) throws VlabException{
        super(parent, tab, tabIndex);
        try{
            cc = new CellConstraints();
            String col = layout.getColumnSpecs();
            String row = layout.getRowSpecs();
            LayoutDecorator layoutDecorator = new LayoutTemplate(COL_TEMPLATES, ROW_TEMPLATES);
            col = layoutDecorator.applyColumnTemplate(col);
            row = layoutDecorator.applyRowTemplate(row);
            log.debug("Col spec-"+col);
            log.debug("Row spec-"+row);
            setLayout(new FormLayout(col, row));
        }catch(Exception e){
            log.error("Error occured during pane creation. Details:"+e.getMessage());
            throw new VlabException("Error occured during panle creation. Details:"+e.getMessage(), e);
        }
    }
    
    
    /**
     * Add the resource element to the given container
     * @param component
     * @param constraints
     */
    public void addElement(ElementDocument.Element element[]) throws VlabException{
        log.debug("<addElement>");
        int elemSize = element != null ? element.length : 0;
        log.debug("Elements count:"+elemSize);
        for(int i=0; i<elemSize; i++){
            
            ElementComponent component = ComponentBuilder.createComponent(element[i]);
            String name = component.getResourceElement().getName();
            ((JResourcePropertyDialog )parent).putComponent(name, component);
            component.setTabIndex(tabIndex);
            if(name.startsWith("expLength")){
                this.duration.add(component);
                log.debug("expLength");
                log.debug("Dur Size-"+duration.size());
            } else if(name.startsWith("reservTimeFrom") ||
                    name.startsWith("reservTimeTo") ){
                this.slot.add(component);
                log.debug("reservTimeFrom");
                log.debug("Slot Size-"+slot.size());
            }
            
            /*if(element[i].getComponent().isSetVisible() && !element[i].getComponent().getVisible()){
                continue;
            }*/
            
            //register component
            ( (ResourceElementSubject) parent).register(component);
            
            ConstraintsDocument.Constraints constraints = component.getLayoutConstraints();
            add(component.getConcreteJavaComponent(),
                    cc.xywh(constraints.getCol(),
                    constraints.getRow(),
                    constraints.getColSpan(),
                    constraints.getRowSpan()));
            //---- Fake section TBD FIXME
            
            if(name.equals("specificationTypeValue")){
                //add action event
                timeSpecType = (JComboBox)component.getConcreteJavaComponent();
                timeSpecType.addItemListener(new ItemListener() {
                    public void itemStateChanged(ItemEvent e) {
                        timeTypeSpectemStateChanged(e);
                    }
                });
            }
            //---- Fake section [END]
            
        }
        log.debug("</addElement>");
    }
    
    List duration = new ArrayList();
    List slot = new ArrayList();
    
    // TBD FAKE METHOD - to be removed
    private void timeTypeSpectemStateChanged(ItemEvent e) {
        log.debug("timeTypeSpectemStateChanged");
        if(duration.size() == 0 || slot.size() == 0){
            log.debug("null data");
            return;
        }
        
        int index = timeSpecType.getSelectedIndex();
        switch(index) {
            case 0  :   //as soon as possible                
                setListVisible(duration, true);
                setListVisible(slot, false);
                break;
            case 1  :   //duration
                setListVisible(duration, false);
                setListVisible(slot, true);
                break;
        }
    }
    
    // TBD Fake method - to be removed
    private void setListVisible(List list , boolean visible){
        Iterator it = list.iterator();
        while(it.hasNext()){
            ElementComponent comp = (ElementComponent) it.next();
            comp.setVisible(visible);            
        }
    }
    
    
    /**
     * Add the resource group elements to the given container
     * @param elementGroupList list of group element
     */
    public void addGroupElement(ElementGroupDocument.ElementGroup elementGroupList[]) {
        log.debug("<addGroupElement>");
        int groupSize = elementGroupList != null ? elementGroupList.length : 0;
        log.debug("Groups count:"+groupSize);
        for(int i=0; i<groupSize; i++){
            if(elementGroupList[i].isSetVisible() && !elementGroupList[i].getVisible()){
                continue;
            }
            
        }
        log.debug("</addGroupElement>");
        
    }
    
    
    
    
    
}
