/*
 * LayoutTemplate.java
 *
 * Created on January 30, 2006, 12:53 PM
 *
 */

package pl.psnc.vlab.ssa.gui.layout;

import java.util.Enumeration;
import java.util.Locale;
import java.util.ResourceBundle;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.exception.VlabException;


/**
 * The JForm Layout Template implementation. It applies templates to the row and column
 * specifications
 * @author @author <a href="mailto:osa@man.poznan.pl">Dominik Stoklosa (~osa~)</a>
 * @email osa@man.poznan.pl
 */
public class LayoutTemplate implements LayoutDecorator {
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());    
    
    /** Holds the column specification */
    private ResourceBundle columnTemplates;
    
    /** Holds the row specification */
    private ResourceBundle rowTemplates;
    
    /** Specification separator */
    private static final String SPEC_SEP = ",";
    
    /** Creates a new instance of LayoutTemplate */
    public LayoutTemplate(String columnTemplates, String rowTemplates) throws VlabException{
        try{
            this.columnTemplates = ResourceBundle.getBundle(columnTemplates, Locale.getDefault());
            this.rowTemplates    = ResourceBundle.getBundle(rowTemplates, Locale.getDefault());
            Enumeration enuM = this.rowTemplates.getKeys();
            while(enuM.hasMoreElements()){
                log.debug("elem="+enuM.nextElement());
            }
        } catch(Exception err){
            throw new VlabException("Error while loading layout templates.Details:"+err.getMessage(), err);
        }
        
    }
    
    /**
     * Applies the row template on the given column
     * sepcification
     * @param rowSpecification row specification
     * @return new row specification, after the transformation
     */
    public String applyRowTemplate(String rowSpecification) {
        return applyTemplate(rowTemplates, rowSpecification);
    }
    
    
    /**
     * Applies the column template on the given column
     * sepcification
     * @param columnSpecification column specification
     * @return new column specification, after the transformation
     */
    public String applyColumnTemplate(String columnSpecification) {
        return applyTemplate(columnTemplates, columnSpecification);
    }
    
    
    /**
     * Applies tempalte on the given specification
     * @param bundle the list of templates
     * @param spec specification
     * @return specification after transformation
     */
    private String applyTemplate(ResourceBundle bundle, String spec){
        String newSpecArray[] = spec.split(SPEC_SEP);
        StringBuffer res = new StringBuffer();
        int size = newSpecArray != null ? newSpecArray.length : 0;
        for(int i=0; i<size; i++){
            try{
                String part = bundle.getString(newSpecArray[i].trim());
                res.append(part);
            }catch(Exception e){
                log.debug("Couldn't find value [key="+newSpecArray[i]+"]");
                res.append(newSpecArray[i]);
            }
            if(i <size-1){
                res.append(SPEC_SEP);
            }                        
        }
        return res.toString();
    }   
}
