/*
 * TextFieldElement.java
 *
 * Created on 17 czerwiec 2004, 10:53
 */

package pl.psnc.vlab.ssa.gui.resource.components;
import java.util.ArrayList;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.ssa.gui.resource.model.ResourceElementObserver;
//schema
import pl.psnc.vlab.rds.ElementDocument;
import pl.psnc.vlab.cds.ItemDocument;


/**
 * This class represents the JCheckBox component. Having the element
 * details specyfied the JCheckBox component will be configured properly
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class CheckBoxElement extends ElementAbstractButtonComponent implements ResourceElementObserver{
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /**
     * Creates a new instance of TextFieldElement, with the
     * element details specyfied by the resource element object
     * @param resourceElement
     */
    public CheckBoxElement(ElementDocument.Element resourceElement) {
        super(resourceElement);
        elementComponent = new JCheckBox();
        initElement();
    }
    
    /**
     * Sets the element properties
     */
    protected void initElement(){
        //call parent to initialize its components
        super.initElement();                
    }
    
    
    /**
     * Notifies the observer that he has to update his data
     */
    public void update() {
                
        String value = Boolean.toString(doCast().isSelected());
        log.debug("<update:"+value+">");
        
        ElementDocument.Element.Out out = resourceElement.getOut();
        int itemSize = out.sizeOfItemArray();
        if(itemSize > 1){
            for(int i=itemSize-1; i>0; i--){
                out.removeItem(i);
            }
        }
        ItemDocument.Item outItem = null;
        if(itemSize == 0){
            outItem = out.addNewItem();
        } else {
            outItem = out.getItemArray(0);
        }
        outItem.setId(0);
        outItem.setStringValue(value);
        log.debug("</update>");
    }
    
    
    public void addChangeListener(ArrayList grp1, ArrayList grp2) {
        log.debug("<addChangeListener:checBox>");
        final ArrayList list1 = grp1;
        final ArrayList list2 = grp2;
        doCast().addChangeListener(
        new ChangeListener() {
            public void stateChanged(ChangeEvent e){
                log.debug("stateChanged");
                JCheckBox source = (JCheckBox) e.getSource();
                boolean enabled = source.isSelected();
                update(list1, enabled);
                update(list2, !enabled);
            }
        });
        log.debug("</addChangeListener:checBox>");
    }
    
    private void update(ArrayList list, boolean enabled){
        log.debug("Update: "+enabled+">");
        int size = list != null ? list.size() : 0;
        for(int i=0; i<size; i++){
            JComponent component = (JComponent) list.get(i);
            component.setEnabled(enabled);
        }
        log.debug("/Update>");
    }
    
}
