/*
 * ElementFactory.java
 *
 * Created on 17 czerwiec 2004, 11:05
 */

package pl.psnc.vlab.ssa.gui.resource.components;
import javax.swing.AbstractButton;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.util.image.ImageFactory;
//schema
import pl.psnc.vlab.rds.ElementDocument;
import pl.psnc.vlab.cds.ItemDocument;



/**
 * The abstract super class for building the
 * java components from the resource elements.
 * It contains a set of common util methods for the text components the i.e. for setting
 * the java component properties (name, tooltip, etc.)
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public abstract class ElementAbstractButtonComponent extends ElementComponent {
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    
    /**
     * Creates a new instance of the Element Component
     * @param resourceElement details specyfied by the resource element object
     */
    public ElementAbstractButtonComponent(ElementDocument.Element resourceElement){
        super(resourceElement);
    }
    
    /**
     * The method is used to initialize the java component
     * and set the component properties, which are taken from
     * the resource element description
     */
    protected void initElement(){
        //call parent to initialize its components
        super.initElement();
        //set selected
        setSelected(isSelected());
        //set icon
        ImageIcon icon = getIcon();
        ImageIcon selectedIcon = getSelectedIcon();
        if(icon != null && selectedIcon != null && !icon.equals(selectedIcon) ){
            setIcon(icon);
            setSelectedIcon(selectedIcon);
        }
    }
    
    
    
    /**
     * Determines wheter the componet should be set as selected
     * initialy.
     * @return true if the component is selected, false otherwise
     */
    public boolean isSelected(){
        //check the model first
        ItemDocument.Item items[] = getOutItems();
        int size = items != null ? items.length : 0;
        if(size == 1){
            return Boolean.valueOf(items[0].getStringValue()).booleanValue();
        }
        if(resourceElement.getComponent().isSetSelected()){
            return resourceElement.getComponent().getSelected();
        }
        if(resourceElementType.isSetSelected()){
            return resourceElementType.getSelected();
        }
        return true;       
    }
    
    
    /**
     * Sets the font for this component.
     * @param font the desired Font for this component
     */
    public void setSelected(boolean selected){
        //set resource as selected
        resourceElement.getComponent().setSelected(selected);
        //set component as selected
        doCast().setSelected(selected);
    }
    
    
    /**
     * Get the element icon
     * @return the component icon
     */
    public ImageIcon getIcon(){
        String iconName = null;
        if(resourceElement.getComponent().isSetIcon()){
            iconName = resourceElement.getComponent().getIcon();
        } else if(resourceElementType.isSetIcon()){
            iconName = resourceElementType.getIcon();
        } else {
            return null;
        }
        ImageIcon icon = null;
        try{
            icon = ImageFactory.getImageBean(iconName).getImage();
            return icon;
        }catch(Exception e){
            log.debug("Icon not found");
            return null;
        }
    }
    
    
    /**
     * Sets the button's default icon. This icon is also used as the "pressed" and "disabled" icon if there is no explicitly set pressed icon.
     * @param icon the icon used as the default image
     */
    public void setIcon(Icon defaultIcon){
        if(defaultIcon == null){
            return;
        }
        //set the button default icon
        doCast().setIcon(defaultIcon);
    }
    
    /**
     * Get the element selected icon
     * @return
     */
    public ImageIcon getSelectedIcon(){
        String iconName = null;
        if(resourceElement.getComponent().isSetSelectedIcon()){
            iconName = resourceElement.getComponent().getSelectedIcon();
        } else if(resourceElementType.isSetSelectedIcon()){
            iconName = resourceElementType.getSelectedIcon();
        } else {
            return null;
        }
        ImageIcon icon = null;
        try{
            icon = ImageFactory.getImageBean(iconName).getImage();
            return icon;
        }catch(Exception e){
            log.debug("Icon not found");
            return null;
        }
    }
    
    
    /**
     * Returns the selected icon for the button.
     * @param selectedIcon the selectedIcon property
     */
    public void setSelectedIcon(Icon selectedIcon){
        if(selectedIcon == null){
            return;
        }
        //set the button selected icon
        doCast().setSelectedIcon(selectedIcon);
    }
    
    
    
    
    /**
     * Cast the element component to the AbstractButton interface
     * @return element component as an AbstractButton
     */
    protected AbstractButton doCast(){
        return (AbstractButton) elementComponent;
    }
    
}