/*
 * TextFieldElement.java
 *
 * Created on 17 czerwiec 2004, 10:53
 */

package pl.psnc.vlab.ssa.gui.resource.components;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.rds.ElementDocument;
import pl.psnc.vlab.util.image.ImageFactory;

/**
 * This class represents the JLabel component. Having the element
 * details specyfied the JLabel component will be configured properly
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class LabelElement extends ElementComponent{
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Instance of label icon */
    private ImageIcon icon;
    
    /**
     * Creates a new instance of LabelElement, with the
     * element details specyfied by the resource element object
     * @param resourceElement
     */
    public LabelElement(ElementDocument.Element resourceElement) {
        super(resourceElement);
        elementComponent = new JLabel();
        initElement();
    }
    
    /**
     * Sets the element properties
     */
    protected void initElement(){
        //call parent to initialize its components
        super.initElement();
        ((JLabel)elementComponent).setText(getDisplayName());
        icon = getIcon();
        if(icon != null){
            ( (JLabel) elementComponent).setIcon(icon);
        }
    }
    
    
    /**
     * Notifies the observers that they should update their data
     */
    public void update() { }
    
    
    /**
     * Get the element icon
     * @return the component icon
     */
    public ImageIcon getIcon(){
        log.debug("<getIcon>");
        String iconName = null;
        if(resourceElement.getComponent().isSetIcon()){
            iconName = resourceElement.getComponent().getIcon();
        } else if(resourceElementType.isSetIcon()){
            iconName = resourceElementType.getIcon();
        } else {
            log.debug("</getIcon:null>");
            return null;
        }
        ImageIcon icon = null;
        try{
            icon = ImageFactory.getImageBean(iconName).getImage();
            log.debug("</getIcon:"+iconName+">");
            return icon;
        }catch(Exception e){
            log.error("Icon not found");
            log.debug("</getIcon:null>");
            return null;
        }
    }
    
    
}
