/*
 * TextFieldElement.java
 *
 * Created on 17 czerwiec 2004, 10:53
 */

package pl.psnc.vlab.ssa.gui.resource.components;

import pl.psnc.vlab.ssa.gui.resource.renderer.ListBoxDefaultRenderer;
import pl.psnc.vlab.ssa.gui.resource.ComponentBuilder;
import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.ListModel;
import javax.swing.event.ListSelectionListener;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.rds.ElementDocument;
import pl.psnc.vlab.rds.ElementDocument.Element;
import pl.psnc.vlab.cds.ItemDocument;
import pl.psnc.vlab.ssa.gui.resource.model.ModelDataListModel;
import pl.psnc.vlab.ssa.gui.resource.model.ModelDataListSelectionModel;


/**
 * This class represents the JList component. Having the element
 * details specyfied the JList component will be configured properly
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class ListBoxElement extends ElementComponent implements ListSelectionListener{
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /**
     * Creates a new instance of TextFieldElement, with the
     * element details specyfied by the resource element object
     * @param resourceElement
     */
    public ListBoxElement(Element resourceElement) {
        super(resourceElement);
        elementComponent = new JList();
        doCast().addListSelectionListener(this);
        doCast().setCellRenderer(new ListBoxDefaultRenderer(doCast(), 1, 200));
        initElement();
    }
    
    /**
     * Sets the element properties
     */
    protected void initElement(){
        //call parent to initialize its components
        super.initElement();
        //set model
        installModel();
        //tbd set selection model
    }
    
    
    /**
     * Get the concrete component
     */
    public JComponent getConcreteJavaComponent() {
        return new JScrollPane(this.elementComponent);
    }
    
    
    /**
     *  Set the model for this component
     */
    private  void installModel(){
        log.debug("<installModel>");        
        ModelDataListModel dataListModel = new ModelDataListModel(doCast() , getModelItems() );       
        doCast().setModel(dataListModel);
        ModelDataListSelectionModel selectionModel = new ModelDataListSelectionModel(getModelItems(), getOutItems());
        doCast().setSelectionModel(selectionModel);        
        selectionModel.setSelectedItems();        
        log.debug("</installModel>");
    }
                
    
    public void valueChanged(javax.swing.event.ListSelectionEvent e) {
        //empty body tbd 
    }
    
    
    /**
     * Notifies the observers that they should update their data
     * @param resource the resource description
     */
    public void update() {
        log.debug("<update>");
        //get out data
        ItemDocument.Item selectedItems[] = ((ModelDataListModel) doCast().getModel()).getSelectedItems();
        ElementDocument.Element.Out out = resourceElement.getOut();
        if(selectedItems == null){
            log.debug("Clear selection");
            int size = out.sizeOfItemArray();
            for(int i=0; i<size; i++){
                out.sizeOfItemArray();
                out.removeItem(i);
            }
        } else {
            out.setItemArray(selectedItems);
        }
        log.debug("</update>");
    }
    
    /**
     * Cast the element component to the JList interface
     * @return element component as an JList
     */
    private JList doCast(){
        return (JList) elementComponent;
    }
    
}

