/*
 * TextFieldElement.java
 *
 * Created on 17 czerwiec 2004, 10:53
 */

package pl.psnc.vlab.ssa.gui.resource.components;

import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.rds.ElementDocument;
import pl.psnc.vlab.cds.ItemDocument;
import pl.psnc.vlab.ssa.gui.resource.model.ResourceElementObserver;

/**
 * This class represents the JSpinner component. Having the element
 * details specyfied the JSpinner component will be configured properly
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class SpinnerElement extends ElementComponent implements ResourceElementObserver{
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Model - Integer model */
    private String MODEL_INTEGER = "java.lang.Integer";
    /** Model - Double model */
    private String MODEL_DOUBLE = "java.lang.Double";
    
    /**
     * Creates a new instance of TextFieldElement, with the
     * element details specyfied by the resource element object
     * @param resourceElement
     */
    public SpinnerElement(ElementDocument.Element resourceElement) {
        super(resourceElement);
        elementComponent = new JSpinner();
        initElement();
    }
    
    /**
     * Sets the element properties
     */
    protected void initElement(){
        //call parent to initialize its components
        super.initElement();
        //set spinner model
        SpinnerNumberModel model = null;
        if(getModelClassName().equals(MODEL_INTEGER)){
            model =  new SpinnerNumberModel(toInt(getInitialValue()), toInt(getMin()), toInt(getMax()), toInt(getStep()));
        } else if(getModelClassName().equals(MODEL_DOUBLE)){
            model =  new SpinnerNumberModel(toDouble(getInitialValue()), toDouble(getMin()), toDouble(getMax()), toDouble(getStep()));
        } else {
            model = new SpinnerNumberModel();
        }
        doCast().setModel(model);
        //set editor
        JSpinner.NumberEditor editor = new JSpinner.NumberEditor(doCast());
    }
                
    
    
    /**
     * Gets the initial value of the JSpinner field
     * @return
     */
    private String getInitialValue(){
        //check the model first
        ItemDocument.Item items[] = getOutItems();
        int size = items != null ? items.length : 0;
        try{
            if(size == 1){
                return items[0].getStringValue();
            } else {
                if(this.resourceElement.getComponent().isSetValue()){
                    return this.resourceElement.getComponent().getValue();
                }
                if(this.resourceElementType.isSetValue()){
                    return this.resourceElementType.getValue();
                }
            }
        }catch(NumberFormatException e){
            log.debug("Invalid value-not number");
            if(this.resourceElement.getComponent().isSetValue()){
                return this.resourceElement.getComponent().getValue();
            }
            if(this.resourceElementType.isSetValue()){
                return this.resourceElementType.getValue();
            }
        }
        return "0";
    }
    
    
    /**
     * Gets the minimum value of the JSpinner field
     * @return
     */
    private String getMin(){
        if(this.resourceElement.getComponent().isSetMin()){
            return this.resourceElement.getComponent().getMin();
        }
        if(this.resourceElementType.isSetMin()){
            return this.resourceElementType.getMin();
        }
        return "0";
    }
    
    
    /**
     * Gets the maximum value of the JSpinner field
     * @return
     */
    private String getMax(){
        if(this.resourceElement.getComponent().isSetMax()){
            return this.resourceElement.getComponent().getMax();
        }
        if(this.resourceElementType.isSetMax()){
            return this.resourceElementType.getMax();
        }
        return "1000";
    }
    
    
    /**
     * Gets the step value of the JSpinner field
     * @return
     */
    private String getStep(){
        if(this.resourceElement.getComponent().isSetStep()){
            return this.resourceElement.getComponent().getStep();
        }
        if(this.resourceElementType.isSetStep()){
            return this.resourceElementType.getStep();
        }
        return "1";
    }
    
    
    
    /**
     * Notifies the observer that he has to update his data
     */
    public void update() {
        setEnabled(this.elementComponent.isEnabled());      //tbd
        String value = getValue();
        log.debug("<update:"+getComponentName()+" " +value+">");
        
        ElementDocument.Element.Out out = resourceElement.getOut();
        int itemSize = out.sizeOfItemArray();
        if(itemSize > 1){
            for(int i=itemSize-1; i>0; i--){
                out.removeItem(i);
            }
        }
        ItemDocument.Item outItem = null;
        if(itemSize == 0){
            outItem = out.addNewItem();
        } else {
            outItem = out.getItemArray(0);
        }
        outItem.setId(0);
        outItem.setStringValue(value.toString());
        log.debug("</update>");
    }
    
    
    private int toInt(String value){
        int res = 0;
        try{
            res = Integer.parseInt(value);
        }catch(Exception e){
            log.error("err="+e.getMessage());
        }
        return res;
    }
    
    
    private double toDouble(String value){
        double res = 0.0;
        try{
            res = Double.parseDouble(value);
        }catch(Exception e){
            log.error("err="+e.getMessage());
        }
        return res;
    }
    
    private String getValue(){
        Object o = doCast().getValue();
        String res = "";
        if(getModelClassName().equals(MODEL_INTEGER)){
            res = ((Integer) o).toString();
        } else if(getModelClassName().equals(MODEL_DOUBLE)){
            res = ((Double) o).toString();
        }
        return res;
    }
    
    
    /**
     * Cast the element component to the JTextComponent interface
     * @return element component as an JTextComponent
     */
    private JSpinner doCast(){
        return (JSpinner) elementComponent;
    }
    
    /**
     * Sets the component background color
     * @param new color
     */
    public void setBackground(java.awt.Color newColor) {
        log.debug("Setting bg");
        JTextField tf =((JSpinner.DefaultEditor) doCast().getEditor()).getTextField();
        tf.setBackground(newColor);
    }
    
}
