    /*
     * TextFieldElement.java
     *
     * Created on 17 czerwiec 2004, 10:53
     */

package pl.psnc.vlab.ssa.gui.resource.components;

import pl.psnc.vlab.management.session.SessionFactory;
import pl.psnc.vlab.ssa.exception.SSAException;
import pl.psnc.vlab.ssa.gui.resource.pane.SzdBrowserPane;
import pl.psnc.vlab.ssa.management.ProxyFactory;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.ssa.gui.MainFrame;
import pl.psnc.vlab.ssa.gui.resource.model.SzdNodeElementListModel;
import pl.psnc.vlab.values.management.SessionValue;
//schema
import pl.psnc.vlab.rds.ElementDocument;
import pl.psnc.vlab.cds.ItemDocument;


/**
 * This class represents the SzdBrowser component, which is based on JList. Having the element
 * details specyfied the JList component will be configured properly
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class SzdBrowserElement extends ElementComponent {
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /**
     * Creates a new instance of TextFieldElement, with the
     * element details specyfied by the resource element object
     * @param resourceElement
     */
    public SzdBrowserElement(ElementDocument.Element resourceElement) {
        super(resourceElement);
        try{
            //get userCertId and proxy
            SessionValue sessionValue = SessionFactory.getSessionValue();
            byte proxy[] = "badProxy".getBytes();
            try{
                proxy = ProxyFactory.getUserProxy();
            }catch (SSAException ssaE){
                log.error("Error getting proxy");
            }
            elementComponent = new SzdBrowserPane(MainFrame.getInstance(), sessionValue.getUserCertId(), proxy, getOutItemsAsInt() );
            //doCast().getSelectionList().setCellRenderer(new ListBoxDefaultRenderer(doCast().getSelectionList(), 1, 200));
            initElement();
        }catch(Exception e){
            log.error("Error during the Szd Browse initialization");
            //e.printStackTrace();
            //tbd cannot ignore this exception (leave it for now);
        }
    }
    
    /**
     * Sets the element properties
     */
    protected void initElement(){
        //call parent to initialize its components
        super.initElement();
        //set number of the preferred rows
        //this.elementComponent.getSelectionList().setVisibleRowCount(getElementRows());
        //set model
        installModel();
        //tbd set selection model
    }
    
    
    
    /**
     *  Set the model for this component
     */
    private  void installModel(){
        log.debug("<installModel>");
        
        ItemDocument.Item selectedItems[] = getOutItems();
        SzdNodeElementListModel dataListModel = new SzdNodeElementListModel(doCast().getSelectionList() , getOutItems() );
        doCast().getSelectionList().setModel(dataListModel);
        log.debug("</installModel>");
    }
    
    
    
    /**
     * Notifies the observers that they should update their data
     * @param resource the resource description
     */
    public void update() {
        log.debug("<update>");
        //get out data
        ItemDocument.Item selItems[] = ( (SzdNodeElementListModel) doCast().getSelectionList().getModel()).getModelData();
        int size = selItems != null ? selItems.length :0;
        log.debug("Item size:"+size);
        if(size == 0){
            log.debug("<update: NOTHING SELECTED>");
            return;
        }
        ElementDocument.Element.Out out = resourceElement.getOut();
        //log.debug("Pattern-"+out.getPattern());
        out.setItemArray(selItems);
        log.debug("</update>");
    }
    
    
    /**
     * Converts an array of out itmes into array on int made from imtems values
     * @return
     */
    private int[] getOutItemsAsInt(){
        ItemDocument.Item items[] = getOutItems();
        int size = items != null ? items.length :0;
        int res[] = new int[size];
        for(int i=0; i<size; i++){
            res[i] = items[i].getId();
        }
        return res;
    }
    
    /**
     * Cast the element component to the SzdBrowserPane interface
     * @return element component as an SzdBrowserPane
     */
    private SzdBrowserPane doCast(){
        return (SzdBrowserPane) elementComponent;
    }
}

