/*
 * TextFieldElement.java
 *
 * Created on 17 czerwiec 2004, 10:53
 */

package pl.psnc.vlab.ssa.gui.resource.components;
import java.awt.Dimension;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.rds.ElementDocument;
import pl.psnc.vlab.cds.ItemDocument;
import pl.psnc.vlab.ssa.gui.resource.model.ResourceElementObserver;
import pl.psnc.vlab.ssa.util.component.ComponentKeys;


/**
 * This class represents the JTextArea component. Having the element
 * details specified the JTextArea component will be configured properly
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class TextAreaElement extends ElementTextComponent implements ResourceElementObserver{
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Instance of component scroll pane */
    private JScrollPane jCompScrollPane;    
    
    /**
     * Creates a new instance of TextFieldElement, with the
     * element details specyfied by the resource element object
     * @param resourceElement
     */
    public TextAreaElement(ElementDocument.Element resourceElement) {
        super(resourceElement);
        elementComponent = new JTextArea();
        elementComponent.setPreferredSize(new Dimension(150,100));
        this.jCompScrollPane = new JScrollPane(elementComponent);
        initElement();        
    }
    
    /**
     * Sets the element properties
     */
    protected void initElement(){        
        //set columns
        setColumns(getColumns());
        //set rows
        setRows(getRows());
        
        //set text
        ItemDocument.Item items[] = getOutItems();
        int size = items != null ? items.length : 0;
        if(size == 1){                                                      //tbd what if different value i.e 2
            setText( items[0].getStringValue());
        } 
    }
    
    
    /**
     * Get the concrete component
     */
    public javax.swing.JComponent getConcreteJavaComponent() {
        return jCompScrollPane;                
    }
    
    
    /**
     * Notifies the observer that he has to update his data
     */
    public void update() {
        String value = getText();
        log.debug("<update:"+value+">");
        
        ElementDocument.Element.Out out = resourceElement.getOut();
        int itemSize = out.sizeOfItemArray();
        if(itemSize > 1){
            for(int i=itemSize-1; i>0; i--){
                out.removeItem(i);
            }
        }
        ItemDocument.Item outItem = null;
        if(itemSize == 0){
            outItem = out.addNewItem();
        } else {
            outItem = out.getItemArray(0);
        }
        outItem.setId(0);
        outItem.setStringValue(value);
        log.debug("</update>");
    }
    
    
    
        /**
     * Get the number of columns preferred for display
     * If it is not specified the default value is returned
     * @return
     */
    public int getColumns(){
        if(resourceElement.getComponent().isSetColumns()){
            return resourceElement.getComponent().getColumns();
        } else if(resourceElementType.isSetColumns()){
            return resourceElementType.getColumns();
        }
        return ComponentKeys.DEFAULT_COLUMN_COUNT;
    }
    
    
    /**
     * Sets the number of columns in this TextField, and then invalidate the layout.
     * @param columns the number of columns >= 0
     */
    public void setColumns(int columns){
        if(columns <0){
            return;
        }
        //set resource as enabled
        resourceElement.getComponent().setColumns(columns);
        //set component as enabled
        ((JTextArea) elementComponent).setColumns(columns);
    }
    
    
    /**
     * Get the number of rows preferred for display
     * If it is not specyfied the default value is returned
     * @return
     */
    public int getRows(){
        if(resourceElement.getComponent().isSetRows()){
            return resourceElement.getComponent().getRows();
        } else if(resourceElementType.isSetRows()){
            return resourceElementType.getRows();
        }
        return ComponentKeys.DEFAULT_ROW_COUNT;
    }

    
    
    /**
     * Sets the number of rows in this TextArea, and then invalidate the layout.
     * @param rows the number of rows >= 0
     */
    public void setRows(int rows){
        if(rows <0){
            return;
        }
        //set resource as enabled
        resourceElement.getComponent().setRows(rows);
        //set component as enabled
        ((JTextArea) elementComponent).setRows(rows);
    }

    
}
