/*
 * TextFieldElement.java
 *
 * Created on 17 czerwiec 2004, 10:53
 */

package pl.psnc.vlab.ssa.gui.resource.components;

import pl.psnc.vlab.ssa.util.component.ComponentKeys;
import javax.swing.JTextField;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.rds.ElementDocument;
import pl.psnc.vlab.cds.ItemDocument;
import pl.psnc.vlab.ssa.gui.resource.model.ResourceElementObserver;


/**
 * This class represents the JTextField component. Having the element
 * details specyfied the JTextField component will be configured properly
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class TextFieldElement extends ElementTextComponent implements ResourceElementObserver{
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /**
     * Creates a new instance of TextFieldElement, with the
     * element details specyfied by the resource element object
     * @param resourceElement
     */
    public TextFieldElement(ElementDocument.Element resourceElement) {        
        super(resourceElement);        
        elementComponent = new JTextField();
        initElement();
    }
    
    /**
     * Sets the element properties
     */
    protected void initElement(){
        //call parent to initialize its components
        super.initElement();    
        //set columns
        setColumns(getColumns());        
        //set text
        ItemDocument.Item items[] = getOutItems();
        int size = items != null ? items.length : 0;
        if(size == 1){                                                      //tbd what if different value i.e 2
            setText( items[0].getStringValue());
        } 
    }
    
    
    /**
     * Get the concrete component
     */
    public javax.swing.JComponent getConcreteJavaComponent() {
        return this.elementComponent;
    }
    
    
    /**
     * Notifies the observer that he has to update his data
     */
    public void update() {
        String value = getText();
        log.debug("<update:"+value+">");
        
        ElementDocument.Element.Out out = resourceElement.getOut();
        int itemSize = out.sizeOfItemArray();
        if(itemSize > 1){
            for(int i=itemSize-1; i>0; i--){
                out.removeItem(i);
            }
        }
        ItemDocument.Item outItem = null;
        if(itemSize == 0){
            outItem = out.addNewItem();
        } else {
            outItem = out.getItemArray(0);
        }
        outItem.setId(0);
        outItem.setStringValue(value);
        log.debug("</update>");
    }
    
    /**
     * Get the number of columns preferred for display
     * If it is not specified the default value is returned
     * @return
     */
    public int getColumns(){
        if(resourceElement.getComponent().isSetColumns()){
            return resourceElement.getComponent().getColumns();
        } else if(resourceElementType.isSetColumns()){
            return resourceElementType.getColumns();
        }
        return ComponentKeys.DEFAULT_COLUMN_COUNT;
    }
    
    
    /**
     * Sets the number of columns in this TextField, and then invalidate the layout.
     * @param columns the number of columns >= 0
     */
    public void setColumns(int columns){
        if(columns <0){
            return;
        }
        //set resource as enabled
        resourceElement.getComponent().setColumns(columns);
        //set component as enabled
        ((JTextField) elementComponent).setColumns(columns);
    }
    
    
    
}
