package pl.psnc.vlab.ssa.gui.resource.model;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.cds.ItemDocument;
import pl.psnc.vlab.util.FormTools;

/**
 * The Model Data ComboBox Model is used as an default
 * model for the comoboBox component. The data for the model is taken
 * from the resource element description and looks like this:
 * - <modelData>
 *      <item id="0">1D</item>
 *      <item id="1">2D</item>
 *      <item id="2">3D</item>
 *  </modelData>
 *
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class ModelDataComboBoxModel extends DefaultComboBoxModel {
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Stores the model data array */
    private pl.psnc.vlab.cds.ItemDocument.Item[] data = null;
    
    /** Stores an instance of the ComboBox model*/
    private JComboBox comboBox = null;
    
    /**
     * Creates a new instance of ModelDataComboBoxModel
     * @param comboBox the JCompboBox component
     * @param data the model data - list of items from the element modelData
     */
    public ModelDataComboBoxModel(JComboBox comboBox, pl.psnc.vlab.cds.ItemDocument.Item[] data) {
        this.data = data;
        this.comboBox = comboBox;
        int size = data!= null ? data.length : 0;
        log.debug("Model items count:"+size);
        //add model elemetns
        for(int i=0; i<size; i++){
            super.addElement(data[i].getStringValue());
        }
    }
    
    
    /**
     * Get the selected item
     * @return the selected item or null if there is no selection
     */
    public Object getSelectedItem(){
        return super.getSelectedItem();
    }
    
    
    
    /**
     * Set the selected item
     * @param anObject
     */
    public void setSelectedItem(Object anObject) {
        super.setSelectedItem(anObject);
    }
    
    
    /**
     * Get the selected item
     * @return
     */    
    public ItemDocument.Item getSelectedItemAsItem(){
        if(this.data == null){
            return null;
        }
        //get selected values
        String value = (String)this.comboBox.getSelectedItem();        
        if(value == null){
            return null;
        }                
        return getItemByValue(value);
    }
    
    
    /**
     * Get the Item by Value
     * @param value
     * @return
     */    
    private ItemDocument.Item getItemByValue(String value){
        if(FormTools.isValidateFormStringNull(value)){
            return null;
        }
        int size = this.data.length;
        for(int i=0; i<size; i++){
            if(this.data[i].getStringValue().equals(value) ){
                return this.data[i];
            }
        }//end of for
        return null;
    }
    
}