package pl.psnc.vlab.ssa.gui.resource.model;


import javax.swing.DefaultListModel;
import javax.swing.JList;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.cds.ItemDocument;
import pl.psnc.vlab.ssa.gui.resource.renderer.ListBoxDefaultRenderer;
import pl.psnc.vlab.util.FormTools;

/**
 * The Model Data List Model is used as an default
 * model for the jList component. The data for the model is taken
 * from the resource element description and looks like this:
 * - <modelData>
 *      <item id="0">1D</item>
 *      <item id="1">2D</item>
 *      <item id="2">3D</item>
 *  </modelData>
 *
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class ModelDataListModel extends DefaultListModel{
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Stores the model data array */
    private ItemDocument.Item[] data = null;
    
    /** Stores an instance of the ComboBox model*/
    private JList list = null;
    
    /**
     * Creates a new instance of ModelDataComboBoxModel
     * @param list the JCompboBox component
     * @param data the model data - list of items
     */
    public ModelDataListModel(JList list, ItemDocument.Item[] data) {
        this.data = data;
        this.list = list;
        int size = data!= null ? data.length : 0;
        log.debug("Model items count:"+size);
        //add model elemetns
        for(int i=0; i<size; i++){
            //data[i].setId(i);
            super.add(i, data[i].getStringValue());
        }
        ListBoxDefaultRenderer renderer = (ListBoxDefaultRenderer) list.getCellRenderer();
        renderer.update(size);
    }
    
    
    /**
     * Get the list model data
     * @return
     */
    public ItemDocument.Item[] getModelData(){
        return this.data;
    }
    
    
    
    /**
     * Searches for the first occurrence of <code>elem</code>.
     *
     * @param   elem   an object
     * @return  the index of the first occurrence of the argument in this
     *          list; returns <code>-1</code> if the object is not found
     * @see Vector#indexOf(Object)
     */
    public int indexOf(String elemName) {
        if(data == null){
            return -1;
        }
        int size = data.length;
        for(int i=0; i<size; i++){
            if(data[i].getStringValue().equals(elemName) ){
                return i;
            }
        }
        return -1;
    }
    
            
    
    /**
     * Get the selected items
     * @return the list of selected items
     */
    public ItemDocument.Item[] getSelectedItems(){
        if(this.list == null){
            return null;
        }
        //get selected values
        Object values[] = this.list.getSelectedValues();
        int size = values != null ? values.length : 0;
        if(size == 0){
            return null;
        }
        ItemDocument.Item itemList[] = new ItemDocument.Item[size];
        for(int i=0; i<size; i++){
            itemList[i] = getItemByValue( (String) values[i]);
        }
        return itemList;
    }
    
    
    /**
     * Get the Item by Value
     * @param value
     * @return
     */
    private ItemDocument.Item getItemByValue(String value){
        if(FormTools.isValidateFormStringNull(value)){
            return null;
        }
        int size = this.data.length;
        for(int i=0; i<size; i++){
            if(this.data[i].getStringValue().equals(value) ){
                return this.data[i];
            }
        }//end of for
        return null;
    }
}