package pl.psnc.vlab.ssa.gui.resource.model;


import javax.swing.DefaultListModel;
import javax.swing.JList;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.cds.ItemDocument;
import pl.psnc.vlab.szdbrowser.gui.selectedlist.NodeValue;


/**
 * The SzdNodeElement List Model is used as an default
 * model for the szd elements which are displayed in the jList component. The data for the model is taken
 * from the resource element description or the Szd System
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class SzdNodeElementListModel extends DefaultListModel{
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Stores the model data array */
    private ItemDocument.Item[] data = null;
    
    /** Stores an instance of the ComboBox model*/
    private JList list = null;
    
    
    /**
     * Creates new instance of the SzdNodeElementListModsel
     * @param list the JList Component
     * @param data the data associated with this component (array of szd DirNodeElement)
     */
    public SzdNodeElementListModel(JList list, ItemDocument.Item data[]) {
        this.data = data;
        this.list = list;
        int size = data!= null ? data.length : 0;
        log.debug("Model items count:"+size);
        //add model elemetns
        for(int i=0; i<size; i++){
            super.add(i, data[i].getStringValue());
        }
        // tbd -osa 05.2005 ListBoxDefaultRenderer renderer = (ListBoxDefaultRenderer) list.getCellRenderer();
        //tbd renderer.update(size);
    }
    
    /**
     * Get the list model data
     * @return
     */
    public ItemDocument.Item[] getModelData(){
        return this.data;
    }
    
    /**
     * Get the list of model data as an array of elements ids
     * @return
     */
    public int[] getModelDataAsIntArray(){
        int size = data != null ? data.length : 0;
        if(size == 0){
            return null;
        }
        int res[] = new int[size];
        for(int i=0; i<size; i++){
            res[i] = data[i].getId();
        }
        return res;
    }
    
    /**
     * Get the list model data as an item array
     * @return
     */
    public void updateModel(NodeValue list[]){
        log.debug("<updateModel>");
        int size = list != null ? list.length : 0;
        removeAllElements();
        if(size == 0){
            log.debug("</updateModel: empty>");
            return ;
        }        
        log.debug("Adding "+size+ " elements");
        ItemDocument.Item res[] = new ItemDocument.Item[size];
        ItemDocument itemDoc = ItemDocument.Factory.newInstance();
        for(int i=0; i<size; i++){
            res[i] = itemDoc.addNewItem();
            res[i].setId(list[i].getId());
            res[i].setStringValue(list[i].getPath());            
            add(i, res[i].getStringValue());
        }
        this.data = res;        
        fireIntervalAdded(this, 0, size);
        //tbd 05.2005 ListBoxDefaultRenderer renderer = (ListBoxDefaultRenderer) this.list.getCellRenderer();
        // renderer.update(size);
        log.debug("</updateModel>");
    }
    
    
    
    /**
     * Searches for the first occurrence of <code>elem</code>.
     *
     * @param   elem   an object
     * @return  the index of the first occurrence of the argument in this
     *          list; returns <code>-1</code> if the object is not found
     * @see Vector#indexOf(Object)
     */
    public int indexOf(String elemName) {
        if(data == null){
            return -1;
        }
        int size = data.length;
        for(int i=0; i<size; i++){
            if(data[i].getStringValue().equals(elemName) ){
                return i;
            }
        }
        return -1;
    }
    
    
    /**
     * Get the Item by Value
     * @param value
     * @return
     *
     * private ItemDocument.Item getItemByValue(String value){
     * if(FormTools.isValidateFormStringNull(value)){
     * return null;
     * }
     * int size = this.data.length;
     * for(int i=0; i<size; i++){
     * if(this.data[i].getStringValue().equals(value) ){
     * return this.data[i];
     * }
     * }//end of for
     * return null;
     * }*/
}