/*
 * DatePane.java
 *
 * Created on 27 lipiec 2004, 11:58
 */

package pl.psnc.vlab.ssa.gui.resource.pane;

import java.util.Date;
import java.util.Locale;
import javax.swing.JPanel;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import java.util.Calendar;
import javax.swing.JSpinner;
import javax.swing.SpinnerDateModel;
import javax.swing.SpinnerModel;
import pl.psnc.vlab.ssa.gui.MainFrame;
import pl.psnc.vlab.ssa.util.ImageKeys;
import pl.psnc.vlab.util.DateTools;
import pl.psnc.vlab.util.FormTools;
import pl.psnc.vlab.util.image.ImageFactory;


/**
 *
 * @author  Dominik Stoklosa
 */
public class DatePane extends JPanel {
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** An instance of the current locale */
    private Locale locale = null;
    
    /** Date format */
    private String dateFormat = null;
    
    /** Initial Date */
    private String initialDate = null;
    
    /** Time format */
    private String timeFormat = null;
    
    /** Initial Time */
    private String initialTime = null;
    
    /** An instance of the date tools */
    private DateTools dateTools = null;
    
    /** Creates new form DatePane */
    public DatePane(Locale locale, String dateFormat, String initialDate, String timeFormat, String initialTime) {
        initComponents();
        //set icon
        this.jShowCalendar.setIcon(ImageFactory.getImageBean(ImageKeys.ICON_CALENDAR).getImage());
        this.locale = locale;
        this.dateFormat = dateFormat;
        this.initialDate = initialDate;
        this.timeFormat = timeFormat;
        this.initialTime = initialTime;
        this.dateTools = new DateTools(locale, dateFormat, timeFormat);
        initDate();
        initTime();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jDateSpnr = new javax.swing.JSpinner();
        jShowCalendar = new javax.swing.JLabel();
        jTimeSpnr = new javax.swing.JSpinner();

        jDateSpnr.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jDateSpnrMouseClicked(evt);
            }
        });

        add(jDateSpnr);

        jShowCalendar.setToolTipText(java.util.ResourceBundle.getBundle("bundle/messages").getString("message.show.calendar.tip"));
        jShowCalendar.setFocusable(false);
        jShowCalendar.setIconTextGap(8);
        jShowCalendar.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jShowCalendarMouseClicked(evt);
            }
        });

        add(jShowCalendar);

        add(jTimeSpnr);

    }
    // </editor-fold>//GEN-END:initComponents
    
    private void jShowCalendarMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jShowCalendarMouseClicked
        //showCalendarDialog();
    }//GEN-LAST:event_jShowCalendarMouseClicked
    
    private void jDateSpnrMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jDateSpnrMouseClicked
        if(evt.getClickCount() == 2){
           // showCalendarDialog();
        }
    }//GEN-LAST:event_jDateSpnrMouseClicked
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JSpinner jDateSpnr;
    private javax.swing.JLabel jShowCalendar;
    private javax.swing.JSpinner jTimeSpnr;
    // End of variables declaration//GEN-END:variables
    
    /** Initialize the date component */
    private void initDate(){
        if(!FormTools.isValidateFormStringNull(initialDate) && dateTools.isDateValid(initialDate) ){
            log.debug("Valid date:"+initialDate);
            SpinnerModel dateModel = new SpinnerDateModel(dateTools.getDateFromString(initialDate),
                    null, null, Calendar.DAY_OF_MONTH);
            jDateSpnr.setModel(dateModel);
            jDateSpnr.setEditor(new JSpinner.DateEditor(jDateSpnr, dateFormat));
            this.jDateSpnr.setVisible(true);
            this.jShowCalendar.setVisible(true);
        } else {
            log.debug("invalid date");
            this.jDateSpnr.setVisible(false);
            this.jShowCalendar.setVisible(false);
        }
    }
    
    /** Initialize the time component */
    private void initTime(){
        if(!FormTools.isValidateFormStringNull(initialTime) && dateTools.isTimeValid(initialTime) ){
            log.debug("Valid time:"+initialTime);
            SpinnerModel timeModel = new SpinnerDateModel(dateTools.getTimeFromString(initialTime),
                    null, null, Calendar.HOUR);
            jTimeSpnr.setModel(timeModel);
            jTimeSpnr.setEditor(new JSpinner.DateEditor(jTimeSpnr, timeFormat));
            this.jTimeSpnr.setVisible(true);
        } else {
            log.debug("invalid time");
            this.jTimeSpnr.setVisible(false);
        }
    }
    
    
    /**
     * Get the selected date
     * @return
     */
    public String getSelectedDate(){
        if(FormTools.isValidateFormStringNull(initialDate)){
            return null;
        }
        Date date = ( (SpinnerDateModel) this.jDateSpnr.getModel()).getDate();
        return dateTools.getDateAsString(date);
    }
    
    
    /**
     * Get the selected time
     * @return
     */
    public String getSelectedTime(){
        if(FormTools.isValidateFormStringNull(initialTime)){
            return null;
        }
        Date date = ( (SpinnerDateModel) this.jTimeSpnr.getModel()).getDate();
        return dateTools.getTimeAsString(date);
    }
    
    
    /** Show calendar dialog 
    private void showCalendarDialog(){
        String curDate = getSelectedDate();
        CalendarDialog calendarDialog  = null;
        if(curDate == null){
            Date date = MainFrame.getInstance().getStatusBar().getCurrentDate();
            calendarDialog = new CalendarDialog( true, locale, date,  dateFormat );
        } else {
            calendarDialog = new CalendarDialog( true, locale, dateTools.getDateFromString(curDate),  dateFormat );
        }
        calendarDialog.show();
        Date date = dateTools.getDateFromString(calendarDialog.getSelectedDateAsString());
        ((SpinnerDateModel) this.jDateSpnr.getModel()).setValue(date);
    }*/
    
    /**
     * Registers the text to display in a tool tip.
     * The text displays when the cursor lingers over the component.
     * <p>
     * See <a href="http://java.sun.com/docs/books/tutorial/uiswing/components/tooltip.html">How to Use Tool Tips</a>
     * in <em>The Java Tutorial</em>
     * for further documentation.
     *
     * @param text  the string to display; if the text is <code>null</code>,
     *              the tool tip is turned off for this component
     * @see #TOOL_TIP_TEXT_KEY
     * @beaninfo
     *   preferred: true
     * description: The text to display in a tool tip.
     */
    public void setToolTipText(String text) {
        this.jDateSpnr.setToolTipText(text);
        this.jTimeSpnr.setToolTipText(text);
    }
    
    
    /**
     * Sets whether or not this component is enabled. A component that is enabled may respond to user input, while a component that is not enabled cannot respond to user input. Some components may alter their visual representation when they are disabled in order to provide feedback to the user that they cannot take input.
     *
     * Note: Disabling a component does not disable it's children.
     *
     * Note: Disabling a lightweight component does not prevent it from receiving MouseEvents.
     * @param enabled true if this component should be enabled, false otherwise
     */
    public void setEnabled(boolean enabled) {
        this.jDateSpnr.setEnabled(enabled);
        this.jTimeSpnr.setEnabled(enabled);
        this.jShowCalendar.setEnabled(enabled);
    }
    
    public boolean isEnabled() {
        return jDateSpnr.isEnabled();
    }
    
    
}
