/*
 * ResourceElementPanel.java
 *
 * Created on 16 czerwiec 2004, 14:26
 */

package pl.psnc.vlab.ssa.gui.resource.pane;

import pl.psnc.vlab.ssa.util.component.ComponentKeys;
import java.awt.FlowLayout;
import java.util.ArrayList;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.rds.ElementDocument;
import pl.psnc.vlab.rds.ElementGroupDocument;
import pl.psnc.vlab.ssa.gui.resource.ComponentBuilder;
import pl.psnc.vlab.ssa.gui.resource.components.ElementComponent;

/**
 * The  ResourceGroupElementPanel stores all the components within one
 * elements group.
 * @author  Dominik Stoklosa
 */
public class ResourceGroupElementPanel extends JPanel {
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Stores the value of the resource element */
    private  ElementGroupDocument.ElementGroup group = null;
    
    /** Stores the value of the Resource property Dialog */
    private JResourcePropertyDialog propertyDialog = null;
    
    /** Stores the value of the tab index where the given elements group will be
     * placed. */
    private int tabIndex = 0;
    
    /** Creates new form ResourceElementPanel */
    public ResourceGroupElementPanel(JDialog propertyDialog, ElementGroupDocument.ElementGroup group ,int tabIndex) {
        initComponents();
        this.propertyDialog = (JResourcePropertyDialog) propertyDialog;
        this.group = group;
        this.tabIndex = tabIndex;
        fillGroup();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jLeftPanel = new javax.swing.JPanel();
        jElementName = new javax.swing.JLabel();
        jRightPanel = new javax.swing.JPanel();

        setLayout(new javax.swing.BoxLayout(this, javax.swing.BoxLayout.X_AXIS));

        jLeftPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 10, 10));

        jElementName.setFont(new java.awt.Font("Tahoma", 0, 11));
        jLeftPanel.add(jElementName);

        add(jLeftPanel);

        jRightPanel.setLayout(new javax.swing.BoxLayout(jRightPanel, javax.swing.BoxLayout.Y_AXIS));

        add(jRightPanel);

    }
    // </editor-fold>//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jElementName;
    private javax.swing.JPanel jLeftPanel;
    private javax.swing.JPanel jRightPanel;
    // End of variables declaration//GEN-END:variables
    
    private void fillGroup(){
        log.debug("<fillGroup>");
        //fill the left panel first
        //set element name
        jElementName.setName(group.getName());
        jElementName.setFont(ComponentKeys.RESOURCE_KEY_FONT);
        //set element display name
        jElementName.setText(group.getDisplayName());
        jElementName.setToolTipText(group.getDisplayName());
        
        //fill the right panel
        ElementDocument.Element[] elementList = group.getElementArray();
        int size = elementList.length;
        for(int i=0; i<size; i++){
            if(elementList[i].getComponent().isSetVisible() && !elementList[i].getComponent().getVisible()){  //check the visibility
                continue;
            }
            JPanel pane = new JPanel();
            pane.setLayout(new FlowLayout(FlowLayout.RIGHT));
            JLabel elementText = new JLabel(elementList[i].getDisplayName());
            elementText.setFont(ComponentKeys.RESOURCE_VALUE_FONT);
            pane.add(elementText);
            ElementComponent elemComponent = ComponentBuilder.createComponent(elementList[i]);
            elementText.setEnabled(elemComponent.isEnabled());
            elemComponent.setTabIndex(tabIndex);
            //register component
            propertyDialog.register(elemComponent);
            this.propertyDialog.putComponent(elementList[i].getName(), elemComponent);
            pane.add(elemComponent.getConcreteJavaComponent());
            pane.setToolTipText(elementList[i].getComponent().getToolTipText());
            jRightPanel.add(pane);
            collectListeners(elementText, elemComponent);
        }
        if(source != null){
            log.debug("Adding change listener"+source.getResourceElement().getName());
            source.addChangeListener(grp1, grp2);
        }
        log.debug("</fillGroup>");
    }
    
    
    //FIXME
    //TBD temporary solution
    ElementComponent source = null;
    ArrayList  grp1 = new ArrayList();
    ArrayList  grp2 = new ArrayList();
    
    private void collectListeners(JLabel elementText, ElementComponent elemComponent){
        log.debug("<collectListeners>");
        if(elemComponent.getResourceElement().getName().equals("asapReservTime")){
            log.debug("Got asap");
            source = elemComponent;
            return;
        }
        if(elemComponent.getResourceElement().getName().equals("expLength")){
            log.debug("Got expLength");
            grp1.add(elemComponent.getConcreteJavaComponent());
            grp1.add(elementText);
        } else if(elemComponent.getResourceElement().getName().equals("reservTimeFrom")){
            log.debug("Got reservTimeFrom");
            grp2.add(elemComponent.getConcreteJavaComponent());
            grp2.add(elementText);
        }else if(elemComponent.getResourceElement().getName().equals("reservTimeTo")){
            log.debug("Got reservTimeTo");
            grp2.add(elemComponent.getConcreteJavaComponent());
            grp2.add(elementText);
        }
        log.debug("<collectListeners>");
    }
}
