/*
 * ProxyFactory.java
 *
 * Created on May 9, 2005, 12:10 PM
 */

package pl.psnc.vlab.ssa.management;

import org.apache.axis.components.logger.LogFactory;
import org.apache.commons.logging.Log;
import pl.psnc.vlab.auth.AuthenticationRemote;
import pl.psnc.vlab.auth.AuthenticationRemoteImpl;
import pl.psnc.vlab.management.session.SessionFactory;
import pl.psnc.vlab.remote.session.VlabSessionRemote;
import pl.psnc.vlab.remote.session.VlabSessionRemoteImpl;
import pl.psnc.vlab.ssa.exception.SSAException;

/**
 * The abstract class responsible for authentication (acquiring user proxy,
 * validating proxy, generating, etc.)
 * @author <a href="mailto:osa@man.poznan.pl">Dominik Stoklosa (~osa~)</a>
 * @email  osa@man.poznan.pl
 */
public abstract class ProxyFactory {
    
    /** Instance of logger */
    private static Log log = LogFactory.getLog("pl.psnc.vlab.ssa.management.ProxyFactory");
    
    /** Holds value of the user proxy */
    private static byte[] userProxy;
    
    
    /**
     * Get user proxy for the given user
     * @param userId user identifier
     * @return
     */
    public static byte[] getUserProxy() throws SSAException {
        log.debug("<getUserProxy>");
        try{
            if(userProxy == null){
                int userId = SessionFactory.getSessionValue().getUserId();
                VlabSessionRemote sessionRemote = new VlabSessionRemoteImpl();
                userProxy = sessionRemote.getUserProxy(userId);
            }
            log.debug("</getUserProxy:"+new String(userProxy)+">");
            return userProxy;
        }catch(Exception e){
            log.debug("</getUserProxy:error>");
            throw new SSAException(e.getMessage(), e);
        }
    }
    
    
    
    /**
     * Validate user proxy
     * @return <code>true</code> when proxy is vlaid <code>false</code> otherwise
     */
    public static boolean isProxyValid() {
        log.debug("<isProxyValid>");
        try{
            byte proxy[] = getUserProxy();            
            AuthenticationRemote authRemote = new AuthenticationRemoteImpl();
            log.debug("</isProxyValid>");
            return authRemote.isProxyValid(proxy);
        }catch(Exception e){            
            log.debug("</isProxyValid:error-"+e.getMessage()+">");
            return false;
        }
    }
    
    
    /**
     * Validate user proxy
     * @return <code>true</code> when proxy is vlaid <code>false</code> otherwise
     * @param proxy proxy to be validated
     */
    public static boolean isProxyValid(byte proxy[]) {
        log.debug("<isProxyValid>");
        try{               
            AuthenticationRemote authRemote = new AuthenticationRemoteImpl();
            log.debug("</isProxyValid>");
            return authRemote.isProxyValid(proxy);
        }catch(Exception e){           
            e.printStackTrace();
            log.debug("</isProxyValid:error-"+e.getMessage()+">");
            return false;
        }
    }
    
    
}
