/*
 * VlabRemote.java
 *
 * Created on 4 marzec 2004, 15:10
 */

package pl.psnc.vlab.ssa.remote;

import javax.swing.Action;
import pl.psnc.vlab.ssa.exception.SSAException;
import pl.psnc.vlab.cds.ItemDocument;

/**
 * This it the interface to the VLab Comunnication Proxy module (which is
 * written in C). It allows sending messages to the VLab sytem.
 * @author  osa
 * @email osa@man.poznan.pl
 */
public interface SSARemote {
    
    /** Submit new scenario to the VLab system.
     * @param scenarioDetails scenario details object
     * @return
     */
    public void sendScenario(Action action, String jgxFile, String resources) throws  SSAException;
            
    
    /**
     * Get the list of supported Java types
     * @throws SSAException
     * @return XML file (CDS schema) with the Java types
     */    
    public String getJavaTypes() throws SSAException;
    
    
    /**
     * Get the list of the VLab resource types
     * @throws SSAException
     * @return XML file (TDS schema) with the resource types
     */    
    public String getResourceTypes() throws SSAException;
    
    
    /**
     * Get the list of the VLab resources for the given user
     * @throws SSAException
     * @return XML file (TDS schema) with the resource types
     */    
    public String getResources(int userId, int labId) throws SSAException;
    
    
    /**
     * Get the list of the VLab resource links
     * @throws SSAException
     * @return XML file (LDS schema) with the resource links
     */    
    public String getResourceLinks() throws SSAException;
    
    
    
    /**
     * Executes the generic query
     * @return the list of items or null in no itmes were found
     * @param tableName the name of the database table
     * @param attributes The list of attributes in the database 
     * table given by the tableName. Each attribute has to be terminated with the semicolon (;) 
     * @param condition the sql WHERE clause
     * @param orderBy the sql ORDER BY clause
     * @throws SSAException
     */    
    public ItemDocument.Item[] executeGenericQuery(String tableName, String attributes, 
    String condition, String orderBy) throws SSAException;
}
