/*
 * VlabRemoteImpl.java
 *
 * Created on 9 marzec 2004, 13:23
 */

package pl.psnc.vlab.ssa.remote;



import java.nio.ByteOrder;
import javax.swing.Action;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.ssa.exception.SSAException;
import pl.psnc.vlab.ssa.util.SSABytesConversion;
import pl.psnc.vlab.ssa.util.SSARemoteKeys;
import pl.psnc.vlab.cds.ItemDocument;
import pl.psnc.vlab.conf.ConfigFactory;
import pl.psnc.vlab.remote.Dgram;
import pl.psnc.vlab.remote.VlabdgramLocator;
import pl.psnc.vlab.remote.VlabdgramPortType;
import pl.psnc.vlab.remote.util.DatagramHelper;
import pl.psnc.vlab.remote.value.ShortMessageValue;
import pl.psnc.vlab.ssa.remote.scenario.SendScenarioTask;
import pl.psnc.vlab.xml.ResourceFactory;


/**
 * This it the implementation of the interface to the VLab Comunnication Proxy
 * module (which is written in C). It allows sending messages to the VLab sytem.
 * @author  osa
 * @email osa@man.poznan.pl
 */
public class SSARemoteImpl implements SSARemote {
    
    /** Instance of SendScenarioTask  */
    private SendScenarioTask sendScenarioTask = null;
    
    /** Instance of the BytesConversion object */
    private SSABytesConversion bytesConversion;
    
    /** Logger instance */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Creates a new instance of the Vlab Remote interface */
    public SSARemoteImpl(){
        bytesConversion = new SSABytesConversion();
    }
    
    
    /** Submit new scenario to the VLab system.
     * @param scenarioDetails scenario details object
     * @return response message from the comm proxy module
     */
    public void sendScenario(Action action, String jgxFile, String resources) throws pl.psnc.vlab.ssa.exception.SSAException {
        sendScenarioTask = new SendScenarioTask(action, jgxFile, resources);
    }
    
    /** Get the ScenarioSendTask object, which represends a thread which is used
     * to submit new dynamic scenario to the VLab systemf
     * @return
     */
    public SendScenarioTask getSendScecarioTask(){
        return sendScenarioTask;
    }
    
    
    
    
    
    
    
    /**
     * Get the list of supported Java types
     * @throws SSAException
     * @return XML file (CDS schema) with the Java types
     */
    public String getJavaTypes() throws SSAException{
        try{            
            log.debug("<getJavaTypes>");            
            VlabdgramPortType port = new VlabdgramLocator().getvlabdgram();
            Dgram datagramIn = new Dgram();
            datagramIn = DatagramHelper.setCommonFields(datagramIn);
            //set operation id
            datagramIn.setOperationId( SSARemoteKeys.MONITORING_GET_JAVA_TYPES);
            //set content size            
            datagramIn.setContentSize(0);
            Dgram datagramOut = port.callCommproxySyncApp(datagramIn);
            
            if(datagramOut.getOperationStatusId() == SSARemoteKeys.OPERATION_STATUS_ERROR){
                ByteOrder byteOrder = DatagramHelper.getByteOrder(datagramOut.getByteOrder());
                ShortMessageValue shortMessageV = bytesConversion.getShortMessage(datagramOut.getBinaryContent(), byteOrder);
                log.debug("Error during call. Details:"+shortMessageV.getMessage() );
                log.debug("</getJavaTypes>");
                throw new SSAException(shortMessageV.getMessage());
            }
            log.debug("</getJavaTypes>");
            return bytesConversion.getString(datagramOut.getBinaryContent());
        }catch(Exception e){
            throw new SSAException(e.getMessage(), e);
        }
    }
    
    
    
    
    /**
     * Get the list of the VLab resource types
     * @throws SSAException
     * @return XML file (TDS schema) with the resource types
     */
    public String getResourceTypes() throws SSAException{
        try{
            log.debug("<getResourceTypes>");
            VlabdgramPortType port = new VlabdgramLocator().getvlabdgram();
            Dgram datagramIn = new Dgram();
            datagramIn = DatagramHelper.setCommonFields(datagramIn);
            //set operation id
            datagramIn.setOperationId( SSARemoteKeys.MONITORING_GET_RESOURCE_TYPES);
            //set content size
            datagramIn.setContentSize(0);
            
            //datagramIn.setBinaryProxy("noProxy".getBytes());
            Dgram datagramOut = port.callCommproxySyncApp(datagramIn);
            
            if(datagramOut.getOperationStatusId() == SSARemoteKeys.OPERATION_STATUS_ERROR){
                ByteOrder byteOrder = DatagramHelper.getByteOrder(datagramOut.getByteOrder());
                ShortMessageValue shortMessageV = bytesConversion.getShortMessage(datagramOut.getBinaryContent(), byteOrder);
                log.debug("Error during call. Details:"+shortMessageV.getMessage() );
                log.debug("</getResourceTypes>");
                throw new SSAException(shortMessageV.getMessage());
            }
            log.debug("</getResourceTypes>");
            return bytesConversion.getString(datagramOut.getBinaryContent());
        }catch(Exception e){
            throw new SSAException(e.getMessage(), e);
        }
    }
    
    
    
    /**
     * Get the list of the VLab resources for the given user
     * @throws SSAException
     * @return XML file (TDS schema) with the resource types
     */
    public String getResources(int userId, int labId) throws SSAException{
        try{
            log.debug("<getResources: userId="+userId+"labId="+labId+">");
            VlabdgramPortType port = new VlabdgramLocator().getvlabdgram();
            Dgram datagramIn = new Dgram();
            datagramIn = DatagramHelper.setCommonFields(datagramIn);
            //set user id
            datagramIn.setUserId(userId);
            //set lab id
            datagramIn.setGroupId(labId);
            //set operation id
            datagramIn.setOperationId( SSARemoteKeys.MONITORING_GET_RESOURCES);
            //set content size
            datagramIn.setContentSize(0);
            Dgram datagramOut = port.callCommproxySyncApp(datagramIn);
            
            if(datagramOut.getOperationStatusId() == SSARemoteKeys.OPERATION_STATUS_ERROR){
                
                ByteOrder byteOrder = DatagramHelper.getByteOrder(datagramOut.getByteOrder());
                ShortMessageValue shortMessageV = bytesConversion.getShortMessage(datagramOut.getBinaryContent(), byteOrder);
                log.debug("Error during call. Details:"+shortMessageV.getMessage() );
                log.debug("</getResources>");
                throw new SSAException(shortMessageV.getMessage());
            }
            log.debug("</getResources>");
            return bytesConversion.getString(datagramOut.getBinaryContent());
        }catch(Exception e){
            throw new SSAException(e.getMessage(), e);
        }
    }
    
    
    
    /**
     * Get the list of the VLab resource links
     * @throws SSAException
     * @return XML file (LDS schema) with the resource links
     */
    public String getResourceLinks() throws SSAException{
        try{
            log.debug("<getResourceLinks>");
            VlabdgramPortType port = new VlabdgramLocator().getvlabdgram();
            Dgram datagramIn = new Dgram();
            datagramIn = DatagramHelper.setCommonFields(datagramIn);
            //set operation id
            datagramIn.setOperationId( SSARemoteKeys.MONITORING_GET_RESOURCE_LINKS);
            //set content size
            datagramIn.setContentSize(0);
            Dgram datagramOut = port.callCommproxySyncApp(datagramIn);
            
            if(datagramOut.getOperationStatusId() == SSARemoteKeys.OPERATION_STATUS_ERROR){
                ByteOrder byteOrder = DatagramHelper.getByteOrder(datagramOut.getByteOrder());
                ShortMessageValue shortMessageV = bytesConversion.getShortMessage(datagramOut.getBinaryContent(), byteOrder);
                log.debug("Error during call. Details:"+shortMessageV.getMessage() );
                log.debug("</getResourceLinks>");
                throw new SSAException(shortMessageV.getMessage());
            }
            log.debug("</getResourceLinks>");
            return bytesConversion.getString(datagramOut.getBinaryContent());
        }catch(Exception e){
            throw new SSAException(e.getMessage(), e);
        }
    }
    
    
    /**
     * Executes the generic query
     * @return the list of items or null in no itmes were found
     * @param tableName the name of the database table
     * @param attributes The list of attributes in the database 
     * table given by the tableName. Each attribute has to be terminated with the semicolon (;) 
     * @param condition the sql WHERE clause
     * @param orderBy the sql ORDER BY clause
     * @throws SSAException
     */    
    public ItemDocument.Item[] executeGenericQuery(String tableName, String attributes, 
    String condition, String orderBy) throws SSAException {
        try{
            log.debug("<executeGenericQuery: ["+tableName+" ,"+attributes+" ] >");
            VlabdgramPortType port = new VlabdgramLocator().getvlabdgram();
            Dgram datagramIn = new Dgram();
            datagramIn = DatagramHelper.setCommonFields(datagramIn);
            //set operation id
            datagramIn.setOperationId( SSARemoteKeys.MONITORING_GENERIC_QUERY);
            byte genericSelectQuery[] = bytesConversion.encodeGenericQuery(tableName,
            attributes, condition, orderBy);            
            datagramIn.setBinaryContent(genericSelectQuery);
            datagramIn.setContentSize(genericSelectQuery.length);            
            Dgram datagramOut = port.callCommproxySyncApp(datagramIn);
            
            if(datagramOut.getOperationStatusId() == SSARemoteKeys.OPERATION_STATUS_ERROR){
                ByteOrder byteOrder = DatagramHelper.getByteOrder(datagramOut.getByteOrder());
                ShortMessageValue shortMessageV = bytesConversion.getShortMessage(datagramOut.getBinaryContent(), byteOrder);
                log.debug("Error during call. Details:"+shortMessageV.getMessage() );
                log.debug("</executeGenericQuery:err>");
                throw new SSAException(shortMessageV.getMessage());
            }
            log.debug("</executeGenericQuery>");
            //decode the items 
            String modelXml = bytesConversion.getString(datagramOut.getBinaryContent());
            log.debug("ExternalModel-"+modelXml);
            return ResourceFactory.getExternalModelData(modelXml);
        }catch(Exception e){
            throw new SSAException(e.getMessage(), e);
        }
    }
}
