/*
 * VlabRemoteImpl.java
 *
 * Created on 9 marzec 2004, 13:23
 */

package pl.psnc.vlab.ssa.remote.scenario;

import java.nio.ByteOrder;
import java.rmi.RemoteException;
import javax.swing.Action;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.remote.Dgram;
import pl.psnc.vlab.remote.VlabdgramLocator;
import pl.psnc.vlab.remote.VlabdgramPortType;
import pl.psnc.vlab.remote.util.DatagramHelper;
import pl.psnc.vlab.remote.value.ShortMessageValue;
import pl.psnc.vlab.ssa.exception.SSAException;
import pl.psnc.vlab.ssa.exception.SSAExceptionCodes;
import pl.psnc.vlab.ssa.gui.MainFrame;
import pl.psnc.vlab.ssa.util.SSABytesConversion;
import pl.psnc.vlab.ssa.util.SSARemoteKeys;
import pl.psnc.vlab.util.swing.thread.LongTask;
import pl.psnc.vlab.util.swing.thread.SwingWorker;


/**
 * This it the implementation of the interface to the VLab Comunnication Proxy
 * module (which is written in C). It allows sending messages to the VLab sytem.
 * @author  osa
 * @email osa@man.poznan.pl
 */
public class SendScenarioTask implements LongTask {
    /** Logger instance */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Long tastk status */
    private boolean done = false;
    
    /** The JGraph  */
    private String jgxFile = null;
    /** The Resources description */
    private String resources = null;
    
    /** Stores action object which invoked this thread */
    private final Action action;
    
    /** Represents an exception */
    private SSAException dynExc = null;
    
    /** Result object produced by the executed task */
    private String result = null;
    
    /** Creates a new instance of SendScenarioTask, which automatically starts
     *  new thread which send new scenario to the VLab System
     * @param scenarioDetails scenario details object
     * @param in input datagram with scenario description
     */
    public SendScenarioTask(final Action action, String jgxFile, String resources) {
        this.jgxFile = jgxFile;
        this.resources = resources;
        this.action = action;
        final SwingWorker worker = new SwingWorker() {
            public Object construct() {
                return new ActualSendScenarioTask();
            }
            
            public void finished() {
                log.debug("About to finish");
                action.setEnabled(true);
                MainFrame.getInstance().update(getTask());
            }
        };
        worker.start();
    }
    
    
    
    public boolean isDone(){
        return this.done;
    }
    
    public Object getResult(){
        return this.result;
    }
    
    public Exception getException(){
        return this.dynExc;
    }
    
    public int getCurrent() {
        return 0;
    }
    
    public int getTaskLength() {
        return 0;
    }
    
    
    /** tbd
     * Get the current state of the executed task
     * @return the current state of the executed task (where <CODE>getTaskLength() is max size </CODE>
     */
    public int getProgress(){
        return 0;
    }
    
    private LongTask getTask() {
        return this;
    }
    
    public boolean exceptionThrown() {
        return !(dynExc == null);
    }
    
    /**
     * The actual long running task.  This runs in a SwingWorker thread.
     */
    class ActualSendScenarioTask {
        ActualSendScenarioTask() {
            try{
                log.debug("<sendScenario>");                                
                //tbd 
                log.debug("</sendScenario>");
                
           /* } catch(RemoteException cE){
                cE.printStackTrace();
                done = true;
                log.error("Error: "+cE.getMessage());
                log.debug("</sendScenario-Err1>");
                result = cE.toString();
                dynExc = new SSAException(cE.getMessage(), SSAExceptionCodes.CONNECTION_REFUSED, cE);
            */
            } catch(java.lang.Exception er) {
                er.printStackTrace();
                done = true;
                log.debug("</sendScenario-Err2>");
                result = er.getLocalizedMessage();
                dynExc = new SSAException(er.getLocalizedMessage(), SSAExceptionCodes.GENERAL_EXCEPTION, er);
            }            
        }
        
        
    }
    
    
    
}
