/*
 * ComponentHelper.java
 *
 * Created on 6 grudzie� 2004, 15:03
 */

package pl.psnc.vlab.ssa.util.component;

import java.lang.reflect.Constructor;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.rds.ElementDocument;

/**
 * The CDS Java Componet Helper class. Contains a set of util classes.
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class ComponentHelper {
    
    
    /** Logger instance */
    private static Log log = LogFactory.getLog("pl.psnc.vlab.ssa.util.ComponentHelper");
        
    
    /**
     * Creates a new instance of the java component specified by the component class.
     * It takes the property dialog and element objects as an construcotr parameters.
     *
     * @param componentClass component class name to be created
     * @param propertyDialog property dialog, where the component will be placed
     * @param element element document associated with this component
     * @return new CDS component or null if element couldn't be created
     */    
    public static Object getComponent(String componentClass,  ElementDocument.Element element){
        log.debug("<getComponent: "+componentClass+">");
        try{
            Class compClass = Class.forName(componentClass);
            Constructor compConstructor = compClass.getConstructor(new Class[] { pl.psnc.vlab.rds.ElementDocument.Element.class });
            //log.debug("Cons="+compConstructor.toString());
            
            Object o = compConstructor.newInstance(new Object[] { element});
            log.debug("</getComponent>");
            return o;
        }catch(Exception e){            
            log.error("Component : "+componentClass +" couldn't be created :"+e.getMessage());
            return null;
        }
    }
    
    /**
     * Creates an instance of the given class. THe class is instanced with 
     * the default constructor
     * @param className
     * @return
     */
    public static Object loadClass(String className){
        try{
            Class c = Class.forName(className);
            return c.newInstance();
        }catch(Exception e){
            return null;
        }
    }
    
}
