/*
 * PropertyFactory.java
 *
 * Created on 16 czerwiec 2004, 11:00
 */

package pl.psnc.vlab.ssa.util.component;

import javax.swing.JDialog;
import javax.swing.JPanel;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.exception.VlabException;
import pl.psnc.vlab.rds.LayoutDocument;
import pl.psnc.vlab.rds.ResourceDocument;
import pl.psnc.vlab.rds.TabDocument;
import pl.psnc.vlab.ssa.gui.layout.DefaultTabContentPane;
import pl.psnc.vlab.ssa.gui.layout.JFormTabContentPane;
import pl.psnc.vlab.ssa.gui.layout.PaneDecorator;

/**
 * The Property Factory is used for creating the property dialog for the
 * given resource.
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class ResourcePropertyFactory {
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Stores the resource details */
    private static ResourceDocument.Resource resource = null;
    
    /** Stores the value of the Resource property Dialog */
    private JDialog propertyDialog = null;
    
    /**
     * Creates a new instance of PropertyFactory
     * @param resource
     */
    public ResourcePropertyFactory(JDialog propertyDialog, ResourceDocument.Resource resource) {
        this.propertyDialog = propertyDialog;
        this.resource = resource;
    }
    
    
    /**
     * Get the tab list for the given resource
     * @param resource
     * @return
     */
    public TabDocument.Tab[] getTabList(ResourceDocument.Resource resource){
        log.debug("<getTabList>");
        ResourceDocument.Resource.Tabs tabs = resource.getTabs();
        log.debug("</getTabList>");
        return tabs.getTabArray();
    }
    
    
    /**
     * Get the Panel filled with the tab elements
     * @param tab
     * @return
     */
    public JPanel getTabPanel(TabDocument.Tab tab, int tabIndex) throws VlabException{
        log.debug("<getTabPanel>");                
        PaneDecorator tabPanel = null;
        LayoutDocument.Layout layout = tab.getLayout();     //get tab pane layout
        if(layout == null){
            log.debug("DefaultTabContentPane");
            //create new Tab panel
            tabPanel = new DefaultTabContentPane(propertyDialog, tab, tabIndex);                        
        } else {            //assumption to use JForms Layout Manager
            log.debug("JGoodiesContentPane");
            tabPanel = new JFormTabContentPane(propertyDialog, tab, layout, tabIndex);            
        }
        
        
        tabPanel.addElement(tab.getElementArray());
        tabPanel.addGroupElement(tab.getElementGroupArray());        
        log.debug("</getTabPanel>");
        return (JPanel) tabPanel;
    }
                
}
