/*
 * SSABundleFactory.java
 *
 * Created on March 16, 2005, 3:28 PM
 */

package pl.psnc.vlab.ssa.util.i18n;

import java.util.Locale;
import pl.psnc.vlab.ssa.management.SystemConfigurationFactory;
import pl.psnc.vlab.util.i18n.ResourceBundleManager;

/**
 * Factory class for the SSA properties (dynamicScenario.properties) retrieval
 * @author <a href="mailto:osa@man.poznan.pl">Dominik Stoklosa (~osa~)</a>
 * @email osa@man.poznan.pl
 */
public abstract class SSABundleFactory {
    
    /** The resource property file for the SSA */
    private static final String BUNDLE = "bundle/dynamicScenario";
    
    /** Stores the key with message sent whent the specified key value cannot be foundf */
    private static final String KEY_NOT_FOUND = "message.key.not.found";
    
    /** Instance of SSABundle */
    private static ResourceBundleManager ssaBundleInstance;
    
    /**
     * Get the value for the given key for coresponding resource bundle
     * @param key key for the desired string
     * @throws pl.psnc.vlab.exception.VlabException 
     * @return value of the given key
     */
    public static String getValue(String key){
        if(ssaBundleInstance == null){
            //get current locale
            Locale locale = SystemConfigurationFactory.getInstance().getCurrentLocale();
            ssaBundleInstance = new ResourceBundleManager(BUNDLE, locale, KEY_NOT_FOUND);
        }
        return ssaBundleInstance.getValue(key);
    }
}
