/*
 * ComponentFactory.java
 *
 * Created on 26 maj 2004, 15:58
 */

package pl.psnc.vlab.xml;

import java.net.URL;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.cds.CdsDocument;
import pl.psnc.vlab.cds.ComponentDocument;
import pl.psnc.vlab.ssa.util.Keys;

/**
 * The Component Factory class is used to get the list of VLab resources available for
 * the current user
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class ComponentFactory {
        
    /** Instance of logger */
    private static Log log = LogFactory.getLog("pl.psnc.vlab.xml.ComponentFactory");
    
    /** Stores the list of components */
    private static ComponentDocument.Component[] components = null;
    
    
    //initialize an array
    static {
        try{            
            CdsDocument cdsDoc = CdsDocument.Factory.parse(new URL(Keys.CDS_LIST));
            CdsDocument.Cds cds = cdsDoc.getCds();
            CdsDocument.Cds.Components comp = cds.getComponents();
            components = comp.getComponentArray();
        }catch(Exception e){            
            e.printStackTrace();
            log.error("No components:"+e.getMessage());
            components = null;
        }
    }
    
    
    /**
     * Get the CDS component form the available list by the component 
     * identifier
     * @param id component identifier
     * @return component or null if no component was found
     */    
    public static ComponentDocument.Component getComponentById(int id){
        if(components == null){
            return null;
        }
        
        int size = components.length;
        for(int i=0; i<size; i++){
            if(components[i].getId() == id){
                return components[i];
            }
        }
        return null;
    }
}
