/*
 * ComponentFactory.java
 *
 * Created on 26 maj 2004, 15:58
 */

package pl.psnc.vlab.xml;

import java.net.URL;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.conf.ConfigFactory;
import pl.psnc.vlab.dev.DeviceDocument;
import pl.psnc.vlab.dev.DevicesDocument;
import pl.psnc.vlab.exception.VlabException;
import pl.psnc.vlab.ssa.management.SSAConfiguration;

/**
 * The Wizard Scenario Factory is used to retrieve the list of predefined scenarios
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class DeviceFactory {
    
    /** Instance of logger*/
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Stores the devices document */
    private DevicesDocument devicesDocument;
    
    /** Stores DeviceFactory  */
    private static DeviceFactory instance;
    
    /** Stores the scenario categories */
    private DeviceDocument.Device[] deviceList;
    
    /** Creates new instance of DeviceFactory */
    private  DeviceFactory() throws VlabException{
        try{
            // get file location
            SSAConfiguration config = (SSAConfiguration) ConfigFactory.getConfigFactory();
            String devLocation = config.getDeviceLocation();
            URL url = new URL(devLocation);
            devicesDocument = DevicesDocument.Factory.parse(url);
        }catch(Exception e){
            e.printStackTrace();
            throw new VlabException("Failed to load device description. Details:"+e.getMessage(), e);
        }
    }
    
    
    /** Get instance of Device Factory */
    public  static DeviceFactory getInstance() throws VlabException{
        if(instance == null){
            instance = new DeviceFactory();
        }
        return instance;
    }
    
    
    
    /**
     * Get list of devices
     * @return array of devices
     */
    public DeviceDocument.Device[] getDeviceList(){
        log.debug("<getDeviceList>");
        try{
            if(deviceList == null){
                deviceList = devicesDocument.getDevices().getDeviceArray();
            }
            log.debug("</getDeviceList>");
            return deviceList;
        }catch(Exception e){            
            log.debug("</getDeviceList: null>");
            return null;
        }
    }
    
    
    /**
     * Get device by given id
     * @param id device id
     * @return 
     */
    public DeviceDocument.Device getDevice(int id){
        if(deviceList == null ){
            getDeviceList();
        }
        int size = deviceList !=null ? deviceList.length :0;
        for (int i = 0; i < size; i++) {
            if(deviceList[i].getId() == id){
                return deviceList[i];
            }
        }
        return null;
    }
    
    /**
     * Get category by name. The list of cateogires is taken from getCategoryList 
     * method
     * @param name name of the category to be found 
     * @return category or null
     
    public CategoryDocument.Category getCategory(String name){
        log.debug("<getCategory for name="+name+">");
        if(FormTools.isValidateFormStringNull(name)){
            log.debug("</getCategory: null>");
            return null;
        }
        CategoryDocument.Category[] list = getCategoryList();
        //tbd think it over; consider using binary search
        //impl. comparator as inner class
        int size = list != null ? list.length : 0;
        for(int i=0; i<size; i++){
            if(name.equals(list[i].getName())){
                log.debug("</getScenarioList>");
                return list[i];
            }
        }
        log.debug("</getScenarioList: not found>");
        return null;
    }
    */
}
