/*
 * ComponentFactory.java
 *
 * Created on 26 maj 2004, 15:58
 */

package pl.psnc.vlab.xml;

import java.net.URL;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.lds.LdsDocument;
import pl.psnc.vlab.lds.LdsDocument.Factory;
import pl.psnc.vlab.lds.LdsDocument.Lds;
import pl.psnc.vlab.lds.LdsDocument.Lds.Links;
import pl.psnc.vlab.lds.LinkDocument.Link;
import pl.psnc.vlab.ssa.util.Keys;


/**
 * The Link Factory class is used to retrieve the connection
 * definition between resource nodes defined by the vlabLDS Schema.
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class LinksFactory {
    
    /** Instance of logger */
    private static Log log = LogFactory.getLog("pl.psnc.vlab.xml.LinksFactory");
    
    /** Stores the list of available connections between resources */
    private static Link[] linkList = null;
    
    //initialize an array
    static {
        try{
            LdsDocument ldsDoc = Factory.parse(new URL(Keys.LINK_RULES_LIST));            
            Lds lds = ldsDoc.getLds();
            Links links = lds.getLinks();
            linkList =  links.getLinkArray();
        }catch(Exception e){
            e.printStackTrace();
            log.error("No linkList:"+e.getMessage());
            linkList = null;
        }
    }
    
    /**
     * Test the connection between the source and destination resource.
     * @param sourceId
     * @param destId
     * @return if the connection is defined true is returned
     */
    public static boolean canConnect(int sourceId, int destId){
        log.debug("<canConnect: source:"+sourceId +" dest:"+destId+">");
        int size = linkList != null ? linkList.length : 0;
        for(int i=0; i<size; i++){
            if(linkList[i].getSource().getResourceId() == sourceId &&
                    linkList[i].getTarget().getResourceId() == destId){
                return true;
            }
        }
        log.debug("</canConnect>");
        return false;
    }
}
