/*
 * ComponentFactory.java
 *
 * Created on 26 maj 2004, 15:58
 */

package pl.psnc.vlab.xml;
import java.net.URL;
import java.util.ArrayList;
import javax.swing.Action;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xmlbeans.XmlOptions;
import pl.psnc.vlab.graph.util.GraphHelper;
import pl.psnc.vlab.graph.vertex.type.VLabVertexType;
//schema
import pl.psnc.vlab.rds.RdsDocument;
import pl.psnc.vlab.rds.ResourceDocument;
import pl.psnc.vlab.cds.ItemDocument;
import pl.psnc.vlab.cds.ModelDataDocument;
import pl.psnc.vlab.ssa.util.Keys;

/**
 * The Component Factory class is used to get the list of Java components
 * defined by the vlabCDS Schema.
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class ResourceFactory {
    
    /** Instance of logger */
    private static Log log = LogFactory.getLog("pl.psnc.vlab.xml.ResourceFactory");
    
    /** Stores the list of available resources */
    private static ResourceDocument.Resource[] resourceList = null;
    
    
    /** Identyfier generator */
    private static int resourceId = 1;
    
    //initialize an array
    static {
        try{
            RdsDocument rdsDoc = RdsDocument.Factory.parse(new URL(Keys.RESOURCE_LIST));
            RdsDocument.Rds rds = rdsDoc.getRds();
            RdsDocument.Rds.Resources resources = rds.getResources();
            resourceList =  resources.getResourceArray();
        }catch(Exception e){
            e.printStackTrace();
            log.error("No resources:"+e.getMessage());
            resourceList = null;
        }
    }
    
    
    /** Gets the VLab resource by id
     * @param id
     * @return
     */
    public static ResourceDocument.Resource getResourceById(int id){
        int size = resourceList != null ? resourceList.length : 0;
        for(int i=0; i<size; i++){
            if(resourceList[i].getId() == id){
                //return resourceList[i];
                String res = resourceAsString(resourceList[i]);
                return getResource(res);
            }
        }
        return null;
    }
    
    
    /**
     * Get the resource from the xml source
     * @param resourceRaw the resource in the xml representation
     * @return
     */
    public static ResourceDocument.Resource getResource(String resourceRaw){
        if(resourceRaw == null){
            return null;
        }
        try{
            ResourceDocument resDoc = ResourceDocument.Factory.parse(resourceRaw);
            return resDoc.getResource();
        }catch(org.apache.xmlbeans.XmlException e){
            e.printStackTrace();
            return null;
        }
    }
    
    /**
     * Converts the resource object into a string buffer
     * @param resource the resource object
     * @return
     */
    public static String resourceAsString(ResourceDocument.Resource resource){
        if(resource == null){
            return null;
        }
        
        XmlOptions options = new XmlOptions();
        options.setSaveOuter();
        options.setDocumentSourceName("resource");
        String xml = resource.xmlText(options);
        return xml;
    }
    
    /** Gets the list of available resources
     * @return
     */
    public static ArrayList getResourceList(VLabVertexType resourceType){
        log.debug("<getResourceList: type="+resourceType.getTypeName()+">" );
        if(resourceList == null || resourceList.length == 0){
            log.debug("<getResourceList: NULL>");
            return null;
        }
        int size = resourceList.length;
        ArrayList list = new ArrayList(size);
        int parentTypeId = GraphHelper.getParentTypeId(resourceType.getTypeId());
        log.debug("Parent type-"+parentTypeId);
        for(int i=0; i<size; i++){
            int resParentTypeId = GraphHelper.getParentTypeId(resourceList[i].getTypeId());
            log.debug("resParentTypeId="+resParentTypeId);
            if(parentTypeId == resParentTypeId){
                list.add(resourceList[i]);
            }
            
        }
        log.debug("</getResourceList>");
        if(list.size() == 0){
            return null;
        } else {
            return list;
        }
    }
    
    /**
     * This is the unique resource identyfier generator.
     * @return
     */
    public static synchronized int getNextResourceId(){
        resourceId++;
        return resourceId;
    }
    
    /**
     * Set the action state. If there aro no resources for the
     * current action , the action will be disabled
     * @param list
     * @param action
     */
    private void setAction(ResourceDocument.Resource list[], Action action){
        int size = list != null ? list.length : 0;
        if(size <= 0){
            //disable the action
            action.setEnabled(false);
        }
    }
    
    
    /**
     * Get the list of external data model items. The external data model
     * is provide as an raw XML
     * @param modelData xml encoded data model
     * @return list of items, which represetns a model data
     */
    public static ItemDocument.Item[] getExternalModelData(String modelData){
        if(modelData == null){
            return null;
        }
        try{
            ModelDataDocument modelDoc = ModelDataDocument.Factory.parse(modelData);
            return modelDoc.getModelData().getItemArray();
        }catch(org.apache.xmlbeans.XmlException e){
            e.printStackTrace();
            return null;
        }
    }
}
