/*
 * ComponentFactory.java
 *
 * Created on 26 maj 2004, 15:58
 */

package pl.psnc.vlab.xml;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Date;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xmlbeans.XmlOptions;
import pl.psnc.vlab.exception.VlabException;
import pl.psnc.vlab.management.session.SessionFactory;
import pl.psnc.vlab.rds.RdsDocument;
import pl.psnc.vlab.rds.ResourceDocument;
import pl.psnc.vlab.rds.ScenarioDocument;
import pl.psnc.vlab.ssa.gui.MainFrame;
import pl.psnc.vlab.ssa.util.Keys;
import pl.psnc.vlab.ssa.management.SystemConfigurationFactory;
import pl.psnc.vlab.ssa.management.SystemConfigurationValue;
import pl.psnc.vlab.util.DateTools;
import pl.psnc.vlab.util.FileTools;
import pl.psnc.vlab.values.management.SessionValue;



/**
 * The Dynamic Scenario Builder class is used to build the model of the
 * scenario created by the user
 *
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class ScenarioBuilder {
    
    /** Instance of logger */
    private static Log log = LogFactory.getLog("pl.psnc.vlab.xml.ScenarioBuilder");
    
    /** Holds the Scenario document instance */
    private static RdsDocument rdsDocument = null;
    
    
    
    /**
     * Create a new instance of the Rds Document. If document
     * given as an argument is != null then it becomes active
     * Rds Document.
     * @param rdsDocument
     */
    public static void createInstace(RdsDocument rdsDoc){
        if(rdsDoc == null){
            log.debug("<createInstace>");
            rdsDocument = RdsDocument.Factory.newInstance();
            rdsDocument.addNewRds().addNewResources();
            rdsDocument.getRds().addNewScenario();
            //set dates
            ScenarioDocument.Scenario scen = rdsDocument.getRds().getScenario();
            SystemConfigurationValue sysConfVO = SystemConfigurationFactory.getInstance();
            DateTools dateTools = new DateTools(sysConfVO.getCurrentLocale(), Keys.DATE_FORMAT, Keys.TIME_FORMAT);
            Date curDate = new Date(); //MainFrame.getInstance().getStatusBar().getCurrentDate();
            String date = dateTools.getDateAsString(curDate);
            String time = dateTools.getTimeAsString(curDate);
            scen.setCreationDate(date+" "+time);
            scen.setModificationDate(date+" "+time);            
            rdsDocument.getRds().setScenario(scen);
        } else {
            rdsDocument = rdsDoc;
        }
        log.debug("</createInstace>");
    }
    
    /**
     * Set the ScenarioDocument in the RdsDocument. The ScenarioDocument contains
     * the genral information about the given scenario i.e name, description
     * @param scenario
     */
    public static void setScenarioDescription(ScenarioDocument.Scenario scenario){
        if(rdsDocument == null){
            createInstace(null);
        }
        rdsDocument.getRds().setScenario(scenario);
    }
    
    
    /**
     * get the ScenarioDocument from the RdsDocument. The ScenarioDocument contains
     * the genral information about the given scenario i.e name, description
     * @param scenario
     */
    public static ScenarioDocument.Scenario getScenarioDescription(){
        if(rdsDocument != null){
            return rdsDocument.getRds().getScenario();
        }
        return null;
    }
    
    
    /**
     * Insert an array of resources into the scenario document
     * @param resourceList
     */
    public static void insertResources(ArrayList resourceList){
        ResourceDocument.Resource[] resourceArray = (ResourceDocument.Resource[]) resourceList.toArray(new ResourceDocument.Resource[0]);
        rdsDocument.getRds().getResources().setResourceArray(resourceArray);
    }
    
    
    public static void saveScenarioDetails(String file) throws IOException, FileNotFoundException{
        XmlOptions options = new XmlOptions();
        options.setCharacterEncoding("UTF-8");
        options.setSaveOuter();
        
        String scenario = rdsDocument.getRds().getScenario().xmlText(options);
        FileOutputStream writer = new FileOutputStream(new File(file));
        writer.write(scenario.getBytes("UTF-8"));
        writer.close();
    }
    
    public static void loadScenarioDetails(URL file) throws VlabException {
        try{
            log.debug("<loadScenarioDetails>");
            file = new URL(FileTools.getBase(file)+".info");
            log.debug("File="+file);
            XmlOptions xmlOptions = new XmlOptions();
            xmlOptions.setCharacterEncoding("UTF-8");            
            ScenarioDocument.Scenario scenario = ScenarioDocument.Factory.parse(file, xmlOptions).getScenario();            
            setScenarioDescription(scenario);
            MainFrame.getInstance().updateTitle(scenario.getName());
            log.debug("</loadScenarioDetails>");
        }catch(Exception e){
            log.debug("</loadScenarioDetails:err>");
            throw new VlabException("Error while loading scenario details.Details:"+e.getMessage(), e);
        }
    }
    
    
    public static String toStirng(){
        /*XmlOptions xmlOptions = new XmlOptions();
        xmlOptions.setSaveNamespacesFirst();
        Hashtable t = new Hashtable();
        t.put("http://vlab.psnc.pl/rds", "");
        xmlOptions.setSaveSuggestedPrefixes(t);
        return rdsDocument.xmlText(xmlOptions);*/
        return rdsDocument.toString();
    }
}
