/*
 * ComponentFactory.java
 *
 * Created on 26 maj 2004, 15:58
 */

package pl.psnc.vlab.xml;

import java.net.URL;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.ssa.util.Keys;
import pl.psnc.vlab.tds.TdsDocument;
import pl.psnc.vlab.tds.TypeDocument;


/**
 * The Type Factory is used to get the list of VLab resource's types. The type
 * component describes the resource type name, subtype id, subtype name and icons
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class TypeFactory {
    
    /** Instance of logger*/
    private static Log log = LogFactory.getLog("pl.psnc.vlab.xml.TypeFactory");
    
    /** Stores the list of resource types */
    private static TypeDocument.Type[] types = null;
    
    //initialize an array
    static {
        try{
            TdsDocument tdsDoc = TdsDocument.Factory.parse(new URL(Keys.TYPES_LIST));            
            TdsDocument.Tds tds = tdsDoc.getTds();
            TdsDocument.Tds.Types tdsTypes = tds.getTypes();
            types = tdsTypes.getTypeArray();
        }catch(Exception e){
            e.printStackTrace();
            log.error("No types:"+e.getMessage());
            types = null;
        }
    }
    
    
    /**
     * Get the resource type by the given id
     * @param id
     * @return
     */
    public static TypeDocument.Type getResourceType(int id){        
        int size = types != null ? types.length : 0;
        for(int i=0; i<size; i++){
            if(types[i].getId() == id){                
                return types[i];                
            }
        }        
        return null;
    }
}
