/*
 * ComponentFactory.java
 *
 * Created on 26 maj 2004, 15:58
 */

package pl.psnc.vlab.xml;

import java.io.File;
import java.io.InputStream;
import java.net.URI;
import java.net.URL;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.conf.ConfigFactory;
import pl.psnc.vlab.exception.VlabException;
import pl.psnc.vlab.psc.CategoriesDocument;
import pl.psnc.vlab.psc.CategoryDocument;
import pl.psnc.vlab.psc.ScenarioDocument;
import pl.psnc.vlab.ssa.management.SSAConfiguration;
import pl.psnc.vlab.ssa.remote.SSARemote;
import pl.psnc.vlab.ssa.remote.SSARemoteImpl;
import pl.psnc.vlab.tds.TdsDocument;
import pl.psnc.vlab.tds.TypeDocument;
import pl.psnc.vlab.util.FormTools;


/**
 * The Wizard Scenario Factory is used to retrieve the list of predefined scenarios
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class WizardScenarioFactory {
    
    /** Instance of logger*/
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Stores the category document */
    private CategoriesDocument catDocument;
    
    /** Stores WizardScenarioFactory  */
    private static WizardScenarioFactory instance;
    
    /** Stores the scenario categories */
    private CategoryDocument.Category[] categoryList;
    
    /** Creates new instance of WizardScenarioFactory */
    private  WizardScenarioFactory() throws VlabException{
        try{
            // get file location
            SSAConfiguration config = (SSAConfiguration) ConfigFactory.getConfigFactory();
            String pscLocation = null;//config.getPredefinedScenLocation();
            URL url = new URL(pscLocation);
            catDocument = CategoriesDocument.Factory.parse(url);
        }catch(Exception e){
            e.printStackTrace();
            throw new VlabException("Error rertrieving list of predefined scenarios", e);
        }
    }
    
    
    /** Get instance of Wizard Scenario Factory */
    public  static WizardScenarioFactory getInstance() throws VlabException{
        if(instance == null){
            instance = new WizardScenarioFactory();
        }
        return instance;
    }
    
    
    
    /**
     * Get list of scenario categories
     * @return array of scenario categories
     */
    public CategoryDocument.Category[] getCategoryList(){
        log.debug("<getCategoryList>");
        try{
            if(categoryList == null){
                categoryList = catDocument.getCategories().getCategoryArray();
            }
            log.debug("</getCategoryList>");
            return categoryList;
        }catch(Exception e){
            e.printStackTrace();
            log.debug("</getCategoryList:null>");
            return null;
        }
    }
    
    
    /**
     * Get category by name. The list of cateogires is taken from getCategoryList 
     * method
     * @param name name of the category to be found 
     * @return category or null
     */
    public CategoryDocument.Category getCategory(String name){
        log.debug("<getCategory for name="+name+">");
        if(FormTools.isValidateFormStringNull(name)){
            log.debug("</getCategory: null>");
            return null;
        }
        CategoryDocument.Category[] list = getCategoryList();
        //tbd think it over; consider using binary search
        //impl. comparator as inner class
        int size = list != null ? list.length : 0;
        for(int i=0; i<size; i++){
            if(name.equals(list[i].getName())){
                log.debug("</getScenarioList>");
                return list[i];
            }
        }
        log.debug("</getScenarioList: not found>");
        return null;
    }
    
}
